@@ -39,8 +39,7 @@ func (Clear) Execute(aerc *widgets.Aerc, args []string) error {
 	}
 
 	for _, opt := range opts {
-		switch opt.Option {
-		case 's':
+		if opt.Option == 's' {
 			clearSelected = true
 		}
 	}
@@ -51,8 +50,6 @@ func (Clear) Execute(aerc *widgets.Aerc, args []string) error {
 
 	if clearSelected {
 		defer store.Select(0)
-	} else {
-		store.SetReselect(store.Selected())
 	}
 	store.ApplyClear()
 	acct.SetStatus(statusline.SearchFilterClear())
