@@ -2,9 +2,17 @@ package account
 
 import (
 	"errors"
+	"fmt"
+	"io"
+	gomail "net/mail"
 	"regexp"
 	"strings"
 
+	"github.com/emersion/go-message/mail"
+
+	"git.sr.ht/~rjarry/aerc/config"
+	"git.sr.ht/~rjarry/aerc/lib/ui"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/widgets"
 	"git.sr.ht/~sircmpwn/getopt"
@@ -30,13 +38,24 @@ func (Compose) Execute(aerc *widgets.Aerc, args []string) error {
 		return err
 	}
 	acct := aerc.SelectedAccount()
+	if acct == nil {
+		return errors.New("No account selected")
+	}
 	if template == "" {
-		template = aerc.Config().Templates.NewMessage
+		template = config.Templates.NewMessage
 	}
 
+	msg, err := gomail.ReadMessage(strings.NewReader(body))
+	if errors.Is(err, io.EOF) { // completely empty
+		msg = &gomail.Message{Body: strings.NewReader("")}
+	} else if err != nil {
+		return fmt.Errorf("mail.ReadMessage: %w", err)
+	}
+	headers := mail.HeaderFromMap(msg.Header)
+
 	composer, err := widgets.NewComposer(aerc, acct,
-		aerc.Config(), acct.AccountConfig(), acct.Worker(),
-		template, nil, models.OriginalMail{})
+		acct.AccountConfig(), acct.Worker(),
+		template, &headers, models.OriginalMail{})
 	if err != nil {
 		return err
 	}
@@ -47,9 +66,13 @@ func (Compose) Execute(aerc *widgets.Aerc, args []string) error {
 		} else {
 			tab.Name = subject
 		}
-		tab.Content.Invalidate()
+		ui.Invalidate()
 	})
-	go composer.AppendContents(strings.NewReader(body))
+	go func() {
+		defer log.PanicHandler()
+
+		composer.AppendContents(msg.Body)
+	}()
 	return nil
 }
 
