@@ -3,12 +3,13 @@ package account
 import (
 	"bytes"
 	"errors"
-	"io/ioutil"
+	"io"
 	"os"
 	"path/filepath"
 
 	"git.sr.ht/~rjarry/aerc/commands"
-	"git.sr.ht/~rjarry/aerc/logging"
+	"git.sr.ht/~rjarry/aerc/lib/ui"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/widgets"
 	"git.sr.ht/~sircmpwn/getopt"
@@ -36,9 +37,10 @@ func (Recover) Complete(aerc *widgets.Aerc, args []string) []string {
 	if len(args) == 0 {
 		return files
 	}
-	if args[0] == "-" {
+	switch args[0] {
+	case "-":
 		return []string{"-f"}
-	} else if args[0] == "-f" {
+	case "-f":
 		if len(args) == 1 {
 			for i, file := range files {
 				files[i] = args[0] + " " + file
@@ -49,7 +51,7 @@ func (Recover) Complete(aerc *widgets.Aerc, args []string) []string {
 			return commands.FilterList(files, args[1], args[0]+" ",
 				aerc.SelectedAccountUiConfig().FuzzyComplete)
 		}
-	} else {
+	default:
 		// only accepts one file to recover
 		return commands.FilterList(files, args[0], "", aerc.SelectedAccountUiConfig().FuzzyComplete)
 	}
@@ -68,8 +70,7 @@ func (Recover) Execute(aerc *widgets.Aerc, args []string) error {
 		return err
 	}
 	for _, opt := range opts {
-		switch opt.Option {
-		case 'f':
+		if opt.Option == 'f' {
 			force = true
 		}
 	}
@@ -89,7 +90,7 @@ func (Recover) Execute(aerc *widgets.Aerc, args []string) error {
 			return nil, err
 		}
 		defer recoverFile.Close()
-		data, err := ioutil.ReadAll(recoverFile)
+		data, err := io.ReadAll(recoverFile)
 		if err != nil {
 			return nil, err
 		}
@@ -101,7 +102,7 @@ func (Recover) Execute(aerc *widgets.Aerc, args []string) error {
 	}
 
 	composer, err := widgets.NewComposer(aerc, acct,
-		aerc.Config(), acct.AccountConfig(), acct.Worker(),
+		acct.AccountConfig(), acct.Worker(),
 		"", nil, models.OriginalMail{})
 	if err != nil {
 		return err
@@ -110,10 +111,10 @@ func (Recover) Execute(aerc *widgets.Aerc, args []string) error {
 	tab := aerc.NewTab(composer, "Recovered")
 	composer.OnHeaderChange("Subject", func(subject string) {
 		tab.Name = subject
-		tab.Content.Invalidate()
+		ui.Invalidate()
 	})
 	go func() {
-		defer logging.PanicHandler()
+		defer log.PanicHandler()
 
 		composer.AppendContents(bytes.NewReader(data))
 	}()
