@@ -5,7 +5,8 @@ import (
 	"strings"
 
 	"git.sr.ht/~rjarry/aerc/lib/statusline"
-	"git.sr.ht/~rjarry/aerc/logging"
+	"git.sr.ht/~rjarry/aerc/lib/ui"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/widgets"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
@@ -43,7 +44,7 @@ func (SearchFilter) Execute(aerc *widgets.Aerc, args []string) error {
 		cb := func(msg types.WorkerMessage) {
 			if _, ok := msg.(*types.Done); ok {
 				acct.SetStatus(statusline.FilterResult(strings.Join(args, " ")))
-				logging.Infof("Filter results: %v", store.Uids())
+				log.Tracef("Filter results: %v", store.Uids())
 			}
 		}
 		store.Sort(store.GetCurrentSortCriteria(), cb)
@@ -51,10 +52,10 @@ func (SearchFilter) Execute(aerc *widgets.Aerc, args []string) error {
 		acct.SetStatus(statusline.Search("Searching..."))
 		cb := func(uids []uint32) {
 			acct.SetStatus(statusline.Search(strings.Join(args, " ")))
-			logging.Infof("Search results: %v", uids)
+			log.Tracef("Search results: %v", uids)
 			store.ApplySearch(uids)
 			// TODO: Remove when stores have multiple OnUpdate handlers
-			acct.Messages().Invalidate()
+			ui.Invalidate()
 		}
 		store.Search(args, cb)
 	}
