@@ -4,9 +4,7 @@ import (
 	"git.sr.ht/~rjarry/aerc/commands"
 )
 
-var (
-	ComposeCommands *commands.Commands
-)
+var ComposeCommands *commands.Commands
 
 func register(cmd commands.Command) {
 	if ComposeCommands == nil {
