@@ -1,6 +1,7 @@
 package compose
 
 import (
+	"errors"
 	"fmt"
 	"strings"
 
@@ -11,17 +12,15 @@ import (
 
 type Header struct{}
 
-var (
-	headers = []string{
-		"From",
-		"To",
-		"Cc",
-		"Bcc",
-		"Subject",
-		"Comments",
-		"Keywords",
-	}
-)
+var headers = []string{
+	"From",
+	"To",
+	"Cc",
+	"Bcc",
+	"Subject",
+	"Comments",
+	"Keywords",
+}
 
 func init() {
 	register(Header{})
@@ -32,7 +31,7 @@ func (Header) Aliases() []string {
 }
 
 func (Header) Complete(aerc *widgets.Aerc, args []string) []string {
-	return commands.CompletionFromList(headers, args)
+	return commands.CompletionFromList(aerc, headers, args)
 }
 
 func (Header) Execute(aerc *widgets.Aerc, args []string) error {
@@ -44,17 +43,21 @@ func (Header) Execute(aerc *widgets.Aerc, args []string) error {
 	if err != nil {
 		return err
 	}
-	var (
-		force bool = false
-	)
+
+	if len(args) < optind+1 {
+		return errors.New("command parsing failed")
+	}
+
+	var force bool = false
 	for _, opt := range opts {
-		switch opt.Option {
-		case 'f':
+		if opt.Option == 'f' {
 			force = true
 		}
 	}
 
-	composer, _ := aerc.SelectedTab().(*widgets.Composer)
+	composer, _ := aerc.SelectedTabContent().(*widgets.Composer)
+
+	args[optind] = strings.TrimRight(args[optind], ":")
 
 	value := strings.Join(args[optind+1:], " ")
 
