@@ -1,8 +1,6 @@
 package commands
 
-var (
-	GlobalCommands *Commands
-)
+var GlobalCommands *Commands
 
 func register(cmd Command) {
 	if GlobalCommands == nil {
