@@ -10,6 +10,8 @@ type Help struct{}
 
 var pages = []string{
 	"aerc",
+	"accounts",
+	"binds",
 	"config",
 	"imap",
 	"notmuch",
@@ -50,7 +52,6 @@ func (Help) Execute(aerc *widgets.Aerc, args []string) error {
 				aerc.HumanReadableBindings(),
 				aerc.SelectedAccountUiConfig(),
 				func(_ string) {
-					helpClose(aerc)
 					aerc.CloseDialog()
 				},
 			),
@@ -61,9 +62,3 @@ func (Help) Execute(aerc *widgets.Aerc, args []string) error {
 
 	return TermCore(aerc, []string{"term", "man", page})
 }
-
-func helpClose(aerc *widgets.Aerc) {
-	if content, ok := aerc.SelectedTabContent().(*widgets.MessageViewer); ok {
-		content.UpdateScreen()
-	}
-}
