@@ -34,12 +34,9 @@ func (Copy) Execute(aerc *widgets.Aerc, args []string) error {
 	if err != nil {
 		return err
 	}
-	var (
-		createParents bool
-	)
+	var createParents bool
 	for _, opt := range opts {
-		switch opt.Option {
-		case 'p':
+		if opt.Option == 'p' {
 			createParents = true
 		}
 	}
@@ -54,12 +51,12 @@ func (Copy) Execute(aerc *widgets.Aerc, args []string) error {
 	}
 	store.Copy(uids, strings.Join(args[optind:], " "),
 		createParents, func(
-			msg types.WorkerMessage) {
-
+			msg types.WorkerMessage,
+		) {
 			switch msg := msg.(type) {
 			case *types.Done:
 				aerc.PushStatus("Messages copied.", 10*time.Second)
-				store.ClearVisualMark()
+				store.Marker().ClearVisualMark()
 			case *types.Error:
 				aerc.PushError(msg.Error.Error())
 			}
