@@ -12,9 +12,11 @@ import (
 	"strings"
 	"sync"
 
+	"git.sr.ht/~rjarry/aerc/config"
 	"git.sr.ht/~rjarry/aerc/lib"
 	"git.sr.ht/~rjarry/aerc/lib/format"
-	"git.sr.ht/~rjarry/aerc/logging"
+	"git.sr.ht/~rjarry/aerc/lib/ui"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/widgets"
 	"git.sr.ht/~rjarry/aerc/worker/types"
@@ -73,7 +75,7 @@ func (forward) Execute(aerc *widgets.Aerc, args []string) error {
 	if err != nil {
 		return err
 	}
-	logging.Infof("Forwarding email %s", msg.Envelope.MessageId)
+	log.Debugf("Forwarding email <%s>", msg.Envelope.MessageId)
 
 	h := &mail.Header{}
 	subject := "Fwd: " + msg.Envelope.Subject
@@ -98,7 +100,7 @@ func (forward) Execute(aerc *widgets.Aerc, args []string) error {
 	}
 
 	addTab := func() (*widgets.Composer, error) {
-		composer, err := widgets.NewComposer(aerc, acct, aerc.Config(),
+		composer, err := widgets.NewComposer(aerc, acct,
 			acct.AccountConfig(), acct.Worker(), template, h, original)
 		if err != nil {
 			aerc.PushError("Error: " + err.Error())
@@ -117,7 +119,7 @@ func (forward) Execute(aerc *widgets.Aerc, args []string) error {
 			} else {
 				tab.Name = subject
 			}
-			tab.Content.Invalidate()
+			ui.Invalidate()
 		})
 		return composer, nil
 	}
@@ -132,10 +134,10 @@ func (forward) Execute(aerc *widgets.Aerc, args []string) error {
 		store.FetchFull([]uint32{msg.Uid}, func(fm *types.FullMessage) {
 			tmpFile, err := os.Create(tmpFileName)
 			if err != nil {
-				logging.Warnf("failed to create temporary attachment: %v", err)
+				log.Warnf("failed to create temporary attachment: %v", err)
 				_, err = addTab()
 				if err != nil {
-					logging.Warnf("failed to add tab: %v", err)
+					log.Warnf("failed to add tab: %v", err)
 				}
 				return
 			}
@@ -143,7 +145,7 @@ func (forward) Execute(aerc *widgets.Aerc, args []string) error {
 			defer tmpFile.Close()
 			_, err = io.Copy(tmpFile, fm.Content.Reader)
 			if err != nil {
-				logging.Warnf("failed to write to tmpfile: %w", err)
+				log.Warnf("failed to write to tmpfile: %v", err)
 				return
 			}
 			composer, err := addTab()
@@ -157,7 +159,7 @@ func (forward) Execute(aerc *widgets.Aerc, args []string) error {
 		})
 	} else {
 		if template == "" {
-			template = aerc.Config().Templates.Forwards
+			template = config.Templates.Forwards
 		}
 
 		part := lib.FindPlaintext(msg.BodyStructure, nil)
@@ -193,18 +195,23 @@ func (forward) Execute(aerc *widgets.Aerc, args []string) error {
 					}
 					bs, err := msg.BodyStructure.PartAtIndex(p)
 					if err != nil {
-						logging.Errorf("cannot get PartAtIndex %v: %v", p, err)
+						log.Errorf("cannot get PartAtIndex %v: %v", p, err)
 						continue
 					}
 					store.FetchBodyPart(msg.Uid, p, func(reader io.Reader) {
-						mime := fmt.Sprintf("%s/%s", bs.MIMEType, bs.MIMESubType)
-						name, ok := bs.Params["name"]
+						mime := bs.FullMIMEType()
+						params := lib.SetUtf8Charset(bs.Params)
+						name, ok := params["name"]
 						if !ok {
 							name = fmt.Sprintf("%s_%s_%d", bs.MIMEType, bs.MIMESubType, rand.Uint64())
 						}
 						mu.Lock()
-						composer.AddPartAttachment(name, mime, bs.Params, reader)
+						err := composer.AddPartAttachment(name, mime, params, reader)
 						mu.Unlock()
+						if err != nil {
+							log.Errorf(err.Error())
+							aerc.PushError(err.Error())
+						}
 					})
 				}
 			}
