@@ -5,9 +5,12 @@ import (
 	"fmt"
 	"io"
 
+	"git.sr.ht/~rjarry/aerc/config"
 	"git.sr.ht/~rjarry/aerc/lib"
 	"git.sr.ht/~rjarry/aerc/lib/calendar"
 	"git.sr.ht/~rjarry/aerc/lib/format"
+	"git.sr.ht/~rjarry/aerc/lib/ui"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/widgets"
 	"github.com/emersion/go-message/mail"
@@ -28,7 +31,6 @@ func (invite) Complete(aerc *widgets.Aerc, args []string) []string {
 }
 
 func (invite) Execute(aerc *widgets.Aerc, args []string) error {
-
 	acct := aerc.SelectedAccount()
 	if acct == nil {
 		return errors.New("no account selected")
@@ -47,7 +49,7 @@ func (invite) Execute(aerc *widgets.Aerc, args []string) error {
 		return fmt.Errorf("no invitation found (missing text/calendar)")
 	}
 
-	subject := trimLocalizedRe(msg.Envelope.Subject)
+	subject := trimLocalizedRe(msg.Envelope.Subject, acct.AccountConfig().LocalizedRe)
 	switch args[0] {
 	case "accept":
 		subject = "Accepted: " + subject
@@ -90,9 +92,7 @@ func (invite) Execute(aerc *widgets.Aerc, args []string) error {
 		}
 	}
 
-	var (
-		to []*mail.Address
-	)
+	var to []*mail.Address
 
 	if len(msg.Envelope.ReplyTo) != 0 {
 		to = msg.Envelope.ReplyTo
@@ -100,7 +100,7 @@ func (invite) Execute(aerc *widgets.Aerc, args []string) error {
 		to = msg.Envelope.From
 	}
 
-	if !aerc.Config().Compose.ReplyToSelf {
+	if !config.Compose.ReplyToSelf {
 		for i, v := range to {
 			if v.Address == from.Address {
 				to = append(to[:i], to[i+1:]...)
@@ -148,7 +148,7 @@ func (invite) Execute(aerc *widgets.Aerc, args []string) error {
 	}
 
 	addTab := func(cr *calendar.Reply) error {
-		composer, err := widgets.NewComposer(aerc, acct, aerc.Config(),
+		composer, err := widgets.NewComposer(aerc, acct,
 			acct.AccountConfig(), acct.Worker(), "", h, original)
 		if err != nil {
 			aerc.PushError("Error: " + err.Error())
@@ -156,7 +156,10 @@ func (invite) Execute(aerc *widgets.Aerc, args []string) error {
 		}
 
 		composer.SetContents(cr.PlainText)
-		composer.AppendPart(cr.MimeType, cr.Params, cr.CalendarText)
+		err = composer.AppendPart(cr.MimeType, cr.Params, cr.CalendarText)
+		if err != nil {
+			return fmt.Errorf("failed to write invitation: %w", err)
+		}
 		composer.FocusTerminal()
 
 		tab := aerc.NewTab(composer, subject)
@@ -166,7 +169,7 @@ func (invite) Execute(aerc *widgets.Aerc, args []string) error {
 			} else {
 				tab.Name = subject
 			}
-			tab.Content.Invalidate()
+			ui.Invalidate()
 		})
 
 		composer.OnClose(func(c *widgets.Composer) {
@@ -183,7 +186,10 @@ func (invite) Execute(aerc *widgets.Aerc, args []string) error {
 			aerc.PushError(err.Error())
 			return
 		} else {
-			addTab(cr)
+			err := addTab(cr)
+			if err != nil {
+				log.Warnf("failed to add tab: %v", err)
+			}
 		}
 	})
 	return nil
