@@ -13,6 +13,8 @@ import (
 	"github.com/pkg/errors"
 
 	"git.sr.ht/~rjarry/aerc/lib"
+	"git.sr.ht/~rjarry/aerc/lib/ui"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/widgets"
 	"git.sr.ht/~rjarry/aerc/worker/types"
@@ -41,8 +43,7 @@ func (Recall) Execute(aerc *widgets.Aerc, args []string) error {
 		return err
 	}
 	for _, opt := range opts {
-		switch opt.Option {
-		case 'f':
+		if opt.Option == 'f' {
 			force = true
 		}
 	}
@@ -51,7 +52,7 @@ func (Recall) Execute(aerc *widgets.Aerc, args []string) error {
 		return errors.New("Usage: recall [-f]")
 	}
 
-	widget := aerc.SelectedTab().(widgets.ProvidesMessage)
+	widget := aerc.SelectedTabContent().(widgets.ProvidesMessage)
 	acct := widget.SelectedAccount()
 	if acct == nil {
 		return errors.New("No account selected")
@@ -69,9 +70,9 @@ func (Recall) Execute(aerc *widgets.Aerc, args []string) error {
 	if err != nil {
 		return errors.Wrap(err, "Recall failed")
 	}
-	acct.Logger().Println("Recalling message " + msgInfo.Envelope.MessageId)
+	log.Debugf("Recalling message <%s>", msgInfo.Envelope.MessageId)
 
-	composer, err := widgets.NewComposer(aerc, acct, aerc.Config(),
+	composer, err := widgets.NewComposer(aerc, acct,
 		acct.AccountConfig(), acct.Worker(), "", msgInfo.RFC822Headers,
 		models.OriginalMail{})
 	if err != nil {
@@ -93,7 +94,7 @@ func (Recall) Execute(aerc *widgets.Aerc, args []string) error {
 			} else {
 				tab.Name = subject
 			}
-			tab.Content.Invalidate()
+			ui.Invalidate()
 		})
 		composer.OnClose(func(composer *widgets.Composer) {
 			worker := composer.Worker()
@@ -130,16 +131,15 @@ func (Recall) Execute(aerc *widgets.Aerc, args []string) error {
 							deleteMessage()
 						default:
 						}
-						return
 					},
 				)
 				aerc.AddDialog(confirm)
 			}
-
 		})
 	}
 
-	lib.NewMessageStoreView(msgInfo, store, aerc.Crypto, aerc.DecryptKeys,
+	lib.NewMessageStoreView(msgInfo, acct.UiConfig().AutoMarkRead,
+		store, aerc.Crypto, aerc.DecryptKeys,
 		func(msg lib.MessageView, err error) {
 			if err != nil {
 				aerc.PushError(err.Error())
@@ -183,7 +183,10 @@ func (Recall) Execute(aerc *widgets.Aerc, args []string) error {
 						composer.SetEncrypt(md.IsEncrypted)
 					}
 					if md.IsSigned {
-						composer.SetSign(md.IsSigned)
+						err = composer.SetSign(md.IsSigned)
+						if err != nil {
+							log.Warnf("failed to set signed state: %v", err)
+						}
 					}
 				}
 				addTab()
@@ -197,23 +200,26 @@ func (Recall) Execute(aerc *widgets.Aerc, args []string) error {
 					}
 					bs, err := msg.BodyStructure().PartAtIndex(p)
 					if err != nil {
-						acct.Logger().Println("recall: PartAtIndex:", err)
+						log.Warnf("cannot get PartAtIndex %v: %v", p, err)
 						continue
 					}
 					msg.FetchBodyPart(p, func(reader io.Reader) {
-						mime := fmt.Sprintf("%s/%s", bs.MIMEType, bs.MIMESubType)
-						name, ok := bs.Params["name"]
+						mime := bs.FullMIMEType()
+						params := lib.SetUtf8Charset(bs.Params)
+						name, ok := params["name"]
 						if !ok {
 							name = fmt.Sprintf("%s_%s_%d", bs.MIMEType, bs.MIMESubType, rand.Uint64())
 						}
 						mu.Lock()
-						composer.AddPartAttachment(name, mime, bs.Params, reader)
+						err := composer.AddPartAttachment(name, mime, params, reader)
 						mu.Unlock()
+						if err != nil {
+							log.Errorf(err.Error())
+							aerc.PushError(err.Error())
+						}
 					})
 				}
-
 			})
-
 		})
 
 	return nil
