@@ -10,8 +10,13 @@ import (
 
 	"git.sr.ht/~sircmpwn/getopt"
 
+	"git.sr.ht/~rjarry/aerc/commands/account"
+	"git.sr.ht/~rjarry/aerc/config"
 	"git.sr.ht/~rjarry/aerc/lib"
+	"git.sr.ht/~rjarry/aerc/lib/crypto"
 	"git.sr.ht/~rjarry/aerc/lib/format"
+	"git.sr.ht/~rjarry/aerc/lib/ui"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/widgets"
 	"github.com/emersion/go-message/mail"
@@ -32,22 +37,25 @@ func (reply) Complete(aerc *widgets.Aerc, args []string) []string {
 }
 
 func (reply) Execute(aerc *widgets.Aerc, args []string) error {
-	opts, optind, err := getopt.Getopts(args, "aqT:")
+	opts, optind, err := getopt.Getopts(args, "acqT:")
 	if err != nil {
 		return err
 	}
 	if optind != len(args) {
-		return errors.New("Usage: reply [-aq -T <template>]")
+		return errors.New("Usage: reply [-acq -T <template>]")
 	}
 	var (
-		quote    bool
-		replyAll bool
-		template string
+		quote        bool
+		replyAll     bool
+		closeOnReply bool
+		template     string
 	)
 	for _, opt := range opts {
 		switch opt.Option {
 		case 'a':
 			replyAll = true
+		case 'c':
+			closeOnReply = true
 		case 'q':
 			quote = true
 		case 'T':
@@ -55,7 +63,7 @@ func (reply) Execute(aerc *widgets.Aerc, args []string) error {
 		}
 	}
 
-	widget := aerc.SelectedTab().(widgets.ProvidesMessage)
+	widget := aerc.SelectedTabContent().(widgets.ProvidesMessage)
 	acct := widget.SelectedAccount()
 
 	if acct == nil {
@@ -114,7 +122,7 @@ func (reply) Execute(aerc *widgets.Aerc, args []string) error {
 		to = msg.Envelope.From
 	}
 
-	if !aerc.Config().Compose.ReplyToSelf {
+	if !config.Compose.ReplyToSelf {
 		for i, v := range to {
 			if v.Address == from.Address {
 				to = append(to[:i], to[i+1:]...)
@@ -146,7 +154,7 @@ func (reply) Execute(aerc *widgets.Aerc, args []string) error {
 		to = append(to, envTos...)
 
 		for _, addr := range msg.Envelope.Cc {
-			//dedupe stuff from the to/from headers
+			// dedupe stuff from the to/from headers
 			if recSet.Contains(addr) {
 				continue
 			}
@@ -155,7 +163,7 @@ func (reply) Execute(aerc *widgets.Aerc, args []string) error {
 		recSet.AddList(cc)
 	}
 
-	subject := "Re: " + trimLocalizedRe(msg.Envelope.Subject)
+	subject := "Re: " + trimLocalizedRe(msg.Envelope.Subject, conf.LocalizedRe)
 
 	h := &mail.Header{}
 	h.SetAddressList("to", to)
@@ -173,13 +181,18 @@ func (reply) Execute(aerc *widgets.Aerc, args []string) error {
 		RFC822Headers: msg.RFC822Headers,
 	}
 
+	mv, _ := aerc.SelectedTabContent().(*widgets.MessageViewer)
 	addTab := func() error {
-		composer, err := widgets.NewComposer(aerc, acct, aerc.Config(),
+		composer, err := widgets.NewComposer(aerc, acct,
 			acct.AccountConfig(), acct.Worker(), template, h, original)
 		if err != nil {
 			aerc.PushError("Error: " + err.Error())
 			return err
 		}
+		if (mv != nil) && closeOnReply {
+			mv.Close()
+			aerc.RemoveTab(mv)
+		}
 
 		if args[0] == "reply" {
 			composer.FocusTerminal()
@@ -192,12 +205,16 @@ func (reply) Execute(aerc *widgets.Aerc, args []string) error {
 			} else {
 				tab.Name = subject
 			}
-			tab.Content.Invalidate()
+			ui.Invalidate()
 		})
 
 		composer.OnClose(func(c *widgets.Composer) {
-			if c.Sent() {
+			switch {
+			case c.Sent():
 				store.Answered([]uint32{msg.Uid}, true, nil)
+			case mv != nil && closeOnReply:
+				//nolint:errcheck // who cares?
+				account.ViewMessage{}.Execute(aerc, []string{"-p"})
 			}
 		})
 
@@ -206,7 +223,32 @@ func (reply) Execute(aerc *widgets.Aerc, args []string) error {
 
 	if quote {
 		if template == "" {
-			template = aerc.Config().Templates.QuotedReply
+			template = config.Templates.QuotedReply
+		}
+
+		if crypto.IsEncrypted(msg.BodyStructure) {
+			provider := aerc.SelectedTabContent().(widgets.ProvidesMessage)
+			mv, ok := provider.(*widgets.MessageViewer)
+			if !ok {
+				return fmt.Errorf("message is encrypted. can only quote reply while message is open")
+			}
+			p := provider.SelectedMessagePart()
+			if p == nil {
+				return fmt.Errorf("could not fetch message part")
+			}
+			mv.MessageView().FetchBodyPart(p.Index, func(reader io.Reader) {
+				buf := new(bytes.Buffer)
+				_, err := buf.ReadFrom(reader)
+				if err != nil {
+					log.Warnf("failed to fetch bodypart: %v", err)
+				}
+				original.Text = buf.String()
+				err = addTab()
+				if err != nil {
+					log.Warnf("failed to add tab: %v", err)
+				}
+			})
+			return nil
 		}
 
 		part := lib.FindPlaintext(msg.BodyStructure, nil)
@@ -224,14 +266,20 @@ func (reply) Execute(aerc *widgets.Aerc, args []string) error {
 
 		store.FetchBodyPart(msg.Uid, part, func(reader io.Reader) {
 			buf := new(bytes.Buffer)
-			buf.ReadFrom(reader)
+			_, err := buf.ReadFrom(reader)
+			if err != nil {
+				log.Warnf("failed to fetch bodypart: %v", err)
+			}
 			original.Text = buf.String()
-			addTab()
+			err = addTab()
+			if err != nil {
+				log.Warnf("failed to add tab: %v", err)
+			}
 		})
 		return nil
 	} else {
 		if template == "" {
-			template = aerc.Config().Templates.NewMessage
+			template = config.Templates.NewMessage
 		}
 		return addTab()
 	}
@@ -259,8 +307,8 @@ func (s addrSet) Contains(a *mail.Address) bool {
 	return ok
 }
 
-//setReferencesHeader adds the references header to target based on parent
-//according to RFC2822
+// setReferencesHeader adds the references header to target based on parent
+// according to RFC2822
 func setReferencesHeader(target, parent *mail.Header) error {
 	refs, err := parent.MsgIDList("references")
 	if err != nil {
@@ -285,20 +333,18 @@ func setReferencesHeader(target, parent *mail.Header) error {
 
 // addMimeType adds the proper mime type of the part to the originalMail struct
 func addMimeType(msg *models.MessageInfo, part []int,
-	orig *models.OriginalMail) error {
+	orig *models.OriginalMail,
+) error {
 	// caution, :forward uses the code as well, keep that in mind when modifying
 	bs, err := msg.BodyStructure.PartAtIndex(part)
 	if err != nil {
 		return err
 	}
-	orig.MIMEType = fmt.Sprintf("%s/%s", bs.MIMEType, bs.MIMESubType)
+	orig.MIMEType = bs.FullMIMEType()
 	return nil
 }
 
 // trimLocalizedRe removes known localizations of Re: commonly used by Outlook.
-func trimLocalizedRe(subject string) string {
+func trimLocalizedRe(subject string, localizedRe *regexp.Regexp) string {
 	return strings.TrimPrefix(subject, localizedRe.FindString(subject))
 }
-
-// localizedRe contains a list of known translations for the common Re:
-var localizedRe = regexp.MustCompile(`(?i)^((AW|RE|SV|VS|ODP|R): ?)+`)
