@@ -4,9 +4,7 @@ import (
 	"git.sr.ht/~rjarry/aerc/commands"
 )
 
-var (
-	MessageViewCommands *commands.Commands
-)
+var MessageViewCommands *commands.Commands
 
 func register(cmd commands.Command) {
 	if MessageViewCommands == nil {
