@@ -27,6 +27,7 @@ const (
 
 	STYLE_STATUSLINE_DEFAULT
 	STYLE_STATUSLINE_ERROR
+	STYLE_STATUSLINE_WARNING
 	STYLE_STATUSLINE_SUCCESS
 
 	STYLE_MSGLIST_DEFAULT
@@ -35,6 +36,7 @@ const (
 	STYLE_MSGLIST_FLAGGED
 	STYLE_MSGLIST_DELETED
 	STYLE_MSGLIST_MARKED
+	STYLE_MSGLIST_RESULT
 
 	STYLE_DIRLIST_DEFAULT
 	STYLE_DIRLIST_UNREAD
@@ -65,6 +67,7 @@ var StyleNames = map[string]StyleObject{
 
 	"statusline_default": STYLE_STATUSLINE_DEFAULT,
 	"statusline_error":   STYLE_STATUSLINE_ERROR,
+	"statusline_warning": STYLE_STATUSLINE_WARNING,
 	"statusline_success": STYLE_STATUSLINE_SUCCESS,
 
 	"msglist_default": STYLE_MSGLIST_DEFAULT,
@@ -73,6 +76,7 @@ var StyleNames = map[string]StyleObject{
 	"msglist_flagged": STYLE_MSGLIST_FLAGGED,
 	"msglist_deleted": STYLE_MSGLIST_DELETED,
 	"msglist_marked":  STYLE_MSGLIST_MARKED,
+	"msglist_result":  STYLE_MSGLIST_RESULT,
 
 	"dirlist_default": STYLE_DIRLIST_DEFAULT,
 	"dirlist_unread":  STYLE_DIRLIST_UNREAD,
@@ -99,6 +103,7 @@ type Style struct {
 	Blink     bool
 	Underline bool
 	Reverse   bool
+	Italic    bool
 }
 
 func (s Style) Get() tcell.Style {
@@ -108,7 +113,8 @@ func (s Style) Get() tcell.Style {
 		Bold(s.Bold).
 		Blink(s.Blink).
 		Underline(s.Underline).
-		Reverse(s.Reverse)
+		Reverse(s.Reverse).
+		Italic(s.Italic)
 }
 
 func (s *Style) Normal() {
@@ -116,6 +122,7 @@ func (s *Style) Normal() {
 	s.Blink = false
 	s.Underline = false
 	s.Reverse = false
+	s.Italic = false
 }
 
 func (s *Style) Default() *Style {
@@ -185,6 +192,12 @@ func (s *Style) Set(attr, val string) error {
 		} else {
 			s.Reverse = state
 		}
+	case "italic":
+		if state, err := boolSwitch(val, s.Italic); err != nil {
+			return err
+		} else {
+			s.Italic = state
+		}
 	case "default":
 		s.Default()
 	case "normal":
@@ -217,6 +230,9 @@ func (s Style) composeWith(styles []*Style) Style {
 		if st.Reverse != s.Reverse {
 			newStyle.Reverse = st.Reverse
 		}
+		if st.Italic != s.Italic {
+			newStyle.Italic = st.Italic
+		}
 	}
 	return newStyle
 }
@@ -265,7 +281,8 @@ func (ss StyleSet) Compose(so StyleObject, sos []StyleObject) tcell.Style {
 }
 
 func (ss StyleSet) ComposeSelected(so StyleObject,
-	sos []StyleObject) tcell.Style {
+	sos []StyleObject,
+) tcell.Style {
 	base := *ss.selected[so]
 	styles := make([]*Style, len(sos))
 	for i, so := range sos {
