@@ -4,15 +4,37 @@ import (
 	"errors"
 	"fmt"
 
+	"github.com/go-ini/ini"
 	"github.com/google/shlex"
 
 	"git.sr.ht/~rjarry/aerc/lib/format"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 )
 
-func (trig *TriggersConfig) ExecTrigger(triggerCmd string,
-	triggerFmt func(string) (string, error)) error {
+type TriggersConfig struct {
+	NewEmail       string `ini:"new-email"`
+	ExecuteCommand func(command []string) error
+}
 
+var Triggers = &TriggersConfig{}
+
+func parseTriggers(file *ini.File) error {
+	triggers, err := file.GetSection("triggers")
+	if err != nil {
+		goto out
+	}
+	if err := triggers.MapTo(&Triggers); err != nil {
+		return err
+	}
+out:
+	log.Debugf("aerc.conf: [triggers] %#v", Triggers)
+	return nil
+}
+
+func (trig *TriggersConfig) ExecTrigger(triggerCmd string,
+	triggerFmt func(string) (string, error),
+) error {
 	if len(triggerCmd) == 0 {
 		return errors.New("Trigger command empty")
 	}
@@ -32,19 +54,22 @@ func (trig *TriggersConfig) ExecTrigger(triggerCmd string,
 	return trig.ExecuteCommand(command)
 }
 
-func (trig *TriggersConfig) ExecNewEmail(account *AccountConfig,
-	conf *AercConfig, msg *models.MessageInfo) {
+func (trig *TriggersConfig) ExecNewEmail(
+	account *AccountConfig, msg *models.MessageInfo,
+) {
 	err := trig.ExecTrigger(trig.NewEmail,
 		func(part string) (string, error) {
 			formatstr, args, err := format.ParseMessageFormat(
-				part, conf.Ui.TimestampFormat,
-				conf.Ui.ThisDayTimeFormat,
-				conf.Ui.ThisWeekTimeFormat,
-				conf.Ui.ThisYearTimeFormat,
+				part, Ui.TimestampFormat,
+				Ui.ThisDayTimeFormat,
+				Ui.ThisWeekTimeFormat,
+				Ui.ThisYearTimeFormat,
+				Ui.IconAttachment,
 				format.Ctx{
 					FromAddress: account.From,
 					AccountName: account.Name,
-					MsgInfo:     msg},
+					MsgInfo:     msg,
+				},
 			)
 			if err != nil {
 				return "", err
@@ -52,6 +77,6 @@ func (trig *TriggersConfig) ExecNewEmail(account *AccountConfig,
 			return fmt.Sprintf(formatstr, args...), nil
 		})
 	if err != nil {
-		fmt.Printf("Error from the new-email trigger: %s\n", err)
+		log.Errorf("failed to run new-email trigger: %v", err)
 	}
 }
