@@ -3,7 +3,6 @@ package gpgbin
 import (
 	"bytes"
 	"io"
-	"io/ioutil"
 	"os"
 
 	"git.sr.ht/~rjarry/aerc/models"
@@ -15,25 +14,25 @@ func Verify(m io.Reader, s io.Reader) (*models.MessageDetails, error) {
 	args := []string{"--verify"}
 	if s != nil {
 		// Detached sig, save the sig to a tmp file and send msg over stdin
-		sig, err := ioutil.TempFile("", "sig")
+		sig, err := os.CreateTemp("", "sig")
 		if err != nil {
 			return nil, err
 		}
-		io.Copy(sig, s)
+		_, _ = io.Copy(sig, s)
 		sig.Close()
 		defer os.Remove(sig.Name())
 		args = append(args, sig.Name(), "-")
 	}
-	orig, err := ioutil.ReadAll(m)
+	orig, err := io.ReadAll(m)
 	if err != nil {
 		return nil, err
 	}
 	g := newGpg(bytes.NewReader(orig), args)
-	g.cmd.Run()
+	_ = g.cmd.Run()
 
 	out := bytes.NewReader(g.stdout.Bytes())
 	md := new(models.MessageDetails)
-	parse(out, md)
+	_ = parse(out, md)
 
 	md.Body = bytes.NewReader(orig)
 
