@@ -9,7 +9,7 @@ import (
 	"strings"
 	"time"
 
-	"git.sr.ht/~rjarry/aerc/logging"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"github.com/ProtonMail/go-crypto/openpgp"
 	"github.com/ProtonMail/go-crypto/openpgp/armor"
@@ -28,8 +28,19 @@ var (
 	locked bool
 )
 
+func (m *Mail) KeyringExists() bool {
+	keypath := path.Join(xdg.DataHome(), "aerc", "keyring.asc")
+	keyfile, err := os.Open(keypath)
+	if err != nil {
+		return false
+	}
+	defer keyfile.Close()
+	_, err = openpgp.ReadKeyRing(keyfile)
+	return err == nil
+}
+
 func (m *Mail) Init() error {
-	logging.Infof("Initializing PGP keyring")
+	log.Debugf("Initializing PGP keyring")
 	err := os.MkdirAll(path.Join(xdg.DataHome(), "aerc"), 0o700)
 	if err != nil {
 		return fmt.Errorf("failed to create data directory: %w", err)
@@ -50,13 +61,13 @@ func (m *Mail) Init() error {
 	if os.IsNotExist(err) {
 		return nil
 	} else if err != nil {
-		panic(err)
+		return err
 	}
 	defer keyfile.Close()
 
 	Keyring, err = openpgp.ReadKeyRing(keyfile)
 	if err != nil {
-		panic(err)
+		return err
 	}
 	return nil
 }
