@@ -10,6 +10,7 @@ import (
 
 	"git.sr.ht/~rjarry/aerc/models"
 	"github.com/emersion/go-message/mail"
+	"github.com/mattn/go-runewidth"
 )
 
 // AddressForHumans formats the address. If the address's name
@@ -79,8 +80,13 @@ type Ctx struct {
 }
 
 func ParseMessageFormat(format string, timeFmt string, thisDayTimeFmt string,
-	thisWeekTimeFmt string, thisYearTimeFmt string, ctx Ctx) (
-	string, []interface{}, error) {
+	thisWeekTimeFmt string, thisYearTimeFmt string, iconAttachment string, ctx Ctx) (
+	string, []interface{}, error,
+) {
+	if ctx.MsgInfo.Error != nil {
+		return "", nil,
+			errors.New("(unable to fetch header)")
+	}
 	retval := make([]byte, 0, len(format))
 	var args []interface{}
 
@@ -172,7 +178,7 @@ func ParseMessageFormat(format string, timeFmt string, thisDayTimeFmt string,
 			}
 			retval = append(retval, 's')
 			args = append(args,
-				dummyIfZeroDate(date.Local(),
+				DummyIfZeroDate(date.Local(),
 					timeFmt, thisDayTimeFmt,
 					thisWeekTimeFmt, thisYearTimeFmt))
 		case 'D':
@@ -182,7 +188,7 @@ func ParseMessageFormat(format string, timeFmt string, thisDayTimeFmt string,
 			}
 			retval = append(retval, 's')
 			args = append(args,
-				dummyIfZeroDate(date.Local(),
+				DummyIfZeroDate(date.Local(),
 					timeFmt, thisDayTimeFmt,
 					thisWeekTimeFmt, thisYearTimeFmt))
 		case 'f':
@@ -288,19 +294,21 @@ func ParseMessageFormat(format string, timeFmt string, thisDayTimeFmt string,
 			}
 		case 'Z':
 			// calculate all flags
-			var readReplyFlag = ""
-			var delFlag = ""
-			var flaggedFlag = ""
-			var markedFlag = ""
+			readReplyFlag := ""
+			delFlag := ""
+			flaggedFlag := ""
+			markedFlag := ""
+			hasattachment := ""
 			seen := false
 			recent := false
 			answered := false
 			for _, flag := range ctx.MsgInfo.Flags {
-				if flag == models.SeenFlag {
+				switch flag {
+				case models.SeenFlag:
 					seen = true
-				} else if flag == models.RecentFlag {
+				case models.RecentFlag:
 					recent = true
-				} else if flag == models.AnsweredFlag {
+				case models.AnsweredFlag:
 					answered = true
 				}
 				if flag == models.DeletedFlag {
@@ -326,8 +334,13 @@ func ParseMessageFormat(format string, timeFmt string, thisDayTimeFmt string,
 			if ctx.MsgIsMarked {
 				markedFlag = "*"
 			}
+			for _, bS := range ctx.MsgInfo.BodyStructure.Parts {
+				if strings.ToLower(bS.Disposition) == "attachment" {
+					hasattachment = iconAttachment
+				}
+			}
 			retval = append(retval, '4', 's')
-			args = append(args, readReplyFlag+delFlag+flaggedFlag+markedFlag)
+			args = append(args, readReplyFlag+delFlag+flaggedFlag+markedFlag+hasattachment)
 
 		// Move the below cases to proper alphabetical positions once
 		// implemented
@@ -365,6 +378,22 @@ func ParseMessageFormat(format string, timeFmt string, thisDayTimeFmt string,
 		i = ni + 1
 	}
 
+	const zeroWidthSpace rune = '\u200b'
+	for i, val := range args {
+		if s, ok := val.(string); ok {
+			var out strings.Builder
+			for _, r := range s {
+				w := runewidth.RuneWidth(r)
+				for w > 1 {
+					out.WriteRune(zeroWidthSpace)
+					w -= 1
+				}
+				out.WriteRune(r)
+			}
+			args[i] = out.String()
+		}
+	}
+
 	return string(retval), args, nil
 
 handle_end_error:
@@ -372,8 +401,9 @@ handle_end_error:
 		errors.New("reached end of string while parsing message format")
 }
 
-func dummyIfZeroDate(date time.Time, format string, todayFormat string,
-	thisWeekFormat string, thisYearFormat string) string {
+func DummyIfZeroDate(date time.Time, format string, todayFormat string,
+	thisWeekFormat string, thisYearFormat string,
+) string {
 	if date.IsZero() {
 		return strings.Repeat("?", len(format))
 	}
