@@ -2,13 +2,15 @@ package lib
 
 import (
 	"bytes"
+	"fmt"
 	"io"
+	"strings"
 
 	"github.com/ProtonMail/go-crypto/openpgp"
-	"github.com/emersion/go-message"
 	_ "github.com/emersion/go-message/charset"
 
 	"git.sr.ht/~rjarry/aerc/lib/crypto"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/lib"
 	"git.sr.ht/~rjarry/aerc/worker/types"
@@ -26,10 +28,16 @@ type MessageView interface {
 	// Returns the message store that this message was originally sourced from
 	Store() *MessageStore
 
+	// Fetches the full message
+	FetchFull(cb func(io.Reader))
+
 	// Fetches a specific body part for this message
 	FetchBodyPart(part []int, cb func(io.Reader))
 
 	MessageDetails() *models.MessageDetails
+
+	// SeenFlagSet returns true if the "seen" flag has been set
+	SeenFlagSet() bool
 }
 
 func usePGP(info *models.BodyStructure) bool {
@@ -57,20 +65,28 @@ type MessageStoreView struct {
 	message       []byte
 	details       *models.MessageDetails
 	bodyStructure *models.BodyStructure
+	setSeen       bool
 }
 
-func NewMessageStoreView(messageInfo *models.MessageInfo,
+func NewMessageStoreView(messageInfo *models.MessageInfo, setSeen bool,
 	store *MessageStore, pgp crypto.Provider, decryptKeys openpgp.PromptFunction,
 	cb func(MessageView, error),
 ) {
+	if messageInfo == nil {
+		// Call nils to the callback, the split view will use this to
+		// display an empty view
+		cb(nil, nil)
+		return
+	}
 	msv := &MessageStoreView{
 		messageInfo, store,
 		nil, nil, messageInfo.BodyStructure,
+		setSeen,
 	}
 
 	if usePGP(messageInfo.BodyStructure) {
-		store.FetchFull([]uint32{messageInfo.Uid}, func(fm *types.FullMessage) {
-			reader := lib.NewCRLFReader(fm.Content.Reader)
+		msv.FetchFull(func(fm io.Reader) {
+			reader := lib.NewCRLFReader(fm)
 			md, err := pgp.Decrypt(reader, decryptKeys)
 			if err != nil {
 				cb(nil, err)
@@ -81,7 +97,7 @@ func NewMessageStoreView(messageInfo *models.MessageInfo,
 				cb(nil, err)
 				return
 			}
-			decrypted, err := message.Read(bytes.NewBuffer(msv.message))
+			decrypted, err := lib.ReadMessage(bytes.NewBuffer(msv.message))
 			if err != nil {
 				cb(nil, err)
 				return
@@ -98,7 +114,13 @@ func NewMessageStoreView(messageInfo *models.MessageInfo,
 	} else {
 		cb(msv, nil)
 	}
-	store.Flag([]uint32{messageInfo.Uid}, models.SeenFlag, true, nil)
+	if setSeen {
+		store.Flag([]uint32{messageInfo.Uid}, models.SeenFlag, true, nil)
+	}
+}
+
+func (msv *MessageStoreView) SeenFlagSet() bool {
+	return msv.setSeen
 }
 
 func (msv *MessageStoreView) MessageInfo() *models.MessageInfo {
@@ -117,20 +139,38 @@ func (msv *MessageStoreView) MessageDetails() *models.MessageDetails {
 	return msv.details
 }
 
+func (msv *MessageStoreView) FetchFull(cb func(io.Reader)) {
+	if msv.message == nil && msv.messageStore != nil {
+		msv.messageStore.FetchFull([]uint32{msv.messageInfo.Uid},
+			func(fm *types.FullMessage) {
+				cb(fm.Content.Reader)
+			})
+		return
+	}
+	cb(bytes.NewReader(msv.message))
+}
+
 func (msv *MessageStoreView) FetchBodyPart(part []int, cb func(io.Reader)) {
-	if msv.message == nil {
+	if msv.message == nil && msv.messageStore != nil {
 		msv.messageStore.FetchBodyPart(msv.messageInfo.Uid, part, cb)
 		return
 	}
 
 	buf := bytes.NewBuffer(msv.message)
-	msg, err := message.Read(buf)
+	msg, err := lib.ReadMessage(buf)
 	if err != nil {
 		panic(err)
 	}
 	reader, err := lib.FetchEntityPartReader(msg, part)
 	if err != nil {
-		panic(err)
+		errMsg := fmt.Errorf("Failed to fetch message part: %w", err)
+		log.Errorf(errMsg.Error())
+		if msv.message != nil {
+			log.Warnf("Displaying raw message part")
+			reader = bytes.NewReader(msv.message)
+		} else {
+			reader = strings.NewReader(errMsg.Error())
+		}
 	}
 	cb(reader)
 }
