@@ -5,15 +5,18 @@ import (
 	"sync"
 	"time"
 
+	"git.sr.ht/~rjarry/aerc/lib/iterator"
 	"git.sr.ht/~rjarry/aerc/lib/marker"
 	"git.sr.ht/~rjarry/aerc/lib/sort"
-	"git.sr.ht/~rjarry/aerc/logging"
+	"git.sr.ht/~rjarry/aerc/lib/ui"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
 
 // Accesses to fields must be guarded by MessageStore.Lock/Unlock
 type MessageStore struct {
+	sync.Mutex
 	Deleted  map[uint32]interface{}
 	DirInfo  models.DirectoryInfo
 	Messages map[uint32]*models.MessageInfo
@@ -35,10 +38,13 @@ type MessageStore struct {
 	filter      []string
 
 	sortCriteria []*types.SortCriterion
+	sortDefault  []*types.SortCriterion
 
-	threadedView bool
-	buildThreads bool
-	builder      *ThreadBuilder
+	threadedView       bool
+	reverseThreadOrder bool
+	sortThreadSiblings bool
+	buildThreads       bool
+	builder            *ThreadBuilder
 
 	// Map of uids we've asked the worker to fetch
 	onUpdate       func(store *MessageStore) // TODO: multiple onUpdate handlers
@@ -48,6 +54,10 @@ type MessageStore struct {
 	pendingHeaders map[uint32]interface{}
 	worker         *types.Worker
 
+	needsFlags         []uint32
+	fetchFlagsDebounce *time.Timer
+	fetchFlagsDelay    time.Duration
+
 	triggerNewEmail        func(*models.MessageInfo)
 	triggerDirectoryChange func()
 
@@ -57,6 +67,9 @@ type MessageStore struct {
 
 	// threads mutex protects the store.threads and store.threadCallback
 	threadsMutex sync.Mutex
+
+	iterFactory iterator.Factory
+	onSelect    func(*models.MessageInfo)
 }
 
 const MagicUid = 0xFFFFFFFF
@@ -65,8 +78,9 @@ func NewMessageStore(worker *types.Worker,
 	dirInfo *models.DirectoryInfo,
 	defaultSortCriteria []*types.SortCriterion,
 	thread bool, clientThreads bool, clientThreadsDelay time.Duration,
+	reverseOrder bool, reverseThreadOrder bool, sortThreadSiblings bool,
 	triggerNewEmail func(*models.MessageInfo),
-	triggerDirectoryChange func(),
+	triggerDirectoryChange func(), onSelect func(*models.MessageInfo),
 ) *MessageStore {
 	if !dirInfo.Caps.Thread {
 		clientThreads = true
@@ -81,20 +95,29 @@ func NewMessageStore(worker *types.Worker,
 
 		bodyCallbacks: make(map[uint32][]func(*types.FullMessage)),
 
-		threadedView: thread,
-		buildThreads: clientThreads,
+		threadedView:       thread,
+		buildThreads:       clientThreads,
+		reverseThreadOrder: reverseThreadOrder,
+		sortThreadSiblings: sortThreadSiblings,
 
 		filter:       []string{"filter"},
 		sortCriteria: defaultSortCriteria,
+		sortDefault:  defaultSortCriteria,
 
 		pendingBodies:  make(map[uint32]interface{}),
 		pendingHeaders: make(map[uint32]interface{}),
 		worker:         worker,
 
+		needsFlags:      []uint32{},
+		fetchFlagsDelay: 50 * time.Millisecond,
+
 		triggerNewEmail:        triggerNewEmail,
 		triggerDirectoryChange: triggerDirectoryChange,
 
 		threadBuilderDelay: clientThreadsDelay,
+
+		iterFactory: iterator.NewFactory(reverseOrder),
+		onSelect:    onSelect,
 	}
 }
 
@@ -213,31 +236,38 @@ func (store *MessageStore) Update(msg types.WorkerMessage) {
 			store.runThreadBuilderNow()
 		}
 	case *types.DirectoryThreaded:
-		var uids []uint32
 		newMap := make(map[uint32]*models.MessageInfo)
 
-		for i := len(msg.Threads) - 1; i >= 0; i-- {
-			_ = msg.Threads[i].Walk(func(t *types.Thread, level int, currentErr error) error {
-				uid := t.Uid
-				uids = append([]uint32{uid}, uids...)
-				if msg, ok := store.Messages[uid]; ok {
-					newMap[uid] = msg
-				} else {
-					newMap[uid] = nil
-					directoryChange = true
-				}
-				return nil
-			})
-		}
-		store.Messages = newMap
-		store.uids = uids
+		store.builder = NewThreadBuilder(store.iterFactory)
+		store.builder.RebuildUids(msg.Threads, store.reverseThreadOrder)
+		store.uids = store.builder.Uids()
 		store.threads = msg.Threads
+
+		for _, uid := range store.uids {
+			if msg, ok := store.Messages[uid]; ok {
+				newMap[uid] = msg
+			} else {
+				newMap[uid] = nil
+				directoryChange = true
+			}
+		}
+
+		store.Messages = newMap
 		update = true
 	case *types.MessageInfo:
 		if existing, ok := store.Messages[msg.Info.Uid]; ok && existing != nil {
 			merge(existing, msg.Info)
 		} else if msg.Info.Envelope != nil {
 			store.Messages[msg.Info.Uid] = msg.Info
+			if store.selectedUid == msg.Info.Uid {
+				store.onSelect(msg.Info)
+			}
+		}
+		if msg.NeedsFlags {
+			store.Lock()
+			store.needsFlags = append(store.needsFlags, msg.Info.Uid)
+			store.Unlock()
+			store.fetchFlags()
 		}
 		seen := false
 		recent := false
@@ -290,6 +320,9 @@ func (store *MessageStore) Update(msg types.WorkerMessage) {
 			}
 		}
 		store.uids = uids
+		if len(uids) == 0 {
+			store.Select(MagicUid)
+		}
 
 		var newResults []uint32
 		for _, res := range store.results {
@@ -340,11 +373,27 @@ func (store *MessageStore) update(threads bool) {
 	if store.onUpdateDirs != nil {
 		store.onUpdateDirs()
 	}
-	if store.BuildThreads() && store.ThreadedView() && threads {
-		store.runThreadBuilder()
+	if store.ThreadedView() && threads {
+		switch {
+		case store.BuildThreads():
+			store.runThreadBuilder()
+		default:
+			if store.builder == nil {
+				store.builder = NewThreadBuilder(store.iterFactory)
+			}
+			store.builder.RebuildUids(store.Threads(), store.reverseThreadOrder)
+		}
 	}
 }
 
+func (store *MessageStore) SetReverseThreadOrder(reverse bool) {
+	store.reverseThreadOrder = reverse
+}
+
+func (store *MessageStore) ReverseThreadOrder() bool {
+	return store.reverseThreadOrder
+}
+
 func (store *MessageStore) SetThreadedView(thread bool) {
 	store.threadedView = thread
 	if store.buildThreads {
@@ -364,6 +413,12 @@ func (store *MessageStore) Threads() []*types.Thread {
 	return store.threads
 }
 
+func (store *MessageStore) ThreadsIterator() iterator.Iterator {
+	store.threadsMutex.Lock()
+	defer store.threadsMutex.Unlock()
+	return store.iterFactory.NewIterator(store.threads)
+}
+
 func (store *MessageStore) ThreadedView() bool {
 	return store.threadedView
 }
@@ -373,26 +428,34 @@ func (store *MessageStore) BuildThreads() bool {
 }
 
 func (store *MessageStore) runThreadBuilder() {
+	if store.builder == nil {
+		store.builder = NewThreadBuilder(store.iterFactory)
+		for _, msg := range store.Messages {
+			store.builder.Update(msg)
+		}
+	}
 	if store.threadBuilderDebounce != nil {
 		if store.threadBuilderDebounce.Stop() {
-			logging.Infof("thread builder debounced")
+			log.Tracef("thread builder debounced")
 		}
 	}
 	store.threadBuilderDebounce = time.AfterFunc(store.threadBuilderDelay, func() {
 		store.runThreadBuilderNow()
+		ui.QueueRedraw()
 	})
 }
 
 // runThreadBuilderNow runs the threadbuilder without any debounce logic
 func (store *MessageStore) runThreadBuilderNow() {
 	if store.builder == nil {
-		store.builder = NewThreadBuilder()
+		store.builder = NewThreadBuilder(store.iterFactory)
 		for _, msg := range store.Messages {
 			store.builder.Update(msg)
 		}
 	}
 	// build new threads
-	th := store.builder.Threads(store.uids)
+	th := store.builder.Threads(store.uids, store.reverseThreadOrder,
+		store.sortThreadSiblings)
 
 	// save local threads to the message store variable and
 	// run callback if defined (callback should reposition cursor)
@@ -422,7 +485,7 @@ func (store *MessageStore) SelectedThread() *types.Thread {
 			return nil
 		})
 		if err != nil {
-			logging.Errorf("SelectedThread failed: %w", err)
+			log.Errorf("SelectedThread failed: %v", err)
 		}
 		if found {
 			break
@@ -528,14 +591,18 @@ func (store *MessageStore) Uids() []uint32 {
 	return store.uids
 }
 
+func (store *MessageStore) UidsIterator() iterator.Iterator {
+	return store.iterFactory.NewIterator(store.Uids())
+}
+
 func (store *MessageStore) Selected() *models.MessageInfo {
 	return store.Messages[store.selectedUid]
 }
 
 func (store *MessageStore) SelectedUid() uint32 {
 	if store.selectedUid == MagicUid && len(store.Uids()) > 0 {
-		uids := store.Uids()
-		store.selectedUid = uids[len(uids)-1]
+		iter := store.UidsIterator()
+		store.Select(store.Uids()[iter.StartIndex()])
 	}
 	return store.selectedUid
 }
@@ -546,10 +613,17 @@ func (store *MessageStore) Select(uid uint32) {
 		store.threadCallback = nil
 	}
 	store.threadsMutex.Unlock()
+	store.selectPriv(uid)
+}
+
+func (store *MessageStore) selectPriv(uid uint32) {
 	store.selectedUid = uid
 	if store.marker != nil {
 		store.marker.UpdateVisualMark()
 	}
+	if store.onSelect != nil {
+		store.onSelect(store.Selected())
+	}
 }
 
 func (store *MessageStore) NextPrev(delta int) {
@@ -558,28 +632,26 @@ func (store *MessageStore) NextPrev(delta int) {
 		return
 	}
 
-	uid := store.SelectedUid()
+	iter := store.iterFactory.NewIterator(uids)
 
-	newIdx := store.FindIndexByUid(uid)
+	newIdx := store.FindIndexByUid(store.SelectedUid())
 	if newIdx < 0 {
-		store.Select(uids[len(uids)-1])
+		store.Select(uids[iter.StartIndex()])
 		return
 	}
-
-	newIdx -= delta
-	if newIdx >= len(uids) {
-		newIdx = len(uids) - 1
-	} else if newIdx < 0 {
-		newIdx = 0
-	}
-
+	newIdx = iterator.MoveIndex(
+		newIdx,
+		delta,
+		iter,
+		iterator.FixBounds,
+	)
 	store.Select(uids[newIdx])
 
 	if store.BuildThreads() && store.ThreadedView() {
 		store.threadsMutex.Lock()
 		store.threadCallback = func() {
 			if uids := store.Uids(); len(uids) > newIdx {
-				store.selectedUid = uids[newIdx]
+				store.selectPriv(uids[newIdx])
 			}
 		}
 		store.threadsMutex.Unlock()
@@ -588,15 +660,7 @@ func (store *MessageStore) NextPrev(delta int) {
 	if store.marker != nil {
 		store.marker.UpdateVisualMark()
 	}
-
-	nextResultIndex := len(store.results) - store.resultIndex - 2*delta
-	if nextResultIndex < 0 || nextResultIndex >= len(store.results) {
-		return
-	}
-	nextResultUid := store.results[nextResultIndex]
-	if nextResultUid == store.SelectedUid() {
-		store.resultIndex += delta
-	}
+	store.updateResults()
 }
 
 func (store *MessageStore) Next() {
@@ -634,6 +698,16 @@ func (store *MessageStore) ApplySearch(results []uint32) {
 	store.NextResult()
 }
 
+// IsResult returns true if uid is a search result
+func (store *MessageStore) IsResult(uid uint32) bool {
+	for _, hit := range store.results {
+		if hit == uid {
+			return true
+		}
+	}
+	return false
+}
+
 func (store *MessageStore) SetFilter(args []string) {
 	store.filter = append(store.filter, args...)
 }
@@ -644,21 +718,38 @@ func (store *MessageStore) ApplyClear() {
 	if store.onFilterChange != nil {
 		store.onFilterChange(store)
 	}
-	store.Sort(nil, nil)
+	store.Sort(store.sortDefault, nil)
+}
+
+func (store *MessageStore) updateResults() {
+	if len(store.results) == 0 || store.resultIndex < 0 {
+		return
+	}
+	uid := store.SelectedUid()
+	for i, u := range store.results {
+		if uid == u {
+			store.resultIndex = i
+			break
+		}
+	}
 }
 
 func (store *MessageStore) nextPrevResult(delta int) {
 	if len(store.results) == 0 {
 		return
 	}
-	store.resultIndex += delta
-	if store.resultIndex >= len(store.results) {
-		store.resultIndex = 0
-	}
+	iter := store.iterFactory.NewIterator(store.results)
 	if store.resultIndex < 0 {
-		store.resultIndex = len(store.results) - 1
+		store.resultIndex = iter.StartIndex()
+	} else {
+		store.resultIndex = iterator.MoveIndex(
+			store.resultIndex,
+			delta,
+			iter,
+			iterator.WrapBounds,
+		)
 	}
-	store.Select(store.results[len(store.results)-store.resultIndex-1])
+	store.Select(store.results[store.resultIndex])
 	store.update(false)
 }
 
@@ -740,3 +831,17 @@ func (store *MessageStore) Capabilities() *models.Capabilities {
 func (store *MessageStore) SelectedIndex() int {
 	return store.FindIndexByUid(store.selectedUid)
 }
+
+func (store *MessageStore) fetchFlags() {
+	if store.fetchFlagsDebounce != nil {
+		store.fetchFlagsDebounce.Stop()
+	}
+	store.fetchFlagsDebounce = time.AfterFunc(store.fetchFlagsDelay, func() {
+		store.Lock()
+		store.worker.PostAction(&types.FetchMessageFlags{
+			Uids: store.needsFlags,
+		}, nil)
+		store.needsFlags = []uint32{}
+		store.Unlock()
+	})
+}
