@@ -14,7 +14,6 @@ const (
 )
 
 type Bordered struct {
-	Invalidatable
 	borders  uint
 	content  Drawable
 	uiConfig *config.UIConfig
@@ -28,20 +27,15 @@ func NewBordered(
 		content:  content,
 		uiConfig: uiConfig,
 	}
-	content.OnInvalidate(b.contentInvalidated)
 	return b
 }
 
-func (bordered *Bordered) contentInvalidated(d Drawable) {
-	bordered.Invalidate()
-}
-
 func (bordered *Bordered) Children() []Drawable {
 	return []Drawable{bordered.content}
 }
 
 func (bordered *Bordered) Invalidate() {
-	bordered.DoInvalidate(bordered)
+	Invalidate()
 }
 
 func (bordered *Bordered) Draw(ctx *Context) {
