@@ -2,6 +2,7 @@ package ui
 
 import (
 	"io"
+	"sync"
 
 	"github.com/gdamore/tcell/v2"
 	"github.com/mattn/go-runewidth"
@@ -10,32 +11,31 @@ import (
 )
 
 type Tabs struct {
-	Tabs       []*Tab
+	tabs       []*Tab
 	TabStrip   *TabStrip
 	TabContent *TabContent
-	Selected   int
+	curIndex   int
 	history    []int
+	m          sync.Mutex
 
 	uiConfig *config.UIConfig
 
-	onInvalidateStrip   func(d Drawable)
-	onInvalidateContent func(d Drawable)
-
-	parent   *Tabs
+	parent   *Tabs //nolint:structcheck // used within this file
 	CloseTab func(index int)
 }
 
 type Tab struct {
 	Content        Drawable
 	Name           string
-	invalid        bool
 	pinned         bool
 	indexBeforePin int
 	uiConf         *config.UIConfig
 }
 
-type TabStrip Tabs
-type TabContent Tabs
+type (
+	TabStrip   Tabs
+	TabContent Tabs
+)
 
 func NewTabs(uiConf *config.UIConfig) *Tabs {
 	tabs := &Tabs{}
@@ -54,33 +54,29 @@ func (tabs *Tabs) Add(content Drawable, name string, uiConf *config.UIConfig) *T
 		Name:    name,
 		uiConf:  uiConf,
 	}
-	tabs.Tabs = append(tabs.Tabs, tab)
-	tabs.TabStrip.Invalidate()
-	content.OnInvalidate(tabs.invalidateChild)
+	tabs.tabs = append(tabs.tabs, tab)
+	tabs.selectPriv(len(tabs.tabs) - 1)
 	return tab
 }
 
-func (tabs *Tabs) invalidateChild(d Drawable) {
-	if tabs.Selected >= len(tabs.Tabs) {
-		return
-	}
-
-	if tabs.Tabs[tabs.Selected].Content == d {
-		if tabs.onInvalidateContent != nil {
-			tabs.onInvalidateContent(tabs.TabContent)
-		}
+func (tabs *Tabs) Names() []string {
+	var names []string
+	tabs.m.Lock()
+	for _, tab := range tabs.tabs {
+		names = append(names, tab.Name)
 	}
+	tabs.m.Unlock()
+	return names
 }
 
 func (tabs *Tabs) Remove(content Drawable) {
+	tabs.m.Lock()
+	defer tabs.m.Unlock()
 	indexToRemove := -1
-	removeTab := func() {}
-	for i, tab := range tabs.Tabs {
+	for i, tab := range tabs.tabs {
 		if tab.Content == content {
-			removeTab = func() {
-				tabs.Tabs = append(tabs.Tabs[:i], tabs.Tabs[i+1:]...)
-				tabs.removeHistory(i)
-			}
+			tabs.tabs = append(tabs.tabs[:i], tabs.tabs[i+1:]...)
+			tabs.removeHistory(i)
 			indexToRemove = i
 			break
 		}
@@ -89,141 +85,189 @@ func (tabs *Tabs) Remove(content Drawable) {
 		return
 	}
 	// only pop the tab history if the closing tab is selected
-	if indexToRemove == tabs.Selected {
+	if indexToRemove == tabs.curIndex {
 		index, ok := tabs.popHistory()
 		if ok {
-			tabs.Select(index)
-			interactive, ok := tabs.Tabs[tabs.Selected].Content.(Interactive)
-			if ok {
-				interactive.Focus(true)
-			}
+			tabs.selectPriv(index)
 		}
-	} else if indexToRemove < tabs.Selected {
+	} else if indexToRemove < tabs.curIndex {
 		// selected tab is now one to the left of where it was
-		tabs.Selected--
+		tabs.selectPriv(tabs.curIndex - 1)
+	}
+	interactive, ok := tabs.tabs[tabs.curIndex].Content.(Interactive)
+	if ok {
+		interactive.Focus(true)
 	}
-	removeTab()
-	tabs.TabStrip.Invalidate()
 }
 
 func (tabs *Tabs) Replace(contentSrc Drawable, contentTarget Drawable, name string) {
+	tabs.m.Lock()
+	defer tabs.m.Unlock()
 	replaceTab := &Tab{
 		Content: contentTarget,
 		Name:    name,
 	}
-	for i, tab := range tabs.Tabs {
+	for i, tab := range tabs.tabs {
 		if tab.Content == contentSrc {
-			tabs.Tabs[i] = replaceTab
-			tabs.Select(i)
+			tabs.tabs[i] = replaceTab
+			tabs.selectPriv(i)
 			if c, ok := contentSrc.(io.Closer); ok {
 				c.Close()
 			}
 			break
 		}
 	}
-	tabs.TabStrip.Invalidate()
-	contentTarget.OnInvalidate(tabs.invalidateChild)
+	Invalidate()
 }
 
-func (tabs *Tabs) Select(index int) {
-	if index >= len(tabs.Tabs) {
-		index = len(tabs.Tabs) - 1
+func (tabs *Tabs) Get(index int) *Tab {
+	tabs.m.Lock()
+	defer tabs.m.Unlock()
+	if index < 0 || index >= len(tabs.tabs) {
+		return nil
+	}
+	return tabs.tabs[index]
+}
+
+func (tabs *Tabs) Selected() *Tab {
+	tabs.m.Lock()
+	defer tabs.m.Unlock()
+	if tabs.curIndex < 0 || tabs.curIndex >= len(tabs.tabs) {
+		return nil
+	}
+	return tabs.tabs[tabs.curIndex]
+}
+
+func (tabs *Tabs) Select(index int) bool {
+	tabs.m.Lock()
+	defer tabs.m.Unlock()
+	return tabs.selectPriv(index)
+}
+
+func (tabs *Tabs) selectPriv(index int) bool {
+	if index < 0 || index >= len(tabs.tabs) {
+		return false
 	}
 
-	if tabs.Selected != index {
+	if tabs.curIndex != index {
 		// only push valid tabs onto the history
-		if tabs.Selected < len(tabs.Tabs) {
-			tabs.pushHistory(tabs.Selected)
+		if tabs.curIndex < len(tabs.tabs) {
+			tabs.pushHistory(tabs.curIndex)
 		}
-		tabs.Selected = index
-		tabs.TabStrip.Invalidate()
-		tabs.TabContent.Invalidate()
+		tabs.curIndex = index
+		Invalidate()
 	}
+	return true
+}
+
+func (tabs *Tabs) SelectName(name string) bool {
+	tabs.m.Lock()
+	defer tabs.m.Unlock()
+	for i, tab := range tabs.tabs {
+		if tab.Name == name {
+			return tabs.selectPriv(i)
+		}
+	}
+	return false
 }
 
 func (tabs *Tabs) SelectPrevious() bool {
+	tabs.m.Lock()
+	defer tabs.m.Unlock()
 	index, ok := tabs.popHistory()
 	if !ok {
 		return false
 	}
-	tabs.Select(index)
-	return true
+	return tabs.selectPriv(index)
 }
 
-func (tabs *Tabs) MoveTab(to int) {
-	from := tabs.Selected
+func (tabs *Tabs) MoveTab(to int, relative bool) {
+	tabs.m.Lock()
+	tabs.moveTabPriv(to, relative)
+	tabs.m.Unlock()
+}
 
+func (tabs *Tabs) moveTabPriv(to int, relative bool) {
+	from := tabs.curIndex
+
+	if relative {
+		to = from + to
+	}
 	if to < 0 {
 		to = 0
 	}
-
-	if to >= len(tabs.Tabs) {
-		to = len(tabs.Tabs) - 1
+	if to >= len(tabs.tabs) {
+		to = len(tabs.tabs) - 1
 	}
 
-	tab := tabs.Tabs[from]
-	if to > from {
-		copy(tabs.Tabs[from:to], tabs.Tabs[from+1:to+1])
+	tab := tabs.tabs[from]
+	switch {
+	case to > from:
+		copy(tabs.tabs[from:to], tabs.tabs[from+1:to+1])
 		for i, h := range tabs.history {
 			if h == from {
 				tabs.history[i] = to
 			}
 			if h > from && h <= to {
-				tabs.history[i] -= 1
+				tabs.history[i]--
 			}
 		}
-	} else if from > to {
-		copy(tabs.Tabs[to+1:from+1], tabs.Tabs[to:from])
+	case from > to:
+		copy(tabs.tabs[to+1:from+1], tabs.tabs[to:from])
 		for i, h := range tabs.history {
 			if h == from {
 				tabs.history[i] = to
 			}
 			if h >= to && h < from {
-				tabs.history[i] += 1
+				tabs.history[i]++
 			}
 		}
-	} else {
+	default:
 		return
 	}
 
-	tabs.Tabs[to] = tab
-	tabs.Selected = to
-	tabs.TabStrip.Invalidate()
+	tabs.tabs[to] = tab
+	tabs.curIndex = to
+	Invalidate()
 }
 
 func (tabs *Tabs) PinTab() {
-	if tabs.Tabs[tabs.Selected].pinned {
+	tabs.m.Lock()
+	defer tabs.m.Unlock()
+	if tabs.tabs[tabs.curIndex].pinned {
 		return
 	}
 
-	pinEnd := len(tabs.Tabs)
-	for i, t := range tabs.Tabs {
+	pinEnd := len(tabs.tabs)
+	for i, t := range tabs.tabs {
 		if !t.pinned {
 			pinEnd = i
 			break
 		}
 	}
 
-	for _, t := range tabs.Tabs {
-		if t.pinned && t.indexBeforePin > tabs.Selected-pinEnd {
+	for _, t := range tabs.tabs {
+		if t.pinned && t.indexBeforePin > tabs.curIndex-pinEnd {
 			t.indexBeforePin -= 1
 		}
 	}
 
-	tabs.Tabs[tabs.Selected].pinned = true
-	tabs.Tabs[tabs.Selected].indexBeforePin = tabs.Selected - pinEnd
+	tabs.tabs[tabs.curIndex].pinned = true
+	tabs.tabs[tabs.curIndex].indexBeforePin = tabs.curIndex - pinEnd
 
-	tabs.MoveTab(pinEnd)
+	tabs.moveTabPriv(pinEnd, false)
 }
 
 func (tabs *Tabs) UnpinTab() {
-	if !tabs.Tabs[tabs.Selected].pinned {
+	tabs.m.Lock()
+	defer tabs.m.Unlock()
+	if !tabs.tabs[tabs.curIndex].pinned {
 		return
 	}
 
-	pinEnd := len(tabs.Tabs)
-	for i, t := range tabs.Tabs {
-		if i != tabs.Selected && t.pinned && t.indexBeforePin > tabs.Tabs[tabs.Selected].indexBeforePin {
+	pinEnd := len(tabs.tabs)
+	for i, t := range tabs.tabs {
+		if i != tabs.curIndex && t.pinned && t.indexBeforePin > tabs.tabs[tabs.curIndex].indexBeforePin {
 			t.indexBeforePin += 1
 		}
 		if !t.pinned {
@@ -232,25 +276,29 @@ func (tabs *Tabs) UnpinTab() {
 		}
 	}
 
-	tabs.Tabs[tabs.Selected].pinned = false
+	tabs.tabs[tabs.curIndex].pinned = false
 
-	tabs.MoveTab(tabs.Tabs[tabs.Selected].indexBeforePin + pinEnd - 1)
+	tabs.moveTabPriv(tabs.tabs[tabs.curIndex].indexBeforePin+pinEnd-1, false)
 }
 
 func (tabs *Tabs) NextTab() {
-	next := tabs.Selected + 1
-	if next >= len(tabs.Tabs) {
+	tabs.m.Lock()
+	next := tabs.curIndex + 1
+	if next >= len(tabs.tabs) {
 		next = 0
 	}
-	tabs.Select(next)
+	tabs.selectPriv(next)
+	tabs.m.Unlock()
 }
 
 func (tabs *Tabs) PrevTab() {
-	next := tabs.Selected - 1
+	tabs.m.Lock()
+	next := tabs.curIndex - 1
 	if next < 0 {
-		next = len(tabs.Tabs) - 1
+		next = len(tabs.tabs) - 1
 	}
-	tabs.Select(next)
+	tabs.selectPriv(next)
+	tabs.m.Unlock()
 }
 
 func (tabs *Tabs) pushHistory(index int) {
@@ -274,7 +322,7 @@ func (tabs *Tabs) removeHistory(index int) {
 			continue
 		}
 		if item > index {
-			item = item - 1
+			item--
 		}
 		// dedup
 		if i > 0 && len(newHist) > 0 && item == newHist[len(newHist)-1] {
@@ -288,13 +336,14 @@ func (tabs *Tabs) removeHistory(index int) {
 // TODO: Color repository
 func (strip *TabStrip) Draw(ctx *Context) {
 	x := 0
-	for i, tab := range strip.Tabs {
+	strip.parent.m.Lock()
+	for i, tab := range strip.tabs {
 		uiConfig := strip.uiConfig
 		if tab.uiConf != nil {
 			uiConfig = tab.uiConf
 		}
 		style := uiConfig.GetStyle(config.STYLE_TAB)
-		if strip.Selected == i {
+		if strip.curIndex == i {
 			style = uiConfig.GetStyleSelected(config.STYLE_TAB)
 		}
 		tabWidth := 32
@@ -311,69 +360,69 @@ func (strip *TabStrip) Draw(ctx *Context) {
 			break
 		}
 	}
+	strip.parent.m.Unlock()
 	ctx.Fill(x, 0, ctx.Width()-x, 1, ' ',
 		strip.uiConfig.GetStyle(config.STYLE_TAB))
 }
 
 func (strip *TabStrip) Invalidate() {
-	if strip.onInvalidateStrip != nil {
-		strip.onInvalidateStrip(strip)
-	}
+	Invalidate()
 }
 
 func (strip *TabStrip) MouseEvent(localX int, localY int, event tcell.Event) {
+	strip.parent.m.Lock()
+	defer strip.parent.m.Unlock()
 	changeFocus := func(focus bool) {
-		interactive, ok := strip.parent.Tabs[strip.parent.Selected].Content.(Interactive)
+		interactive, ok := strip.parent.tabs[strip.parent.curIndex].Content.(Interactive)
 		if ok {
 			interactive.Focus(focus)
 		}
 	}
 	unfocus := func() { changeFocus(false) }
 	refocus := func() { changeFocus(true) }
-	switch event := event.(type) {
-	case *tcell.EventMouse:
+	if event, ok := event.(*tcell.EventMouse); ok {
 		switch event.Buttons() {
 		case tcell.Button1:
-			selectedTab, ok := strip.Clicked(localX, localY)
-			if !ok || selectedTab == strip.parent.Selected {
+			selectedTab, ok := strip.clicked(localX, localY)
+			if !ok || selectedTab == strip.parent.curIndex {
 				return
 			}
 			unfocus()
-			strip.parent.Select(selectedTab)
+			strip.parent.selectPriv(selectedTab)
 			refocus()
 		case tcell.WheelDown:
 			unfocus()
-			strip.parent.NextTab()
+			index := strip.parent.curIndex + 1
+			if index >= len(strip.parent.tabs) {
+				index = 0
+			}
+			strip.parent.selectPriv(index)
 			refocus()
 		case tcell.WheelUp:
 			unfocus()
-			strip.parent.PrevTab()
+			index := strip.parent.curIndex - 1
+			if index < 0 {
+				index = len(strip.parent.tabs) - 1
+			}
+			strip.parent.selectPriv(index)
 			refocus()
 		case tcell.Button3:
-			selectedTab, ok := strip.Clicked(localX, localY)
+			selectedTab, ok := strip.clicked(localX, localY)
 			if !ok {
 				return
 			}
 			unfocus()
-			if selectedTab == strip.parent.Selected {
-				strip.parent.CloseTab(selectedTab)
-			} else {
-				current := strip.parent.Selected
-				strip.parent.CloseTab(selectedTab)
-				strip.parent.Select(current)
-			}
+			strip.parent.m.Unlock()
+			strip.parent.CloseTab(selectedTab)
+			strip.parent.m.Lock()
 			refocus()
 		}
 	}
 }
 
-func (strip *TabStrip) OnInvalidate(onInvalidate func(d Drawable)) {
-	strip.onInvalidateStrip = onInvalidate
-}
-
-func (strip *TabStrip) Clicked(mouseX int, mouseY int) (int, bool) {
+func (strip *TabStrip) clicked(mouseX int, mouseY int) (int, bool) {
 	x := 0
-	for i, tab := range strip.Tabs {
+	for i, tab := range strip.tabs {
 		trunc := runewidth.Truncate(tab.Name, 32, "…")
 		length := len(trunc) + 2
 		if x <= mouseX && mouseX < x+length {
@@ -385,41 +434,37 @@ func (strip *TabStrip) Clicked(mouseX int, mouseY int) (int, bool) {
 }
 
 func (content *TabContent) Children() []Drawable {
-	children := make([]Drawable, len(content.Tabs))
-	for i, tab := range content.Tabs {
+	content.parent.m.Lock()
+	children := make([]Drawable, len(content.tabs))
+	for i, tab := range content.tabs {
 		children[i] = tab.Content
 	}
+	content.parent.m.Unlock()
 	return children
 }
 
 func (content *TabContent) Draw(ctx *Context) {
-	if content.Selected >= len(content.Tabs) {
+	content.parent.m.Lock()
+	if content.curIndex >= len(content.tabs) {
 		width := ctx.Width()
 		height := ctx.Height()
 		ctx.Fill(0, 0, width, height, ' ',
 			content.uiConfig.GetStyle(config.STYLE_TAB))
 	}
-
-	tab := content.Tabs[content.Selected]
+	tab := content.tabs[content.curIndex]
+	content.parent.m.Unlock()
 	tab.Content.Draw(ctx)
 }
 
 func (content *TabContent) MouseEvent(localX int, localY int, event tcell.Event) {
-	tab := content.Tabs[content.Selected]
-	switch tabContent := tab.Content.(type) {
-	case Mouseable:
+	content.parent.m.Lock()
+	tab := content.tabs[content.curIndex]
+	content.parent.m.Unlock()
+	if tabContent, ok := tab.Content.(Mouseable); ok {
 		tabContent.MouseEvent(localX, localY, event)
 	}
 }
 
 func (content *TabContent) Invalidate() {
-	if content.onInvalidateContent != nil {
-		content.onInvalidateContent(content)
-	}
-	tab := content.Tabs[content.Selected]
-	tab.Content.Invalidate()
-}
-
-func (content *TabContent) OnInvalidate(onInvalidate func(d Drawable)) {
-	content.onInvalidateContent = onInvalidate
+	Invalidate()
 }
