@@ -4,6 +4,7 @@ import (
 	"errors"
 	"fmt"
 	"io"
+	"strings"
 	"time"
 
 	"github.com/emersion/go-message/mail"
@@ -70,6 +71,7 @@ type MessageInfo struct {
 	Labels        []string
 	InternalDate  time.Time
 	RFC822Headers *mail.Header
+	Refs          []string
 	Size          uint32
 	Uid           uint32
 	Error         error
@@ -89,6 +91,9 @@ func (mi *MessageInfo) InReplyTo() (msgid string, err error) {
 	if mi == nil {
 		return "", errors.New("msg is nil")
 	}
+	if mi.Envelope != nil && mi.Envelope.InReplyTo != "" {
+		return mi.Envelope.InReplyTo, nil
+	}
 	if mi.RFC822Headers == nil {
 		return "", errors.New("header is nil")
 	}
@@ -106,6 +111,9 @@ func (mi *MessageInfo) References() ([]string, error) {
 	if mi == nil {
 		return []string{}, errors.New("msg is nil")
 	}
+	if mi.Refs != nil {
+		return mi.Refs, nil
+	}
 	if mi.RFC822Headers == nil {
 		return []string{}, errors.New("header is nil")
 	}
@@ -142,7 +150,7 @@ type BodyStructure struct {
 	DispositionParams map[string]string
 }
 
-//PartAtIndex returns the BodyStructure at the requested index
+// PartAtIndex returns the BodyStructure at the requested index
 func (bs *BodyStructure) PartAtIndex(index []int) (*BodyStructure, error) {
 	if len(index) == 0 {
 		return bs, nil
@@ -175,6 +183,21 @@ func (bs *BodyStructure) PartAtIndex(index []int) (*BodyStructure, error) {
 	return bs.Parts[curidx].PartAtIndex(rest)
 }
 
+func (bs *BodyStructure) FullMIMEType() string {
+	mime := fmt.Sprintf("%s/%s", bs.MIMEType, bs.MIMESubType)
+	return strings.ToLower(mime)
+}
+
+func (bs *BodyStructure) FileName() string {
+	if filename, ok := bs.DispositionParams["filename"]; ok {
+		return filename
+	} else if filename, ok := bs.Params["name"]; ok {
+		// workaround golang not supporting RFC2231 besides ASCII and UTF8
+		return filename
+	}
+	return ""
+}
+
 type Envelope struct {
 	Date      time.Time
 	Subject   string
@@ -184,6 +207,7 @@ type Envelope struct {
 	Cc        []*mail.Address
 	Bcc       []*mail.Address
 	MessageId string
+	InReplyTo string
 }
 
 // OriginalMail is helper struct used for reply/forward
