@@ -5,6 +5,7 @@ import (
 	"fmt"
 	"io"
 	"net/url"
+	"os/exec"
 	"sort"
 	"strings"
 	"time"
@@ -18,8 +19,9 @@ import (
 	"git.sr.ht/~rjarry/aerc/lib"
 	"git.sr.ht/~rjarry/aerc/lib/crypto"
 	"git.sr.ht/~rjarry/aerc/lib/ui"
-	"git.sr.ht/~rjarry/aerc/logging"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
+	"git.sr.ht/~rjarry/aerc/worker/types"
 )
 
 type Aerc struct {
@@ -27,12 +29,12 @@ type Aerc struct {
 	cmd         func(cmd []string) error
 	cmdHistory  lib.History
 	complete    func(cmd string) []string
-	conf        *config.AercConfig
 	focused     ui.Interactive
 	grid        *ui.Grid
 	simulating  int
 	statusbar   *ui.Stack
 	statusline  *StatusLine
+	pasting     bool
 	pendingKeys []config.KeyStroke
 	prompts     *ui.Stack
 	tabs        *ui.Tabs
@@ -49,15 +51,15 @@ type Choice struct {
 	Command []string
 }
 
-func NewAerc(conf *config.AercConfig,
+func NewAerc(
 	crypto crypto.Provider, cmd func(cmd []string) error,
 	complete func(cmd string) []string, cmdHistory lib.History,
 	deferLoop chan struct{},
 ) *Aerc {
-	tabs := ui.NewTabs(&conf.Ui)
+	tabs := ui.NewTabs(config.Ui)
 
-	statusbar := ui.NewStack(conf.Ui)
-	statusline := NewStatusLine(conf.Ui)
+	statusbar := ui.NewStack(config.Ui)
+	statusline := NewStatusLine(config.Ui)
 	statusbar.Push(statusline)
 
 	grid := ui.NewGrid().Rows([]ui.GridSpec{
@@ -73,34 +75,32 @@ func NewAerc(conf *config.AercConfig,
 
 	aerc := &Aerc{
 		accounts:   make(map[string]*AccountView),
-		conf:       conf,
 		cmd:        cmd,
 		cmdHistory: cmdHistory,
 		complete:   complete,
 		grid:       grid,
 		statusbar:  statusbar,
 		statusline: statusline,
-		prompts:    ui.NewStack(conf.Ui),
+		prompts:    ui.NewStack(config.Ui),
 		tabs:       tabs,
 		Crypto:     crypto,
 	}
 
 	statusline.SetAerc(aerc)
-	conf.Triggers.ExecuteCommand = cmd
+	config.Triggers.ExecuteCommand = cmd
 
-	for i, acct := range conf.Accounts {
-		view, err := NewAccountView(aerc, conf, &conf.Accounts[i], aerc, deferLoop)
+	for _, acct := range config.Accounts {
+		view, err := NewAccountView(aerc, acct, aerc, deferLoop)
 		if err != nil {
-			tabs.Add(errorScreen(err.Error(), conf.Ui), acct.Name, nil)
+			tabs.Add(errorScreen(err.Error()), acct.Name, nil)
 		} else {
 			aerc.accounts[acct.Name] = view
-			conf := view.UiConfig()
-			tabs.Add(view, acct.Name, conf)
+			tabs.Add(view, acct.Name, view.UiConfig())
 		}
 	}
 
-	if len(conf.Accounts) == 0 {
-		wizard := NewAccountWizard(aerc.Config(), aerc)
+	if len(config.Accounts) == 0 {
+		wizard := NewAccountWizard(aerc)
 		wizard.Focus(true)
 		aerc.NewTab(wizard, "New account")
 	}
@@ -136,22 +136,30 @@ func (aerc *Aerc) OnBeep(f func() error) {
 
 func (aerc *Aerc) Beep() {
 	if aerc.beep == nil {
-		logging.Warnf("should beep, but no beeper")
+		log.Warnf("should beep, but no beeper")
 		return
 	}
 	if err := aerc.beep(); err != nil {
-		logging.Errorf("tried to beep, but could not: %v", err)
+		log.Errorf("tried to beep, but could not: %v", err)
 	}
 }
 
-func (aerc *Aerc) Tick() bool {
-	more := false
-	for _, acct := range aerc.accounts {
-		more = acct.Tick() || more
+func (aerc *Aerc) HandleMessage(msg types.WorkerMessage) {
+	if acct, ok := aerc.accounts[msg.Account()]; ok {
+		acct.onMessage(msg)
 	}
+}
 
+func (aerc *Aerc) Invalidate() {
+	ui.Invalidate()
+}
+
+func (aerc *Aerc) Focus(focus bool) {
+	// who cares
+}
+
+func (aerc *Aerc) Draw(ctx *ui.Context) {
 	if len(aerc.prompts.Children()) > 0 {
-		more = true
 		previous := aerc.focused
 		prompt := aerc.prompts.Pop().(*ExLine)
 		prompt.finish = func() {
@@ -162,29 +170,6 @@ func (aerc *Aerc) Tick() bool {
 		aerc.statusbar.Push(prompt)
 		aerc.focus(prompt)
 	}
-
-	return more
-}
-
-func (aerc *Aerc) Children() []ui.Drawable {
-	return aerc.grid.Children()
-}
-
-func (aerc *Aerc) OnInvalidate(onInvalidate func(d ui.Drawable)) {
-	aerc.grid.OnInvalidate(func(_ ui.Drawable) {
-		onInvalidate(aerc)
-	})
-}
-
-func (aerc *Aerc) Invalidate() {
-	aerc.grid.Invalidate()
-}
-
-func (aerc *Aerc) Focus(focus bool) {
-	// who cares
-}
-
-func (aerc *Aerc) Draw(ctx *ui.Context) {
 	aerc.grid.Draw(ctx)
 	if aerc.dialog != nil {
 		if w, h := ctx.Width(), ctx.Height(); w > 8 && h > 4 {
@@ -214,8 +199,8 @@ func (aerc *Aerc) HumanReadableBindings() []string {
 			format(config.FormatKeyStrokes(bind.Output)),
 		))
 	}
-	if binds.Globals && aerc.conf.Bindings.Global != nil {
-		for _, bind := range aerc.conf.Bindings.Global.Bindings {
+	if binds.Globals && config.Binds.Global != nil {
+		for _, bind := range config.Binds.Global.Bindings {
 			result = append(result, fmt.Sprintf(fmtStr+" (Globals)",
 				format(config.FormatKeyStrokes(bind.Input)),
 				format(config.FormatKeyStrokes(bind.Output)),
@@ -241,30 +226,35 @@ func (aerc *Aerc) getBindings() *config.KeyBindings {
 	}
 	switch view := aerc.SelectedTabContent().(type) {
 	case *AccountView:
-		binds := aerc.conf.MergeContextualBinds(aerc.conf.Bindings.MessageList, config.BIND_CONTEXT_ACCOUNT, selectedAccountName, "messages")
-		return aerc.conf.MergeContextualBinds(binds, config.BIND_CONTEXT_FOLDER, view.SelectedDirectory(), "messages")
+		binds := config.Binds.MessageList.ForAccount(selectedAccountName)
+		return binds.ForFolder(view.SelectedDirectory())
 	case *AccountWizard:
-		return aerc.conf.Bindings.AccountWizard
+		return config.Binds.AccountWizard
 	case *Composer:
 		switch view.Bindings() {
 		case "compose::editor":
-			return aerc.conf.MergeContextualBinds(aerc.conf.Bindings.ComposeEditor, config.BIND_CONTEXT_ACCOUNT, selectedAccountName, "compose::editor")
+			return config.Binds.ComposeEditor.ForAccount(
+				selectedAccountName)
 		case "compose::review":
-			return aerc.conf.MergeContextualBinds(aerc.conf.Bindings.ComposeReview, config.BIND_CONTEXT_ACCOUNT, selectedAccountName, "compose::review")
+			return config.Binds.ComposeReview.ForAccount(
+				selectedAccountName)
 		default:
-			return aerc.conf.MergeContextualBinds(aerc.conf.Bindings.Compose, config.BIND_CONTEXT_ACCOUNT, selectedAccountName, "compose")
+			return config.Binds.Compose.ForAccount(
+				selectedAccountName)
 		}
 	case *MessageViewer:
 		switch view.Bindings() {
 		case "view::passthrough":
-			return aerc.conf.MergeContextualBinds(aerc.conf.Bindings.MessageViewPassthrough, config.BIND_CONTEXT_ACCOUNT, selectedAccountName, "view::passthrough")
+			return config.Binds.MessageViewPassthrough.ForAccount(
+				selectedAccountName)
 		default:
-			return aerc.conf.MergeContextualBinds(aerc.conf.Bindings.MessageView, config.BIND_CONTEXT_ACCOUNT, selectedAccountName, "view")
+			return config.Binds.MessageView.ForAccount(
+				selectedAccountName)
 		}
 	case *Terminal:
-		return aerc.conf.Bindings.Terminal
+		return config.Binds.Terminal
 	default:
-		return aerc.conf.Bindings.Global
+		return config.Binds.Global
 	}
 }
 
@@ -277,6 +267,14 @@ func (aerc *Aerc) simulate(strokes []config.KeyStroke) {
 		aerc.Event(simulated)
 	}
 	aerc.simulating -= 1
+	// If we are still focused on the exline, turn on tab complete
+	if exline, ok := aerc.focused.(*ExLine); ok {
+		exline.TabComplete(func(cmd string) ([]string, string) {
+			return aerc.complete(cmd), ""
+		})
+		// send tab to text input to trigger completion
+		exline.Event(tcell.NewEventKey(tcell.KeyTab, 0, tcell.ModNone))
+	}
 }
 
 func (aerc *Aerc) Event(event tcell.Event) bool {
@@ -290,13 +288,22 @@ func (aerc *Aerc) Event(event tcell.Event) bool {
 
 	switch event := event.(type) {
 	case *tcell.EventKey:
+		// If we are in a bracketed paste, don't process the keys for
+		// bindings
+		if aerc.pasting {
+			interactive, ok := aerc.SelectedTabContent().(ui.Interactive)
+			if ok {
+				return interactive.Event(event)
+			}
+			return false
+		}
 		aerc.statusline.Expire()
 		aerc.pendingKeys = append(aerc.pendingKeys, config.KeyStroke{
 			Modifiers: event.Modifiers(),
 			Key:       event.Key(),
 			Rune:      event.Rune(),
 		})
-		aerc.statusline.Invalidate()
+		ui.Invalidate()
 		bindings := aerc.getBindings()
 		incomplete := false
 		result, strokes := bindings.GetBinding(aerc.pendingKeys)
@@ -309,8 +316,7 @@ func (aerc *Aerc) Event(event tcell.Event) bool {
 		case config.BINDING_NOT_FOUND:
 		}
 		if bindings.Globals {
-			result, strokes = aerc.conf.Bindings.Global.
-				GetBinding(aerc.pendingKeys)
+			result, strokes = config.Binds.Global.GetBinding(aerc.pendingKeys)
 			switch result {
 			case config.BINDING_FOUND:
 				aerc.simulate(strokes)
@@ -325,9 +331,9 @@ func (aerc *Aerc) Event(event tcell.Event) bool {
 			exKey := bindings.ExKey
 			if aerc.simulating > 0 {
 				// Keybindings still use : even if you change the ex key
-				exKey = aerc.conf.Bindings.Global.ExKey
+				exKey = config.Binds.Global.ExKey
 			}
-			if event.Key() == exKey.Key && event.Rune() == exKey.Rune {
+			if aerc.isExKey(event, exKey) {
 				aerc.BeginExCommand("")
 				return true
 			}
@@ -338,24 +344,80 @@ func (aerc *Aerc) Event(event tcell.Event) bool {
 			return false
 		}
 	case *tcell.EventMouse:
-		if event.Buttons() == tcell.ButtonNone {
-			return false
-		}
 		x, y := event.Position()
 		aerc.grid.MouseEvent(x, y, event)
 		return true
+	case *tcell.EventPaste:
+		if event.Start() {
+			aerc.pasting = true
+		}
+		if event.End() {
+			aerc.pasting = false
+		}
+		interactive, ok := aerc.SelectedTabContent().(ui.Interactive)
+		if ok {
+			return interactive.Event(event)
+		}
+		return false
 	}
 	return false
 }
 
-func (aerc *Aerc) Config() *config.AercConfig {
-	return aerc.conf
-}
-
 func (aerc *Aerc) SelectedAccount() *AccountView {
 	return aerc.account(aerc.SelectedTabContent())
 }
 
+func (aerc *Aerc) Account(name string) (*AccountView, error) {
+	if acct, ok := aerc.accounts[name]; ok {
+		return acct, nil
+	}
+	return nil, fmt.Errorf("account <%s> not found", name)
+}
+
+func (aerc *Aerc) PrevAccount() (*AccountView, error) {
+	cur := aerc.SelectedAccount()
+	if cur == nil {
+		return nil, fmt.Errorf("no account selected, cannot get prev")
+	}
+	for i, conf := range config.Accounts {
+		if conf.Name == cur.Name() {
+			i -= 1
+			if i == -1 {
+				i = len(config.Accounts) - 1
+			}
+			conf = config.Accounts[i]
+			return aerc.Account(conf.Name)
+		}
+	}
+	return nil, fmt.Errorf("no prev account")
+}
+
+func (aerc *Aerc) NextAccount() (*AccountView, error) {
+	cur := aerc.SelectedAccount()
+	if cur == nil {
+		return nil, fmt.Errorf("no account selected, cannot get next")
+	}
+	for i, conf := range config.Accounts {
+		if conf.Name == cur.Name() {
+			i += 1
+			if i == len(config.Accounts) {
+				i = 0
+			}
+			conf = config.Accounts[i]
+			return aerc.Account(conf.Name)
+		}
+	}
+	return nil, fmt.Errorf("no next account")
+}
+
+func (aerc *Aerc) AccountNames() []string {
+	results := make([]string, 0)
+	for name := range aerc.accounts {
+		results = append(results, name)
+	}
+	return results
+}
+
 func (aerc *Aerc) account(d ui.Drawable) *AccountView {
 	switch tab := d.(type) {
 	case *AccountView:
@@ -371,7 +433,7 @@ func (aerc *Aerc) account(d ui.Drawable) *AccountView {
 func (aerc *Aerc) SelectedAccountUiConfig() *config.UIConfig {
 	acct := aerc.SelectedAccount()
 	if acct == nil {
-		return &aerc.conf.Ui
+		return config.Ui
 	}
 	return acct.UiConfig()
 }
@@ -389,10 +451,9 @@ func (aerc *Aerc) SelectedTab() *ui.Tab {
 }
 
 func (aerc *Aerc) NewTab(clickable ui.Drawable, name string) *ui.Tab {
-	var uiConf *config.UIConfig = nil
+	uiConf := config.Ui
 	if acct := aerc.account(clickable); acct != nil {
-		conf := acct.UiConfig()
-		uiConf = conf
+		uiConf = acct.UiConfig()
 	}
 	tab := aerc.tabs.Add(clickable, name, uiConf)
 	aerc.UpdateStatus()
@@ -480,6 +541,10 @@ func (aerc *Aerc) PushError(text string) *StatusMessage {
 	return aerc.statusline.PushError(text)
 }
 
+func (aerc *Aerc) PushWarning(text string) *StatusMessage {
+	return aerc.statusline.PushWarning(text)
+}
+
 func (aerc *Aerc) PushSuccess(text string) *StatusMessage {
 	return aerc.statusline.PushSuccess(text)
 }
@@ -505,7 +570,16 @@ func (aerc *Aerc) focus(item ui.Interactive) {
 
 func (aerc *Aerc) BeginExCommand(cmd string) {
 	previous := aerc.focused
-	exline := NewExLine(aerc.conf, cmd, func(cmd string) {
+	var tabComplete func(string) ([]string, string)
+	if aerc.simulating != 0 {
+		// Don't try to draw completions for simulated events
+		tabComplete = nil
+	} else {
+		tabComplete = func(cmd string) ([]string, string) {
+			return aerc.complete(cmd), ""
+		}
+	}
+	exline := NewExLine(cmd, func(cmd string) {
 		parts, err := shlex.Split(cmd)
 		if err != nil {
 			aerc.PushError(err.Error())
@@ -522,15 +596,17 @@ func (aerc *Aerc) BeginExCommand(cmd string) {
 	}, func() {
 		aerc.statusbar.Pop()
 		aerc.focus(previous)
-	}, func(cmd string) ([]string, string) {
-		return aerc.complete(cmd), ""
-	}, aerc.cmdHistory)
+	}, tabComplete, aerc.cmdHistory)
 	aerc.statusbar.Push(exline)
 	aerc.focus(exline)
 }
 
+func (aerc *Aerc) PushPrompt(prompt *ExLine) {
+	aerc.prompts.Push(prompt)
+}
+
 func (aerc *Aerc) RegisterPrompt(prompt string, cmd []string) {
-	p := NewPrompt(aerc.conf, prompt, func(text string) {
+	p := NewPrompt(prompt, func(text string) {
 		if text != "" {
 			cmd = append(cmd, text)
 		}
@@ -556,7 +632,7 @@ func (aerc *Aerc) RegisterChoices(choices []Choice) {
 		cmds[c.Key] = c.Command
 	}
 	prompt := strings.Join(texts, ", ") + "? "
-	p := NewPrompt(aerc.conf, prompt, func(text string) {
+	p := NewPrompt(prompt, func(text string) {
 		cmd, ok := cmds[text]
 		if !ok {
 			return
@@ -575,6 +651,7 @@ func (aerc *Aerc) Mailto(addr *url.URL) error {
 	var subject string
 	var body string
 	var acctName string
+	var attachments []string
 	h := &mail.Header{}
 	to, err := mail.ParseAddressList(addr.Opaque)
 	if err != nil && addr.Opaque != "" {
@@ -610,6 +687,11 @@ func (aerc *Aerc) Mailto(addr *url.URL) error {
 		case "subject":
 			subject = strings.Join(vals, ",")
 			h.SetText("Subject", subject)
+		case "attach":
+			for _, path := range vals {
+				// remove a potential file:// prefix.
+				attachments = append(attachments, strings.TrimPrefix(path, "file://"))
+			}
 		default:
 			// any other header gets ignored on purpose to avoid control headers
 			// being injected
@@ -627,7 +709,7 @@ func (aerc *Aerc) Mailto(addr *url.URL) error {
 		return errors.New("No account selected")
 	}
 
-	composer, err := NewComposer(aerc, acct, aerc.Config(),
+	composer, err := NewComposer(aerc, acct,
 		acct.AccountConfig(), acct.Worker(), "", h, models.OriginalMail{})
 	if err != nil {
 		return nil
@@ -649,8 +731,12 @@ func (aerc *Aerc) Mailto(addr *url.URL) error {
 		} else {
 			tab.Name = subject
 		}
-		tab.Content.Invalidate()
+		ui.Invalidate()
 	})
+
+	for _, file := range attachments {
+		composer.AddAttachment(file)
+	}
 	return nil
 }
 
@@ -660,7 +746,7 @@ func (aerc *Aerc) Mbox(source string) error {
 		acctConf = *selectedAcct.acct
 		info := fmt.Sprintf("Loading outgoing mbox mail settings from account [%s]", selectedAcct.Name())
 		aerc.PushStatus(info, 10*time.Second)
-		logging.Infof(info)
+		log.Debugf(info)
 	} else {
 		acctConf.From = "<user@localhost>"
 	}
@@ -671,9 +757,9 @@ func (aerc *Aerc) Mbox(source string) error {
 	acctConf.Postpone = "Drafts"
 	acctConf.CopyTo = "Sent"
 
-	mboxView, err := NewAccountView(aerc, aerc.conf, &acctConf, aerc, nil)
+	mboxView, err := NewAccountView(aerc, &acctConf, aerc, nil)
 	if err != nil {
-		aerc.NewTab(errorScreen(err.Error(), aerc.conf.Ui), acctConf.Name)
+		aerc.NewTab(errorScreen(err.Error()), acctConf.Name)
 	} else {
 		aerc.accounts[acctConf.Name] = mboxView
 		aerc.NewTab(mboxView, acctConf.Name)
@@ -692,7 +778,7 @@ func (aerc *Aerc) CloseBackends() error {
 		err := c.Close()
 		if err != nil {
 			returnErr = err
-			logging.Errorf("Closing backend failed for %s: %v", acct.Name(), err)
+			log.Errorf("Closing backend failed for %s: %v", acct.Name(), err)
 		}
 	}
 	return returnErr
@@ -700,9 +786,6 @@ func (aerc *Aerc) CloseBackends() error {
 
 func (aerc *Aerc) AddDialog(d ui.DrawableInteractive) {
 	aerc.dialog = d
-	aerc.dialog.OnInvalidate(func(_ ui.Drawable) {
-		aerc.Invalidate()
-	})
 	aerc.Invalidate()
 }
 
@@ -714,7 +797,7 @@ func (aerc *Aerc) CloseDialog() {
 func (aerc *Aerc) GetPassword(title string, prompt string) (chText chan string, chErr chan error) {
 	chText = make(chan string, 1)
 	chErr = make(chan error, 1)
-	getPasswd := NewGetPasswd(title, prompt, aerc.conf, func(pw string, err error) {
+	getPasswd := NewGetPasswd(title, prompt, func(pw string, err error) {
 		defer func() {
 			close(chErr)
 			close(chText)
@@ -743,26 +826,21 @@ func (aerc *Aerc) DecryptKeys(keys []openpgp.Key, symmetric bool) (b []byte, err
 			fmt.Sprintf("Enter password for %s (%8X)\nPress <ESC> to cancel",
 				ident.Name, key.PublicKey.KeyId))
 
-		for {
-			select {
-			case err = <-chErr:
-				if err != nil {
-					return nil, err
-				}
-				pass := <-chPass
-				err = key.PrivateKey.Decrypt([]byte(pass))
+		for err := range chErr {
+			if err != nil {
 				return nil, err
-			default:
-				aerc.ui.Tick()
 			}
+			pass := <-chPass
+			err = key.PrivateKey.Decrypt([]byte(pass))
+			return nil, err
 		}
 	}
 	return nil, err
 }
 
 // errorScreen is a widget that draws an error in the middle of the context
-func errorScreen(s string, conf config.UIConfig) ui.Drawable {
-	errstyle := conf.GetStyle(config.STYLE_ERROR)
+func errorScreen(s string) ui.Drawable {
+	errstyle := config.Ui.GetStyle(config.STYLE_ERROR)
 	text := ui.NewText(s, errstyle).Strategy(ui.TEXT_CENTER)
 	grid := ui.NewGrid().Rows([]ui.GridSpec{
 		{Strategy: ui.SIZE_WEIGHT, Size: ui.Const(1)},
@@ -776,3 +854,32 @@ func errorScreen(s string, conf config.UIConfig) ui.Drawable {
 	grid.AddChild(ui.NewFill(' ', tcell.StyleDefault)).At(2, 0)
 	return grid
 }
+
+func (aerc *Aerc) isExKey(event *tcell.EventKey, exKey config.KeyStroke) bool {
+	if event.Key() == tcell.KeyRune {
+		// Compare runes if it's a KeyRune
+		return event.Modifiers() == exKey.Modifiers && event.Rune() == exKey.Rune
+	}
+	return event.Modifiers() == exKey.Modifiers && event.Key() == exKey.Key
+}
+
+// CmdFallbackSearch checks cmds for the first executable availabe in PATH. An error is
+// returned if none are found
+func (aerc *Aerc) CmdFallbackSearch(cmds []string) (string, error) {
+	var tried []string
+	for _, cmd := range cmds {
+		if cmd == "" {
+			continue
+		}
+		params := strings.Split(cmd, " ")
+		_, err := exec.LookPath(params[0])
+		if err != nil {
+			tried = append(tried, cmd)
+			warn := fmt.Sprintf("cmd '%s' not found in PATH, using fallback", cmd)
+			aerc.PushWarning(warn)
+			continue
+		}
+		return cmd, nil
+	}
+	return "", fmt.Errorf("no command found in PATH: %s", tried)
+}
