@@ -4,11 +4,12 @@ import (
 	"bytes"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"net/textproto"
 	"os"
 	"os/exec"
+	"strconv"
 	"strings"
+	"sync"
 	"time"
 
 	"github.com/emersion/go-message/mail"
@@ -23,17 +24,18 @@ import (
 	"git.sr.ht/~rjarry/aerc/lib/format"
 	"git.sr.ht/~rjarry/aerc/lib/templates"
 	"git.sr.ht/~rjarry/aerc/lib/ui"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
 
 type Composer struct {
+	sync.Mutex
 	editors map[string]*headerEditor // indexes in lower case (from / cc / bcc)
 	header  *mail.Header
 	parent  models.OriginalMail // parent of current message, only set if reply
 
 	acctConfig *config.AccountConfig
-	config     *config.AercConfig
 	acct       *AccountView
 	aerc       *Aerc
 
@@ -62,33 +64,16 @@ type Composer struct {
 	textParts []*lib.Part
 }
 
-func NewComposer(aerc *Aerc, acct *AccountView, conf *config.AercConfig,
-	acctConfig *config.AccountConfig, worker *types.Worker, template string,
-	h *mail.Header, orig models.OriginalMail) (*Composer, error) {
-
+func NewComposer(
+	aerc *Aerc, acct *AccountView, acctConfig *config.AccountConfig,
+	worker *types.Worker, template string,
+	h *mail.Header, orig models.OriginalMail,
+) (*Composer, error) {
 	if h == nil {
 		h = new(mail.Header)
 	}
-	if fl, err := h.AddressList("from"); err != nil || fl == nil {
-		fl, err = mail.ParseAddressList(acctConfig.From)
-		// realistically this blows up way before us during the config loading
-		if err != nil {
-			return nil, err
-		}
-		if fl != nil {
-			h.SetAddressList("from", fl)
 
-		}
-	}
-
-	templateData := templates.ParseTemplateData(h, orig)
-	cmpl := completer.New(conf.Compose.AddressBookCmd, func(err error) {
-		aerc.PushError(
-			fmt.Sprintf("could not complete header: %v", err))
-		worker.Logger.Printf("could not complete header: %v", err)
-	}, aerc.Logger())
-
-	email, err := ioutil.TempFile("", "aerc-compose-*.eml")
+	email, err := os.CreateTemp("", "aerc-compose-*.eml")
 	if err != nil {
 		// TODO: handle this better
 		return nil, err
@@ -98,39 +83,126 @@ func NewComposer(aerc *Aerc, acct *AccountView, conf *config.AercConfig,
 		acct:       acct,
 		acctConfig: acctConfig,
 		aerc:       aerc,
-		config:     conf,
 		header:     h,
 		parent:     orig,
 		email:      email,
 		worker:     worker,
 		// You have to backtab to get to "From", since you usually don't edit it
 		focused:   1,
-		completer: cmpl,
+		completer: nil,
 	}
+
+	templateData := templates.ParseTemplateData(h, orig)
 	if err := c.AddTemplate(template, templateData); err != nil {
 		return nil, err
 	}
-	c.buildComposeHeader(aerc, cmpl)
-
 	c.AddSignature()
 
-	c.updateGrid()
-	c.updateCrypto()
-	c.ShowTerminal()
+	if err := c.setupFor(acct); err != nil {
+		return nil, err
+	}
 
-	if c.acctConfig.PgpAutoSign {
-		c.SetSign(true)
-	}
-	if c.acctConfig.PgpOpportunisticEncrypt {
-		c.SetEncrypt(true)
-	}
+	c.ShowTerminal()
 
 	return c, nil
 }
 
-func (c *Composer) buildComposeHeader(aerc *Aerc, cmpl *completer.Completer) {
+func (c *Composer) SwitchAccount(newAcct *AccountView) error {
+	if c.acct == newAcct {
+		log.Tracef("same accounts: no switch")
+		return nil
+	}
+	// sync the header with the editors
+	for _, editor := range c.editors {
+		editor.storeValue()
+	}
+	// ensure that from header is updated, so remove it
+	c.header.Del("from")
+	// update entire composer with new the account
+	if err := c.setupFor(newAcct); err != nil {
+		return err
+	}
+	// sync the header with the editors
+	for _, editor := range c.editors {
+		editor.loadValue()
+	}
+	c.Invalidate()
+	log.Debugf("account sucessfully switched")
+	return nil
+}
 
-	c.layout = aerc.conf.Compose.HeaderLayout
+func (c *Composer) setupFor(view *AccountView) error {
+	c.Lock()
+	defer c.Unlock()
+	// set new account
+	c.acct = view
+	c.worker = view.Worker()
+
+	// Set from header if not already in header
+	if fl, err := c.header.AddressList("from"); err != nil || fl == nil {
+		fl, err = mail.ParseAddressList(view.acct.From)
+		if err != nil {
+			return err
+		}
+		if fl != nil {
+			c.header.SetAddressList("from", fl)
+		}
+	}
+
+	// update completer
+	cmd := view.acct.AddressBookCmd
+	if cmd == "" {
+		cmd = config.Compose.AddressBookCmd
+	}
+	cmpl := completer.New(cmd, func(err error) {
+		c.aerc.PushError(
+			fmt.Sprintf("could not complete header: %v", err))
+		log.Errorf("could not complete header: %v", err)
+	})
+	c.completer = cmpl
+
+	// if editor already exists, we have to get it from the focusable slice
+	// because this will be rebuild during buildComposeHeader()
+	var focusEditor ui.MouseableDrawableInteractive
+	if c.editor != nil && len(c.focusable) > 0 {
+		focusEditor = c.focusable[len(c.focusable)-1]
+	}
+
+	// rebuild editors and focusable slice
+	c.buildComposeHeader(c.aerc, cmpl)
+
+	// restore the editor in the focusable list
+	if focusEditor != nil {
+		c.focusable = append(c.focusable, focusEditor)
+	}
+	if c.focused >= len(c.focusable) {
+		c.focused = len(c.focusable) - 1
+	}
+
+	// redraw the grid
+	c.updateGrid()
+
+	// update the crypto parts
+	c.crypto = nil
+	c.sign = false
+	if c.acct.acct.PgpAutoSign {
+		err := c.SetSign(true)
+		log.Warnf("failed to enable message signing: %v", err)
+	}
+	c.encrypt = false
+	if c.acct.acct.PgpOpportunisticEncrypt {
+		c.SetEncrypt(true)
+	}
+	err := c.updateCrypto()
+	if err != nil {
+		log.Warnf("failed to update crypto: %v", err)
+	}
+
+	return nil
+}
+
+func (c *Composer) buildComposeHeader(aerc *Aerc, cmpl *completer.Completer) {
+	c.layout = config.Compose.HeaderLayout
 	c.editors = make(map[string]*headerEditor)
 	c.focusable = make([]ui.MouseableDrawableInteractive, 0)
 	uiConfig := c.acct.UiConfig()
@@ -140,8 +212,12 @@ func (c *Composer) buildComposeHeader(aerc *Aerc, cmpl *completer.Completer) {
 			h = strings.ToLower(h)
 			c.layout[i][j] = h // normalize to lowercase
 			e := newHeaderEditor(h, c.header, uiConfig)
-			if aerc.conf.Ui.CompletionPopovers {
-				e.input.TabComplete(cmpl.ForHeader(h), uiConfig.CompletionDelay)
+			if uiConfig.CompletionPopovers {
+				e.input.TabComplete(
+					cmpl.ForHeader(h),
+					uiConfig.CompletionDelay,
+					uiConfig.CompletionMinChars,
+				)
 			}
 			c.editors[h] = e
 			switch h {
@@ -159,8 +235,12 @@ func (c *Composer) buildComposeHeader(aerc *Aerc, cmpl *completer.Completer) {
 		if c.header.Has(h) {
 			if _, ok := c.editors[h]; !ok {
 				e := newHeaderEditor(h, c.header, uiConfig)
-				if aerc.conf.Ui.CompletionPopovers {
-					e.input.TabComplete(cmpl.ForHeader(h), uiConfig.CompletionDelay)
+				if uiConfig.CompletionPopovers {
+					e.input.TabComplete(
+						cmpl.ForHeader(h),
+						uiConfig.CompletionDelay,
+						uiConfig.CompletionMinChars,
+					)
 				}
 				c.editors[h] = e
 				c.focusable = append(c.focusable, e)
@@ -193,7 +273,10 @@ func (c *Composer) SetAttachKey(attach bool) error {
 			}
 		}
 		if found {
-			c.DeleteAttachment(name)
+			err := c.DeleteAttachment(name)
+			if err != nil {
+				return fmt.Errorf("failed to delete attachment '%s: %w", name, err)
+			}
 		} else {
 			attach = !attach
 		}
@@ -221,13 +304,17 @@ func (c *Composer) SetAttachKey(attach bool) error {
 			return err
 		}
 
+		newPart, err := lib.NewPart(
+			"application/pgp-keys",
+			map[string]string{"charset": "UTF-8"},
+			r,
+		)
+		if err != nil {
+			return err
+		}
 		c.attachments = append(c.attachments,
 			lib.NewPartAttachment(
-				lib.NewPart(
-					"application/pgp-keys",
-					map[string]string{"charset": "UTF-8"},
-					r,
-				),
+				newPart,
 				c.crypto.signKey+".asc",
 			),
 		)
@@ -249,7 +336,7 @@ func (c *Composer) SetSign(sign bool) error {
 	err := c.updateCrypto()
 	if err != nil {
 		c.sign = !sign
-		return fmt.Errorf("Cannot sign message: %v", err)
+		return fmt.Errorf("Cannot sign message: %w", err)
 	}
 	return nil
 }
@@ -261,7 +348,10 @@ func (c *Composer) Sign() bool {
 func (c *Composer) SetEncrypt(encrypt bool) *Composer {
 	if !encrypt {
 		c.encrypt = encrypt
-		c.updateCrypto()
+		err := c.updateCrypto()
+		if err != nil {
+			log.Warnf("failed to update crypto: %v", err)
+		}
 		return c
 	}
 	// Check on any attempt to encrypt, and any lost focus of "to", "cc", or
@@ -334,42 +424,87 @@ func (c *Composer) updateCrypto() error {
 // Note: this does not reload the editor. You must call this before the first
 // Draw() call.
 func (c *Composer) SetContents(reader io.Reader) *Composer {
-	c.email.Seek(0, io.SeekStart)
-	io.Copy(c.email, reader)
-	c.email.Sync()
-	c.email.Seek(0, io.SeekStart)
+	_, err := c.email.Seek(0, io.SeekStart)
+	if err != nil {
+		log.Warnf("failed to seek beginning of mail: %v", err)
+	}
+	_, err = io.Copy(c.email, reader)
+	if err != nil {
+		log.Warnf("failed to copy mail: %v", err)
+	}
+	err = c.email.Sync()
+	if err != nil {
+		log.Warnf("failed to sync mail: %v", err)
+	}
+	_, err = c.email.Seek(0, io.SeekStart)
+	if err != nil {
+		log.Warnf("failed to seek beginning of mail after sync: %v", err)
+	}
 	return c
 }
 
 func (c *Composer) AppendContents(reader io.Reader) {
-	c.email.Seek(0, io.SeekEnd)
-	io.Copy(c.email, reader)
-	c.email.Sync()
+	_, err := c.email.Seek(0, io.SeekEnd)
+	if err != nil {
+		log.Warnf("failed to seek beginning of mail: %v", err)
+	}
+	_, err = io.Copy(c.email, reader)
+	if err != nil {
+		log.Warnf("failed to copy mail: %v", err)
+	}
+	err = c.email.Sync()
+	if err != nil {
+		log.Warnf("failed to sync mail: %v", err)
+	}
 }
 
 func (c *Composer) AppendPart(mimetype string, params map[string]string, body io.Reader) error {
 	if !strings.HasPrefix(mimetype, "text") {
 		return fmt.Errorf("can only append text mimetypes")
 	}
-	c.textParts = append(c.textParts, lib.NewPart(mimetype, params, body))
+	for _, part := range c.textParts {
+		if part.MimeType == mimetype {
+			return fmt.Errorf("%s part already exists", mimetype)
+		}
+	}
+	newPart, err := lib.NewPart(mimetype, params, body)
+	if err != nil {
+		return err
+	}
+	c.textParts = append(c.textParts, newPart)
 	c.resetReview()
 	return nil
 }
 
+func (c *Composer) RemovePart(mimetype string) error {
+	if mimetype == "text/plain" {
+		return fmt.Errorf("cannot remove text/plain parts")
+	}
+	for i, part := range c.textParts {
+		if part.MimeType != mimetype {
+			continue
+		}
+		c.textParts = append(c.textParts[:i], c.textParts[i+1:]...)
+		c.resetReview()
+		return nil
+	}
+	return fmt.Errorf("%s part not found", mimetype)
+}
+
 func (c *Composer) AddTemplate(template string, data interface{}) error {
 	if template == "" {
 		return nil
 	}
 
 	templateText, err := templates.ParseTemplateFromFile(
-		template, c.config.Templates.TemplateDirs, data)
+		template, config.Templates.TemplateDirs, data)
 	if err != nil {
 		return err
 	}
 
 	mr, err := mail.CreateReader(templateText)
 	if err != nil {
-		return fmt.Errorf("Template loading failed: %v", err)
+		return fmt.Errorf("Template loading failed: %w", err)
 	}
 
 	// copy the headers contained in the template to the compose headers
@@ -380,7 +515,7 @@ func (c *Composer) AddTemplate(template string, data interface{}) error {
 
 	part, err := mr.NextPart()
 	if err != nil {
-		return fmt.Errorf("Could not get body of template: %v", err)
+		return fmt.Errorf("Could not get body of template: %w", err)
 	}
 
 	c.AppendContents(part.Body)
@@ -420,7 +555,7 @@ func (c *Composer) readSignatureFromFile() []byte {
 	if err != nil {
 		return nil
 	}
-	signature, err := ioutil.ReadFile(sigFile)
+	signature, err := os.ReadFile(sigFile)
 	if err != nil {
 		c.aerc.PushError(
 			fmt.Sprintf(" Error loading signature from file: %v", sigFile))
@@ -430,6 +565,8 @@ func (c *Composer) readSignatureFromFile() []byte {
 }
 
 func (c *Composer) FocusTerminal() *Composer {
+	c.Lock()
+	defer c.Unlock()
 	if c.editor == nil {
 		return c
 	}
@@ -467,13 +604,7 @@ func (c *Composer) Draw(ctx *ui.Context) {
 }
 
 func (c *Composer) Invalidate() {
-	c.grid.Invalidate()
-}
-
-func (c *Composer) OnInvalidate(fn func(d ui.Drawable)) {
-	c.grid.OnInvalidate(func(_ ui.Drawable) {
-		fn(c)
-	})
+	ui.Invalidate()
 }
 
 func (c *Composer) Close() {
@@ -493,16 +624,21 @@ func (c *Composer) Close() {
 }
 
 func (c *Composer) Bindings() string {
-	if c.editor == nil {
+	c.Lock()
+	defer c.Unlock()
+	switch c.editor {
+	case nil:
 		return "compose::review"
-	} else if c.editor == c.focusable[c.focused] {
+	case c.focusable[c.focused]:
 		return "compose::editor"
-	} else {
+	default:
 		return "compose"
 	}
 }
 
 func (c *Composer) Event(event tcell.Event) bool {
+	c.Lock()
+	defer c.Unlock()
 	if c.editor != nil {
 		return c.focusable[c.focused].Event(event)
 	}
@@ -510,17 +646,32 @@ func (c *Composer) Event(event tcell.Event) bool {
 }
 
 func (c *Composer) MouseEvent(localX int, localY int, event tcell.Event) {
-	c.grid.MouseEvent(localX, localY, event)
+	c.Lock()
 	for _, e := range c.focusable {
 		he, ok := e.(*headerEditor)
 		if ok && he.focused {
-			c.FocusEditor(he.name)
+			he.focused = false
+		}
+	}
+	c.Unlock()
+	c.grid.MouseEvent(localX, localY, event)
+	c.Lock()
+	defer c.Unlock()
+	for i, e := range c.focusable {
+		he, ok := e.(*headerEditor)
+		if ok && he.focused {
+			c.focusable[c.focused].Focus(false)
+			c.focused = i
+			c.focusable[c.focused].Focus(true)
+			return
 		}
 	}
 }
 
 func (c *Composer) Focus(focus bool) {
+	c.Lock()
 	c.focusable[c.focused].Focus(focus)
+	c.Unlock()
 }
 
 func (c *Composer) Config() *config.AccountConfig {
@@ -535,7 +686,7 @@ func (c *Composer) Worker() *types.Worker {
 	return c.worker
 }
 
-//PrepareHeader finalizes the header, adding the value from the editors
+// PrepareHeader finalizes the header, adding the value from the editors
 func (c *Composer) PrepareHeader() (*mail.Header, error) {
 	for _, editor := range c.editors {
 		editor.storeValue()
@@ -549,9 +700,14 @@ func (c *Composer) PrepareHeader() (*mail.Header, error) {
 			return nil, err
 		}
 	}
-	if !c.header.Has("Date") {
+
+	// update the "Date" header every time PrepareHeader is called
+	if c.acctConfig.SendAsUTC {
+		c.header.SetDate(time.Now().UTC())
+	} else {
 		c.header.SetDate(time.Now())
 	}
+
 	return c.header, nil
 }
 
@@ -589,7 +745,7 @@ func getRecipientsEmail(c *Composer) ([]string, error) {
 
 	// return email addresses as string slice
 	results := []string{}
-	for email, _ := range rcpts {
+	for email := range rcpts {
 		results = append(results, email)
 	}
 	return results, nil
@@ -645,7 +801,10 @@ func (c *Composer) WriteMessage(header *mail.Header, writer io.Writer) error {
 		if err != nil {
 			return err
 		}
-		io.Copy(writer, &buf)
+		_, err = io.Copy(writer, &buf)
+		if err != nil {
+			return fmt.Errorf("failed to write message: %w", err)
+		}
 		return nil
 
 	} else {
@@ -653,6 +812,26 @@ func (c *Composer) WriteMessage(header *mail.Header, writer io.Writer) error {
 	}
 }
 
+func (c *Composer) ShouldWarnAttachment() (bool, error) {
+	regex := config.Compose.NoAttachmentWarning
+
+	if regex == nil || len(c.attachments) > 0 {
+		return false, nil
+	}
+
+	err := c.reloadEmail()
+	if err != nil {
+		return false, errors.Wrap(err, "reloadEmail")
+	}
+
+	body, err := io.ReadAll(c.email)
+	if err != nil {
+		return false, errors.Wrap(err, "io.ReadAll")
+	}
+
+	return regex.Match(body), nil
+}
+
 func writeMsgImpl(c *Composer, header *mail.Header, writer io.Writer) error {
 	if len(c.attachments) == 0 && len(c.textParts) == 0 {
 		// no attachments
@@ -663,13 +842,15 @@ func writeMsgImpl(c *Composer, header *mail.Header, writer io.Writer) error {
 		if err != nil {
 			return errors.Wrap(err, "CreateWriter")
 		}
-		parts := []*lib.Part{
-			lib.NewPart(
-				"text/plain",
-				map[string]string{"Charset": "UTF-8"},
-				c.email,
-			),
+		newPart, err := lib.NewPart(
+			"text/plain",
+			map[string]string{"Charset": "UTF-8"},
+			c.email,
+		)
+		if err != nil {
+			return err
 		}
+		parts := []*lib.Part{newPart}
 		if err := writeMultipartBody(append(parts, c.textParts...), w); err != nil {
 			return errors.Wrap(err, "writeMultipartBody")
 		}
@@ -712,7 +893,7 @@ func writeMultipartBody(parts []*lib.Part, w *mail.Writer) error {
 			return errors.Wrap(err, "CreatePart")
 		}
 		defer bw.Close()
-		if _, err := io.Copy(bw, part.Body); err != nil {
+		if _, err := io.Copy(bw, part.NewReader()); err != nil {
 			return errors.Wrap(err, "io.Copy")
 		}
 	}
@@ -733,11 +914,18 @@ func (c *Composer) AddAttachment(path string) {
 	c.resetReview()
 }
 
-func (c *Composer) AddPartAttachment(name string, mimetype string, params map[string]string, body io.Reader) {
+func (c *Composer) AddPartAttachment(name string, mimetype string,
+	params map[string]string, body io.Reader,
+) error {
+	p, err := lib.NewPart(mimetype, params, body)
+	if err != nil {
+		return err
+	}
 	c.attachments = append(c.attachments, lib.NewPartAttachment(
-		lib.NewPart(mimetype, params, body), name,
+		p, name,
 	))
 	c.resetReview()
+	return nil
 }
 
 func (c *Composer) DeleteAttachment(name string) error {
@@ -761,10 +949,8 @@ func (c *Composer) resetReview() {
 }
 
 func (c *Composer) termEvent(event tcell.Event) bool {
-	switch event := event.(type) {
-	case *tcell.EventMouse:
-		switch event.Buttons() {
-		case tcell.Button1:
+	if event, ok := event.(*tcell.EventMouse); ok {
+		if event.Buttons() == tcell.Button1 {
 			c.FocusTerminal()
 			return true
 		}
@@ -773,6 +959,11 @@ func (c *Composer) termEvent(event tcell.Event) bool {
 }
 
 func (c *Composer) termClosed(err error) {
+	c.Lock()
+	defer c.Unlock()
+	if c.editor == nil {
+		return
+	}
 	c.grid.RemoveChild(c.editor)
 	c.review = newReviewMessage(c, err)
 	c.grid.AddChild(c.review).At(3, 0)
@@ -785,18 +976,23 @@ func (c *Composer) termClosed(err error) {
 }
 
 func (c *Composer) ShowTerminal() {
+	c.Lock()
+	defer c.Unlock()
 	if c.editor != nil {
 		return
 	}
 	if c.review != nil {
 		c.grid.RemoveChild(c.review)
 	}
-	editorName := c.config.Compose.Editor
-	if editorName == "" {
-		editorName = os.Getenv("EDITOR")
+	cmds := []string{
+		config.Compose.Editor,
+		os.Getenv("EDITOR"),
+		"vi",
+		"nano",
 	}
-	if editorName == "" {
-		editorName = "vi"
+	editorName, err := c.aerc.CmdFallbackSearch(cmds)
+	if err != nil {
+		c.acct.PushError(fmt.Errorf("could not start editor: %w", err))
 	}
 	editor := exec.Command("/bin/sh", "-c", editorName+" "+c.email.Name())
 	c.editor, _ = NewTerminal(editor) // TODO: handle error
@@ -807,6 +1003,8 @@ func (c *Composer) ShowTerminal() {
 }
 
 func (c *Composer) PrevField() {
+	c.Lock()
+	defer c.Unlock()
 	c.focusable[c.focused].Focus(false)
 	c.focused--
 	if c.focused == -1 {
@@ -816,12 +1014,20 @@ func (c *Composer) PrevField() {
 }
 
 func (c *Composer) NextField() {
+	c.Lock()
+	defer c.Unlock()
 	c.focusable[c.focused].Focus(false)
 	c.focused = (c.focused + 1) % len(c.focusable)
 	c.focusable[c.focused].Focus(true)
 }
 
 func (c *Composer) FocusEditor(editor string) {
+	c.Lock()
+	defer c.Unlock()
+	c.focusEditor(editor)
+}
+
+func (c *Composer) focusEditor(editor string) {
 	editor = strings.ToLower(editor)
 	c.focusable[c.focused].Focus(false)
 	for i, f := range c.focusable {
@@ -836,6 +1042,8 @@ func (c *Composer) FocusEditor(editor string) {
 
 // AddEditor appends a new header editor to the compose window.
 func (c *Composer) AddEditor(header string, value string, appendHeader bool) {
+	c.Lock()
+	defer c.Unlock()
 	var editor *headerEditor
 	header = strings.ToLower(header)
 	if e, ok := c.editors[header]; ok {
@@ -845,7 +1053,11 @@ func (c *Composer) AddEditor(header string, value string, appendHeader bool) {
 		uiConfig := c.acct.UiConfig()
 		e := newHeaderEditor(header, c.header, uiConfig)
 		if uiConfig.CompletionPopovers {
-			e.input.TabComplete(c.completer.ForHeader(header), uiConfig.CompletionDelay)
+			e.input.TabComplete(
+				c.completer.ForHeader(header),
+				uiConfig.CompletionDelay,
+				uiConfig.CompletionMinChars,
+			)
 		}
 		c.editors[header] = e
 		c.layout = append(c.layout, []string{header})
@@ -869,7 +1081,7 @@ func (c *Composer) AddEditor(header string, value string, appendHeader bool) {
 		editor.storeValue()
 	}
 	if value == "" {
-		c.FocusEditor(c.editors[header].name)
+		c.focusEditor(c.editors[header].name)
 	}
 	c.updateGrid()
 }
@@ -928,7 +1140,8 @@ type headerEditor struct {
 }
 
 func newHeaderEditor(name string, h *mail.Header,
-	uiConfig *config.UIConfig) *headerEditor {
+	uiConfig *config.UIConfig,
+) *headerEditor {
 	he := &headerEditor{
 		input:    ui.NewTextInput("", uiConfig),
 		name:     name,
@@ -939,8 +1152,8 @@ func newHeaderEditor(name string, h *mail.Header,
 	return he
 }
 
-//extractHumanHeaderValue extracts the human readable string for key from the
-//header. If a parsing error occurs the raw value is returned
+// extractHumanHeaderValue extracts the human readable string for key from the
+// header. If a parsing error occurs the raw value is returned
 func extractHumanHeaderValue(key string, h *mail.Header) string {
 	var val string
 	var err error
@@ -959,16 +1172,16 @@ func extractHumanHeaderValue(key string, h *mail.Header) string {
 	return val
 }
 
-//loadValue loads the value of he.name form the underlying header
-//the value is decoded and meant for human consumption.
-//decoding issues are ignored and return their raw values
+// loadValue loads the value of he.name form the underlying header
+// the value is decoded and meant for human consumption.
+// decoding issues are ignored and return their raw values
 func (he *headerEditor) loadValue() {
 	he.input.Set(extractHumanHeaderValue(he.name, he.header))
-	he.input.Invalidate()
+	ui.Invalidate()
 }
 
-//storeValue writes the current state back to the underlying header.
-//errors are ignored
+// storeValue writes the current state back to the underlying header.
+// errors are ignored
 func (he *headerEditor) storeValue() {
 	val := he.input.String()
 	switch strings.ToLower(he.name) {
@@ -991,13 +1204,6 @@ func (he *headerEditor) storeValue() {
 	}
 }
 
-//setValue overwrites the current value of the header editor and flushes it
-//to the underlying header
-func (he *headerEditor) setValue(val string) {
-	he.input.Set(val)
-	he.storeValue()
-}
-
 func (he *headerEditor) Draw(ctx *ui.Context) {
 	name := textproto.CanonicalMIMEHeaderKey(he.name)
 	// Extra character to put a blank cell between the header and the input
@@ -1010,10 +1216,8 @@ func (he *headerEditor) Draw(ctx *ui.Context) {
 }
 
 func (he *headerEditor) MouseEvent(localX int, localY int, event tcell.Event) {
-	switch event := event.(type) {
-	case *tcell.EventMouse:
-		switch event.Buttons() {
-		case tcell.Button1:
+	if event, ok := event.(*tcell.EventMouse); ok {
+		if event.Buttons() == tcell.Button1 {
 			he.focused = true
 		}
 
@@ -1025,13 +1229,7 @@ func (he *headerEditor) MouseEvent(localX int, localY int, event tcell.Event) {
 }
 
 func (he *headerEditor) Invalidate() {
-	he.input.Invalidate()
-}
-
-func (he *headerEditor) OnInvalidate(fn func(ui.Drawable)) {
-	he.input.OnInvalidate(func(_ ui.Drawable) {
-		fn(he)
-	})
+	ui.Invalidate()
 }
 
 func (he *headerEditor) Focus(focused bool) {
@@ -1060,36 +1258,61 @@ type reviewMessage struct {
 	grid     *ui.Grid
 }
 
-var reviewCommands = [][]string{
-	{":send<enter>", "Send"},
-	{":edit<enter>", "Edit"},
-	{":attach<space>", "Add attachment"},
-	{":detach<space>", "Remove attachment"},
-	{":postpone<enter>", "Postpone"},
-	{":abort<enter>", "Abort (discard message, no confirmation)"},
-	{":choose -o d discard abort -o p postpone postpone<enter>", "Abort or postpone"},
-}
-
 func newReviewMessage(composer *Composer, err error) *reviewMessage {
-	bindings := composer.config.MergeContextualBinds(
-		composer.config.Bindings.ComposeReview,
-		config.BIND_CONTEXT_ACCOUNT,
+	bindings := config.Binds.ComposeReview.ForAccount(
 		composer.acctConfig.Name,
-		"compose::review",
 	)
 
+	reviewCommands := [][]string{
+		{":send<enter>", "Send", ""},
+		{":edit<enter>", "Edit", ""},
+		{":attach<space>", "Add attachment", ""},
+		{":detach<space>", "Remove attachment", ""},
+		{":postpone<enter>", "Postpone", ""},
+		{":preview<enter>", "Preview message", ""},
+		{":abort<enter>", "Abort (discard message, no confirmation)", ""},
+		{":choose -o d discard abort -o p postpone postpone<enter>", "Abort or postpone", ""},
+	}
 	var actions []string
+	for _, binding := range bindings.Bindings {
+		inputs := config.FormatKeyStrokes(binding.Input)
+		outputs := config.FormatKeyStrokes(binding.Output)
+		outputs = strings.ReplaceAll(outputs, "<space>", " ")
+		found := false
+		for i, rcmd := range reviewCommands {
+			if outputs == rcmd[0] {
+				found = true
+				if reviewCommands[i][2] == "" {
+					reviewCommands[i][2] = inputs
+				} else {
+					reviewCommands[i][2] += ", " + inputs
+				}
+				break
+			}
+		}
+		if !found {
+			rcmd := []string{outputs, "", inputs}
+			reviewCommands = append(reviewCommands, rcmd)
+		}
+	}
+	longest := 0
+	for _, rcmd := range reviewCommands {
+		if len(rcmd[2]) > longest {
+			longest = len(rcmd[2])
+		}
+	}
 
-	for _, command := range reviewCommands {
-		cmd := command[0]
-		name := command[1]
-		strokes, _ := config.ParseKeyStrokes(cmd)
-		var inputs []string
-		for _, input := range bindings.GetReverseBindings(strokes) {
-			inputs = append(inputs, config.FormatKeyStrokes(input))
+	width := longest
+	if longest < 6 {
+		width = 6
+	}
+	widthstr := strconv.Itoa(width)
+
+	for _, rcmd := range reviewCommands {
+		if rcmd[2] != "" {
+			actions = append(actions, fmt.Sprintf("  %-"+widthstr+"s  %-40s  %s",
+				rcmd[2], rcmd[1], rcmd[0]))
 		}
-		actions = append(actions, fmt.Sprintf("  %-6s  %-40s  %s",
-			strings.Join(inputs[:], ", "), name, cmd))
 	}
 
 	spec := []ui.GridSpec{
@@ -1153,7 +1376,15 @@ func newReviewMessage(composer *Composer, err error) *reviewMessage {
 			grid.AddChild(ui.NewText("text/plain", uiConfig.GetStyle(config.STYLE_DEFAULT))).At(i, 0)
 			i += 1
 			for _, p := range composer.textParts {
-				grid.AddChild(ui.NewText(p.MimeType, uiConfig.GetStyle(config.STYLE_DEFAULT))).At(i, 0)
+				err := composer.updateMultipart(p)
+				if err != nil {
+					msg := fmt.Sprintf("%s error: %s", p.MimeType, err)
+					grid.AddChild(ui.NewText(msg,
+						uiConfig.GetStyle(config.STYLE_ERROR))).At(i, 0)
+				} else {
+					grid.AddChild(ui.NewText(p.MimeType,
+						uiConfig.GetStyle(config.STYLE_DEFAULT))).At(i, 0)
+				}
 				i += 1
 			}
 
@@ -1166,14 +1397,44 @@ func newReviewMessage(composer *Composer, err error) *reviewMessage {
 	}
 }
 
-func (rm *reviewMessage) Invalidate() {
-	rm.grid.Invalidate()
+func (c *Composer) updateMultipart(p *lib.Part) error {
+	command, found := config.Converters[p.MimeType]
+	if !found {
+		// unreachable
+		return fmt.Errorf("no command defined for mime/type")
+	}
+	// reset part body to avoid it leaving outdated if the command fails
+	p.Data = nil
+	err := c.reloadEmail()
+	if err != nil {
+		return errors.Wrap(err, "reloadEmail")
+	}
+	body, err := io.ReadAll(c.email)
+	if err != nil {
+		return errors.Wrap(err, "io.ReadAll")
+	}
+	cmd := exec.Command("sh", "-c", command)
+	cmd.Stdin = bytes.NewReader(body)
+	out, err := cmd.Output()
+	if err != nil {
+		var stderr string
+		var ee *exec.ExitError
+		if errors.As(err, &ee) {
+			// append the first 30 chars of stderr if any
+			stderr = strings.Trim(string(ee.Stderr), " \t\n\r")
+			stderr = strings.ReplaceAll(stderr, "\n", "; ")
+			if stderr != "" {
+				stderr = fmt.Sprintf(": %.30s", stderr)
+			}
+		}
+		return fmt.Errorf("%s: %w%s", command, err, stderr)
+	}
+	p.Data = out
+	return nil
 }
 
-func (rm *reviewMessage) OnInvalidate(fn func(ui.Drawable)) {
-	rm.grid.OnInvalidate(func(_ ui.Drawable) {
-		fn(rm)
-	})
+func (rm *reviewMessage) Invalidate() {
+	ui.Invalidate()
 }
 
 func (rm *reviewMessage) Draw(ctx *ui.Context) {
@@ -1210,13 +1471,7 @@ func (cs *cryptoStatus) Draw(ctx *ui.Context) {
 }
 
 func (cs *cryptoStatus) Invalidate() {
-	cs.status.Invalidate()
-}
-
-func (cs *cryptoStatus) OnInvalidate(fn func(ui.Drawable)) {
-	cs.status.OnInvalidate(func(_ ui.Drawable) {
-		fn(cs)
-	})
+	ui.Invalidate()
 }
 
 func (c *Composer) checkEncryptionKeys(_ string) bool {
@@ -1245,6 +1500,9 @@ func (c *Composer) checkEncryptionKeys(_ string) bool {
 	// If callbacks were registered, encrypt will be set when user removes
 	// recipients with missing keys
 	c.encrypt = true
-	c.updateCrypto()
+	err = c.updateCrypto()
+	if err != nil {
+		log.Warnf("failed update crypto: %v", err)
+	}
 	return true
 }
