@@ -17,7 +17,7 @@ import (
 	"git.sr.ht/~rjarry/aerc/lib"
 	"git.sr.ht/~rjarry/aerc/lib/format"
 	"git.sr.ht/~rjarry/aerc/lib/ui"
-	"git.sr.ht/~rjarry/aerc/logging"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
@@ -47,9 +47,7 @@ type DirectoryLister interface {
 }
 
 type DirectoryList struct {
-	ui.Invalidatable
 	Scrollable
-	aercConf         *config.AercConfig
 	acctConf         *config.AccountConfig
 	store            *lib.DirStore
 	dirs             []string
@@ -59,30 +57,22 @@ type DirectoryList struct {
 	worker           *types.Worker
 	skipSelect       context.Context
 	skipSelectCancel context.CancelFunc
-	uiConf           map[string]*config.UIConfig
 }
 
-func NewDirectoryList(conf *config.AercConfig, acctConf *config.AccountConfig,
+func NewDirectoryList(acctConf *config.AccountConfig,
 	worker *types.Worker,
 ) DirectoryLister {
 	ctx, cancel := context.WithCancel(context.Background())
 
-	uiConfMap := make(map[string]*config.UIConfig)
-
 	dirlist := &DirectoryList{
-		aercConf:         conf,
 		acctConf:         acctConf,
 		store:            lib.NewDirStore(),
 		worker:           worker,
 		skipSelect:       ctx,
 		skipSelectCancel: cancel,
-		uiConf:           uiConfMap,
 	}
 	uiConf := dirlist.UiConfig("")
 	dirlist.spinner = NewSpinner(uiConf)
-	dirlist.spinner.OnInvalidate(func(_ ui.Drawable) {
-		dirlist.Invalidate()
-	})
 	dirlist.spinner.Start()
 
 	if uiConf.DirListTree {
@@ -96,15 +86,7 @@ func (dirlist *DirectoryList) UiConfig(dir string) *config.UIConfig {
 	if dir == "" {
 		dir = dirlist.Selected()
 	}
-	if ui, ok := dirlist.uiConf[dir]; ok {
-		return ui
-	}
-	ui := dirlist.aercConf.GetUiConfig(map[config.ContextType]string{
-		config.UI_CONTEXT_ACCOUNT: dirlist.acctConf.Name,
-		config.UI_CONTEXT_FOLDER:  dir,
-	})
-	dirlist.uiConf[dir] = ui
-	return ui
+	return config.Ui.ForAccount(dirlist.acctConf.Name).ForFolder(dir)
 }
 
 func (dirlist *DirectoryList) List() []string {
@@ -131,7 +113,7 @@ func (dirlist *DirectoryList) UpdateList(done func(dirs []string)) {
 				dirlist.spinner.Stop()
 				dirlist.Invalidate()
 				if done != nil {
-					done(dirs)
+					done(dirlist.dirs)
 				}
 			}
 		})
@@ -152,18 +134,13 @@ func (dirlist *DirectoryList) Select(name string) {
 	ctx, cancel := context.WithCancel(context.Background())
 	dirlist.skipSelect = ctx
 	dirlist.skipSelectCancel = cancel
+	delay := dirlist.UiConfig(name).DirListDelay
 
 	go func(ctx context.Context) {
-		defer logging.PanicHandler()
+		defer log.PanicHandler()
 
 		select {
-		case <-time.After(dirlist.UiConfig(name).DirListDelay):
-			newStore := true
-			for _, s := range dirlist.store.List() {
-				if s == dirlist.selecting {
-					newStore = false
-				}
-			}
+		case <-time.After(delay):
 			dirlist.worker.PostAction(&types.OpenDirectory{Directory: name},
 				func(msg types.WorkerMessage) {
 					switch msg.(type) {
@@ -187,19 +164,12 @@ func (dirlist *DirectoryList) Select(name string) {
 							sort.Strings(dirlist.dirs)
 						}
 						dirlist.sortDirsByFoldersSortConfig()
-						if newStore {
-							store, ok := dirlist.MsgStore(name)
-							if ok {
-								// Fetch directory contents via store.Sort
-								store.Sort(nil, nil)
-							}
-						}
 					}
 					dirlist.Invalidate()
 				})
 			dirlist.Invalidate()
 		case <-ctx.Done():
-			logging.Debugf("dirlist: skip %s", name)
+			log.Tracef("dirlist: skip %s", name)
 			return
 		}
 	}(ctx)
@@ -210,7 +180,7 @@ func (dirlist *DirectoryList) Selected() string {
 }
 
 func (dirlist *DirectoryList) Invalidate() {
-	dirlist.DoInvalidate(dirlist)
+	ui.Invalidate()
 }
 
 func (dirlist *DirectoryList) getDirString(name string, width int, recentUnseen func() string) string {
