@@ -9,22 +9,24 @@ import (
 )
 
 type ExLine struct {
-	ui.Invalidatable
 	commit      func(cmd string)
 	finish      func()
 	tabcomplete func(cmd string) ([]string, string)
 	cmdHistory  lib.History
 	input       *ui.TextInput
-	conf        *config.AercConfig
 }
 
-func NewExLine(conf *config.AercConfig, cmd string, commit func(cmd string), finish func(),
+func NewExLine(cmd string, commit func(cmd string), finish func(),
 	tabcomplete func(cmd string) ([]string, string),
-	cmdHistory lib.History) *ExLine {
-
-	input := ui.NewTextInput("", &conf.Ui).Prompt(":").Set(cmd)
-	if conf.Ui.CompletionPopovers {
-		input.TabComplete(tabcomplete, conf.Ui.CompletionDelay)
+	cmdHistory lib.History,
+) *ExLine {
+	input := ui.NewTextInput("", config.Ui).Prompt(":").Set(cmd)
+	if config.Ui.CompletionPopovers {
+		input.TabComplete(
+			tabcomplete,
+			config.Ui.CompletionDelay,
+			config.Ui.CompletionMinChars,
+		)
 	}
 	exline := &ExLine{
 		commit:      commit,
@@ -32,20 +34,28 @@ func NewExLine(conf *config.AercConfig, cmd string, commit func(cmd string), fin
 		tabcomplete: tabcomplete,
 		cmdHistory:  cmdHistory,
 		input:       input,
-		conf:        conf,
 	}
-	input.OnInvalidate(func(d ui.Drawable) {
-		exline.Invalidate()
-	})
 	return exline
 }
 
-func NewPrompt(conf *config.AercConfig, prompt string, commit func(text string),
-	tabcomplete func(cmd string) ([]string, string)) *ExLine {
+func (x *ExLine) TabComplete(tabComplete func(string) ([]string, string)) {
+	x.input.TabComplete(
+		tabComplete,
+		config.Ui.CompletionDelay,
+		config.Ui.CompletionMinChars,
+	)
+}
 
-	input := ui.NewTextInput("", &conf.Ui).Prompt(prompt)
-	if conf.Ui.CompletionPopovers {
-		input.TabComplete(tabcomplete, conf.Ui.CompletionDelay)
+func NewPrompt(prompt string, commit func(text string),
+	tabcomplete func(cmd string) ([]string, string),
+) *ExLine {
+	input := ui.NewTextInput("", config.Ui).Prompt(prompt)
+	if config.Ui.CompletionPopovers {
+		input.TabComplete(
+			tabcomplete,
+			config.Ui.CompletionDelay,
+			config.Ui.CompletionMinChars,
+		)
 	}
 	exline := &ExLine{
 		commit:      commit,
@@ -53,14 +63,11 @@ func NewPrompt(conf *config.AercConfig, prompt string, commit func(text string),
 		cmdHistory:  &nullHistory{input: input},
 		input:       input,
 	}
-	input.OnInvalidate(func(d ui.Drawable) {
-		exline.Invalidate()
-	})
 	return exline
 }
 
 func (ex *ExLine) Invalidate() {
-	ex.DoInvalidate(ex)
+	ui.Invalidate()
 }
 
 func (ex *ExLine) Draw(ctx *ui.Context) {
@@ -72,8 +79,7 @@ func (ex *ExLine) Focus(focus bool) {
 }
 
 func (ex *ExLine) Event(event tcell.Event) bool {
-	switch event := event.(type) {
-	case *tcell.EventKey:
+	if event, ok := event.(*tcell.EventKey); ok {
 		switch event.Key() {
 		case tcell.KeyEnter, tcell.KeyCtrlJ:
 			cmd := ex.input.String()
