@@ -2,7 +2,6 @@ package widgets
 
 import (
 	"fmt"
-	"log"
 	"math"
 	"strings"
 
@@ -13,17 +12,17 @@ import (
 	"git.sr.ht/~rjarry/aerc/config"
 	"git.sr.ht/~rjarry/aerc/lib"
 	"git.sr.ht/~rjarry/aerc/lib/format"
+	"git.sr.ht/~rjarry/aerc/lib/iterator"
 	"git.sr.ht/~rjarry/aerc/lib/ui"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
 
 type MessageList struct {
-	ui.Invalidatable
 	Scrollable
-	conf          *config.AercConfig
-	logger        *log.Logger
 	height        int
+	width         int
 	nmsgs         int
 	spinner       *Spinner
 	store         *lib.MessageStore
@@ -31,28 +30,24 @@ type MessageList struct {
 	aerc          *Aerc
 }
 
-func NewMessageList(conf *config.AercConfig, logger *log.Logger, aerc *Aerc) *MessageList {
+func NewMessageList(aerc *Aerc, account *AccountView) *MessageList {
 	ml := &MessageList{
-		conf:          conf,
-		logger:        logger,
-		spinner:       NewSpinner(&conf.Ui),
+		spinner:       NewSpinner(account.uiConf),
 		isInitalizing: true,
 		aerc:          aerc,
 	}
-	ml.spinner.OnInvalidate(func(_ ui.Drawable) {
-		ml.Invalidate()
-	})
 	// TODO: stop spinner, probably
 	ml.spinner.Start()
 	return ml
 }
 
 func (ml *MessageList) Invalidate() {
-	ml.DoInvalidate(ml)
+	ui.Invalidate()
 }
 
 func (ml *MessageList) Draw(ctx *ui.Context) {
 	ml.height = ctx.Height()
+	ml.width = ctx.Width()
 	uiConfig := ml.aerc.SelectedAccountUiConfig()
 	ctx.Fill(0, 0, ctx.Width(), ctx.Height(), ' ',
 		uiConfig.GetStyle(config.STYLE_MSGLIST_DEFAULT))
@@ -72,7 +67,13 @@ func (ml *MessageList) Draw(ctx *ui.Context) {
 
 	ml.UpdateScroller(ml.height, len(store.Uids()))
 	if store := ml.Store(); store != nil && len(store.Uids()) > 0 {
-		ml.EnsureScroll(store.SelectedIndex())
+		iter := store.UidsIterator()
+		for i := 0; iter.Next(); i++ {
+			if store.SelectedUid() == iter.Value().(uint32) {
+				ml.EnsureScroll(i)
+				break
+			}
+		}
 	}
 
 	textWidth := ctx.Width()
@@ -88,67 +89,71 @@ func (ml *MessageList) Draw(ctx *ui.Context) {
 		row          int = 0
 	)
 
-	if store.ThreadedView() {
-		threads := store.Threads
-		counter := len(store.Uids())
+	createBaseCtx := func(uid uint32, row int) format.Ctx {
+		return format.Ctx{
+			FromAddress: acct.acct.From,
+			AccountName: acct.Name(),
+			MsgInfo:     store.Messages[uid],
+			MsgNum:      row,
+			MsgIsMarked: store.Marker().IsMarked(uid),
+		}
+	}
 
-		for i := len(threads) - 1; i >= 0; i-- {
-			var lastSubject string
-			threads[i].Walk(func(t *types.Thread, _ int, currentErr error) error {
-				if currentErr != nil {
-					return currentErr
-				}
-				if t.Hidden || t.Deleted {
-					return nil
-				}
-				counter--
-				if counter > len(store.Uids())-1-ml.Scroll() {
-					//skip messages which are higher than the viewport
-					return nil
-				}
-				msg := store.Messages[t.Uid]
-				var prefix string
-				var subject string
-				var normalizedSubject string
-				if msg != nil {
-					prefix = threadPrefix(t)
-					if msg.Envelope != nil {
-						subject = msg.Envelope.Subject
-						normalizedSubject, _ = sortthread.GetBaseSubject(subject)
+	if store.ThreadedView() {
+		var (
+			lastSubject string
+			prevThread  *types.Thread
+			i           int = 0
+		)
+		factory := iterator.NewFactory(!store.ReverseThreadOrder())
+	threadLoop:
+		for iter := store.ThreadsIterator(); iter.Next(); {
+			var cur []*types.Thread
+			err := iter.Value().(*types.Thread).Walk(
+				func(t *types.Thread, _ int, _ error,
+				) error {
+					if t.Hidden || t.Deleted {
+						return nil
 					}
+					cur = append(cur, t)
+					return nil
+				})
+			if err != nil {
+				log.Errorf("thread walk: %v", err)
+			}
+			for curIter := factory.NewIterator(cur); curIter.Next(); {
+				if i < ml.Scroll() {
+					i++
+					continue
 				}
-				fmtCtx := format.Ctx{
-					FromAddress:       acct.acct.From,
-					AccountName:       acct.Name(),
-					MsgInfo:           msg,
-					MsgNum:            row,
-					MsgIsMarked:       store.IsMarked(t.Uid),
-					ThreadPrefix:      prefix,
-					ThreadSameSubject: normalizedSubject == lastSubject,
+				if thread := curIter.Value().(*types.Thread); thread != nil {
+					fmtCtx := createBaseCtx(thread.Uid, row)
+					fmtCtx.ThreadPrefix = threadPrefix(thread,
+						store.ReverseThreadOrder())
+					if fmtCtx.MsgInfo != nil && fmtCtx.MsgInfo.Envelope != nil {
+						baseSubject, _ := sortthread.GetBaseSubject(
+							fmtCtx.MsgInfo.Envelope.Subject)
+						fmtCtx.ThreadSameSubject = baseSubject == lastSubject &&
+							sameParent(thread, prevThread) &&
+							!isParent(thread)
+						lastSubject = baseSubject
+						prevThread = thread
+					}
+					if ml.drawRow(textWidth, ctx, thread.Uid, row, &needsHeaders, fmtCtx) {
+						break threadLoop
+					}
+					row += 1
 				}
-				if ml.drawRow(textWidth, ctx, t.Uid, row, &needsHeaders, fmtCtx) {
-					return types.ErrSkipThread
-				}
-				lastSubject = normalizedSubject
-				row++
-				return nil
-			})
-			if row >= ctx.Height() {
-				break
 			}
 		}
 	} else {
-		uids := store.Uids()
-		for i := len(uids) - 1 - ml.Scroll(); i >= 0; i-- {
-			uid := uids[i]
-			msg := store.Messages[uid]
-			fmtCtx := format.Ctx{
-				FromAddress: acct.acct.From,
-				AccountName: acct.Name(),
-				MsgInfo:     msg,
-				MsgNum:      row,
-				MsgIsMarked: store.IsMarked(uid),
+		iter := store.UidsIterator()
+		for i := 0; iter.Next(); i++ {
+			if i < ml.Scroll() {
+				continue
 			}
+			uid := iter.Value().(uint32)
+			fmtCtx := createBaseCtx(uid, row)
 			if ml.drawRow(textWidth, ctx, uid, row, &needsHeaders, fmtCtx) {
 				break
 			}
@@ -197,19 +202,9 @@ func (ml *MessageList) drawRow(textWidth int, ctx *ui.Context, uid uint32, row i
 	// TODO deprecate subject contextual UIs? Only related setting is styleset,
 	// should implement a better per-message styling method
 	// Check if we have any applicable ContextualUIConfigs
-	confs := ml.aerc.conf.GetContextualUIConfigs()
-	uiConfig := acct.Directories().UiConfig()
-	for _, c := range confs {
-		if c.ContextType == config.UI_CONTEXT_SUBJECT && msg.Envelope != nil {
-			if c.Regex.Match([]byte(msg.Envelope.Subject)) {
-				confParams := map[config.ContextType]string{
-					config.UI_CONTEXT_ACCOUNT: acct.AccountConfig().Name,
-					config.UI_CONTEXT_FOLDER:  acct.Directories().Selected(),
-					config.UI_CONTEXT_SUBJECT: msg.Envelope.Subject,
-				}
-				uiConfig = ml.conf.GetUiConfig(confParams)
-			}
-		}
+	uiConfig := acct.Directories().UiConfig(store.DirInfo.Name)
+	if msg.Envelope != nil {
+		uiConfig = uiConfig.ForSubject(msg.Envelope.Subject)
 	}
 
 	msg_styles := []config.StyleObject{}
@@ -239,15 +234,19 @@ func (ml *MessageList) drawRow(textWidth int, ctx *ui.Context, uid uint32, row i
 	if _, ok := store.Deleted[msg.Uid]; ok {
 		msg_styles = append(msg_styles, config.STYLE_MSGLIST_DELETED)
 	}
+	// search result
+	if store.IsResult(msg.Uid) {
+		msg_styles = append(msg_styles, config.STYLE_MSGLIST_RESULT)
+	}
 
 	// marked message
-	if store.IsMarked(msg.Uid) {
+	if store.Marker().IsMarked(msg.Uid) {
 		msg_styles = append(msg_styles, config.STYLE_MSGLIST_MARKED)
 	}
 
 	var style tcell.Style
 	// current row
-	if row == ml.store.SelectedIndex()-ml.Scroll() {
+	if msg.Uid == ml.store.SelectedUid() {
 		style = uiConfig.GetComposedStyleSelected(config.STYLE_MSGLIST_DEFAULT, msg_styles)
 	} else {
 		style = uiConfig.GetComposedStyle(config.STYLE_MSGLIST_DEFAULT, msg_styles)
@@ -259,6 +258,7 @@ func (ml *MessageList) drawRow(textWidth int, ctx *ui.Context, uid uint32, row i
 		uiConfig.ThisDayTimeFormat,
 		uiConfig.ThisWeekTimeFormat,
 		uiConfig.ThisYearTimeFormat,
+		uiConfig.IconAttachment,
 		fmtCtx)
 	if err != nil {
 		ctx.Printf(0, row, style, "%v", err)
@@ -285,8 +285,7 @@ func (ml *MessageList) drawScrollbar(ctx *ui.Context) {
 }
 
 func (ml *MessageList) MouseEvent(localX int, localY int, event tcell.Event) {
-	switch event := event.(type) {
-	case *tcell.EventMouse:
+	if event, ok := event.(*tcell.EventMouse); ok {
 		switch event.Buttons() {
 		case tcell.Button1:
 			if ml.aerc == nil {
@@ -304,14 +303,14 @@ func (ml *MessageList) MouseEvent(localX int, localY int, event tcell.Event) {
 				if msg == nil {
 					return
 				}
-				lib.NewMessageStoreView(msg, store, ml.aerc.Crypto,
-					ml.aerc.DecryptKeys,
+				lib.NewMessageStoreView(msg, acct.UiConfig().AutoMarkRead,
+					store, ml.aerc.Crypto, ml.aerc.DecryptKeys,
 					func(view lib.MessageView, err error) {
 						if err != nil {
 							ml.aerc.PushError(err.Error())
 							return
 						}
-						viewer := NewMessageViewer(acct, ml.aerc.Config(), view)
+						viewer := NewMessageViewer(acct, view)
 						ml.aerc.NewTab(viewer, msg.Envelope.Subject)
 					})
 			}
@@ -341,11 +340,15 @@ func (ml *MessageList) Height() int {
 	return ml.height
 }
 
+func (ml *MessageList) Width() int {
+	return ml.width
+}
+
 func (ml *MessageList) storeUpdate(store *lib.MessageStore) {
 	if ml.Store() != store {
 		return
 	}
-	store.Reselect()
+	ml.Invalidate()
 }
 
 func (ml *MessageList) SetStore(store *lib.MessageStore) {
@@ -355,7 +358,8 @@ func (ml *MessageList) SetStore(store *lib.MessageStore) {
 	ml.store = store
 	if store != nil {
 		ml.spinner.Stop()
-		ml.nmsgs = len(store.Uids())
+		uids := store.Uids()
+		ml.nmsgs = len(uids)
 		store.OnUpdate(ml.storeUpdate)
 		store.OnFilterChange(func(store *lib.MessageStore) {
 			if ml.Store() != store {
@@ -387,8 +391,31 @@ func (ml *MessageList) Selected() *models.MessageInfo {
 }
 
 func (ml *MessageList) Select(index int) {
+	// Note that the msgstore.Select function expects a uid as argument
+	// whereas the msglist.Select expects the message number
 	store := ml.Store()
-	store.Select(index)
+	uids := store.Uids()
+	if len(uids) == 0 {
+		store.Select(lib.MagicUid)
+		return
+	}
+
+	iter := store.UidsIterator()
+
+	var uid uint32
+	if index < 0 {
+		uid = uids[iter.EndIndex()]
+	} else {
+		uid = uids[iter.StartIndex()]
+		for i := 0; iter.Next(); i++ {
+			if i >= index {
+				uid = iter.Value().(uint32)
+				break
+			}
+		}
+	}
+	store.Select(uid)
+
 	ml.Invalidate()
 }
 
@@ -399,13 +426,17 @@ func (ml *MessageList) drawEmptyMessage(ctx *ui.Context) {
 		uiConfig.GetStyle(config.STYLE_MSGLIST_DEFAULT), "%s", msg)
 }
 
-func threadPrefix(t *types.Thread) string {
+func threadPrefix(t *types.Thread, reverse bool) string {
 	var arrow string
 	if t.Parent != nil {
 		if t.NextSibling != nil {
 			arrow = "├─>"
 		} else {
-			arrow = "└─>"
+			if reverse {
+				arrow = "┌─>"
+			} else {
+				arrow = "└─>"
+			}
 		}
 	}
 	var prefix []string
@@ -428,3 +459,11 @@ func threadPrefix(t *types.Thread) string {
 	ps := strings.Join(prefix, "")
 	return fmt.Sprintf("%v%v", ps, arrow)
 }
+
+func sameParent(left, right *types.Thread) bool {
+	return left.Root() == right.Root()
+}
+
+func isParent(t *types.Thread) bool {
+	return t == t.Root()
+}
