@@ -12,7 +12,6 @@ import (
 )
 
 type PGPInfo struct {
-	ui.Invalidatable
 	details  *models.MessageDetails
 	uiConfig *config.UIConfig
 }
@@ -71,7 +70,7 @@ func (p *PGPInfo) DrawEncryption(ctx *ui.Context, y int) {
 	x := ctx.Printf(0, y, validStyle, "%s Encrypted", icon)
 	x += ctx.Printf(x+1, y, defaultStyle, "To %s (%8X) ", p.details.DecryptedWith, p.details.DecryptedWithKeyId)
 	if !p.details.IsSigned {
-		x += ctx.Printf(x, y, warningStyle, "(message not signed!)")
+		ctx.Printf(x, y, warningStyle, "(message not signed!)")
 	}
 }
 
@@ -95,5 +94,5 @@ func (p *PGPInfo) Draw(ctx *ui.Context) {
 }
 
 func (p *PGPInfo) Invalidate() {
-	p.DoInvalidate(p)
+	ui.Invalidate()
 }
