@@ -9,10 +9,10 @@ import (
 
 	"git.sr.ht/~rjarry/aerc/config"
 	"git.sr.ht/~rjarry/aerc/lib/ui"
+	"git.sr.ht/~rjarry/aerc/log"
 )
 
 type Spinner struct {
-	ui.Invalidatable
 	frame  int64 // access via atomic
 	frames []string
 	stop   chan struct{}
@@ -37,6 +37,8 @@ func (s *Spinner) Start() {
 	atomic.StoreInt64(&s.frame, 0)
 
 	go func() {
+		defer log.PanicHandler()
+
 		for {
 			select {
 			case <-s.stop:
@@ -45,7 +47,7 @@ func (s *Spinner) Start() {
 				return
 			case <-time.After(200 * time.Millisecond):
 				atomic.AddInt64(&s.frame, 1)
-				s.Invalidate()
+				ui.QueueRedraw()
 			}
 		}
 	}()
@@ -78,5 +80,5 @@ func (s *Spinner) Draw(ctx *ui.Context) {
 }
 
 func (s *Spinner) Invalidate() {
-	s.DoInvalidate(s)
+	ui.Invalidate()
 }
