@@ -8,11 +8,10 @@ import (
 
 	"git.sr.ht/~rjarry/aerc/config"
 	"git.sr.ht/~rjarry/aerc/lib/ui"
-	"git.sr.ht/~rjarry/aerc/logging"
+	"git.sr.ht/~rjarry/aerc/log"
 )
 
 type StatusLine struct {
-	ui.Invalidatable
 	stack    []*StatusMessage
 	fallback StatusMessage
 	aerc     *Aerc
@@ -23,7 +22,7 @@ type StatusMessage struct {
 	message string
 }
 
-func NewStatusLine(uiConfig config.UIConfig) *StatusLine {
+func NewStatusLine(uiConfig *config.UIConfig) *StatusLine {
 	return &StatusLine{
 		fallback: StatusMessage{
 			style:   uiConfig.GetStyle(config.STYLE_STATUSLINE_DEFAULT),
@@ -33,7 +32,7 @@ func NewStatusLine(uiConfig config.UIConfig) *StatusLine {
 }
 
 func (status *StatusLine) Invalidate() {
-	status.DoInvalidate(status)
+	ui.Invalidate()
 }
 
 func (status *StatusLine) Draw(ctx *ui.Context) {
@@ -77,7 +76,7 @@ func (status *StatusLine) Push(text string, expiry time.Duration) *StatusMessage
 	}
 	status.stack = append(status.stack, msg)
 	go (func() {
-		defer logging.PanicHandler()
+		defer log.PanicHandler()
 
 		time.Sleep(expiry)
 		for i, m := range status.stack {
@@ -98,6 +97,12 @@ func (status *StatusLine) PushError(text string) *StatusMessage {
 	return msg
 }
 
+func (status *StatusLine) PushWarning(text string) *StatusMessage {
+	msg := status.Push(text, 10*time.Second)
+	msg.Color(status.uiConfig().GetStyle(config.STYLE_STATUSLINE_WARNING))
+	return msg
+}
+
 func (status *StatusLine) PushSuccess(text string) *StatusMessage {
 	msg := status.Push(text, 10*time.Second)
 	msg.Color(status.uiConfig().GetStyle(config.STYLE_STATUSLINE_SUCCESS))
