@@ -3,18 +3,18 @@ package imap
 import (
 	"github.com/emersion/go-imap"
 
-	"git.sr.ht/~rjarry/aerc/logging"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
 
 func (imapw *IMAPWorker) handleListDirectories(msg *types.ListDirectories) {
 	mailboxes := make(chan *imap.MailboxInfo)
-	logging.Infof("Listing mailboxes")
+	log.Tracef("Listing mailboxes")
 	done := make(chan interface{})
 
 	go func() {
-		defer logging.PanicHandler()
+		defer log.PanicHandler()
 
 		for mbox := range mailboxes {
 			if !canOpen(mbox) {
@@ -32,17 +32,58 @@ func (imapw *IMAPWorker) handleListDirectories(msg *types.ListDirectories) {
 		done <- nil
 	}()
 
-	if err := imapw.client.List("", "*", mailboxes); err != nil {
-		<-done
-		imapw.worker.PostMessage(&types.Error{
-			Message: types.RespondTo(msg),
-			Error:   err,
-		}, nil)
-	} else {
-		<-done
-		imapw.worker.PostMessage(
-			&types.Done{Message: types.RespondTo(msg)}, nil)
+	switch {
+	case imapw.liststatus:
+		items := []imap.StatusItem{
+			imap.StatusMessages,
+			imap.StatusRecent,
+			imap.StatusUnseen,
+		}
+		statuses, err := imapw.client.liststatus.ListStatus(
+			"",
+			"*",
+			items,
+			mailboxes,
+		)
+		if err != nil {
+			<-done
+			imapw.worker.PostMessage(&types.Error{
+				Message: types.RespondTo(msg),
+				Error:   err,
+			}, nil)
+			return
+
+		}
+		for _, status := range statuses {
+			imapw.worker.PostMessage(&types.DirectoryInfo{
+				Info: &models.DirectoryInfo{
+					Flags:          status.Flags,
+					Name:           status.Name,
+					ReadOnly:       status.ReadOnly,
+					AccurateCounts: true,
+
+					Exists: int(status.Messages),
+					Recent: int(status.Recent),
+					Unseen: int(status.Unseen),
+					Caps:   imapw.caps,
+				},
+				SkipSort: true,
+			}, nil)
+		}
+	default:
+		err := imapw.client.List("", "*", mailboxes)
+		if err != nil {
+			<-done
+			imapw.worker.PostMessage(&types.Error{
+				Message: types.RespondTo(msg),
+				Error:   err,
+			}, nil)
+			return
+		}
 	}
+	<-done
+	imapw.worker.PostMessage(
+		&types.Done{Message: types.RespondTo(msg)}, nil)
 }
 
 func canOpen(mbox *imap.MailboxInfo) bool {
@@ -62,7 +103,7 @@ func (imapw *IMAPWorker) handleSearchDirectory(msg *types.SearchDirectory) {
 		}, nil)
 	}
 
-	logging.Infof("Executing search")
+	log.Tracef("Executing search")
 	criteria, err := parseSearch(msg.Argv)
 	if err != nil {
 		emitError(err)
