@@ -5,11 +5,12 @@ import (
 
 	sortthread "github.com/emersion/go-imap-sortthread"
 
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
 
 func (imapw *IMAPWorker) handleOpenDirectory(msg *types.OpenDirectory) {
-	imapw.worker.Logger.Printf("Opening %s", msg.Directory)
+	log.Debugf("Opening %s", msg.Directory)
 
 	sel, err := imapw.client.Select(msg.Directory, false)
 	if err != nil {
@@ -24,9 +25,9 @@ func (imapw *IMAPWorker) handleOpenDirectory(msg *types.OpenDirectory) {
 }
 
 func (imapw *IMAPWorker) handleFetchDirectoryContents(
-	msg *types.FetchDirectoryContents) {
-
-	imapw.worker.Logger.Printf("Fetching UID list")
+	msg *types.FetchDirectoryContents,
+) {
+	log.Tracef("Fetching UID list")
 
 	searchCriteria, err := parseSearch(msg.FilterCriteria)
 	if err != nil {
@@ -51,7 +52,9 @@ func (imapw *IMAPWorker) handleFetchDirectoryContents(
 	} else {
 		if err != nil {
 			// Non fatal, but we do want to print to get some debug info
-			imapw.worker.Logger.Printf("can't check for SORT support: %v", err)
+			log.Errorf("can't check for SORT support: %v", err)
+		} else if len(sortCriteria) > 0 {
+			log.Warnf("SORT is not supported but requested: list messages by UID")
 		}
 		uids, err = imapw.client.UidSearch(searchCriteria)
 	}
@@ -61,8 +64,11 @@ func (imapw *IMAPWorker) handleFetchDirectoryContents(
 			Error:   err,
 		}, nil)
 	} else {
-		imapw.worker.Logger.Printf("Found %d UIDs", len(uids))
-		imapw.seqMap.Clear()
+		log.Tracef("Found %d UIDs", len(uids))
+		if len(msg.FilterCriteria) == 1 {
+			// Only initialize if we are not filtering
+			imapw.seqMap.Initialize(uids)
+		}
 		imapw.worker.PostMessage(&types.DirectoryContents{
 			Message: types.RespondTo(msg),
 			Uids:    uids,
@@ -85,7 +91,8 @@ var sortFieldMap sortFieldMapT = sortFieldMapT{
 }
 
 func translateSortCriterions(
-	cs []*types.SortCriterion) []sortthread.SortCriterion {
+	cs []*types.SortCriterion,
+) []sortthread.SortCriterion {
 	result := make([]sortthread.SortCriterion, 0, len(cs))
 	for _, c := range cs {
 		if f, ok := sortFieldMap[c.Field]; ok {
@@ -96,8 +103,9 @@ func translateSortCriterions(
 }
 
 func (imapw *IMAPWorker) handleDirectoryThreaded(
-	msg *types.FetchDirectoryThreaded) {
-	imapw.worker.Logger.Printf("Fetching threaded UID list")
+	msg *types.FetchDirectoryThreaded,
+) {
+	log.Tracef("Fetching threaded UID list")
 
 	searchCriteria, err := parseSearch(msg.FilterCriteria)
 	if err != nil {
@@ -107,7 +115,7 @@ func (imapw *IMAPWorker) handleDirectoryThreaded(
 		}, nil)
 		return
 	}
-	threads, err := imapw.client.thread.UidThread(sortthread.References,
+	threads, err := imapw.client.thread.UidThread(imapw.threadAlgorithm,
 		searchCriteria)
 	if err != nil {
 		imapw.worker.PostMessage(&types.Error{
@@ -117,8 +125,18 @@ func (imapw *IMAPWorker) handleDirectoryThreaded(
 	} else {
 		aercThreads, count := convertThreads(threads, nil)
 		sort.Sort(types.ByUID(aercThreads))
-		imapw.worker.Logger.Printf("Found %d threaded messages", count)
-		imapw.seqMap.Clear()
+		log.Tracef("Found %d threaded messages", count)
+		if len(msg.FilterCriteria) == 1 {
+			// Only initialize if we are not filtering
+			var uids []uint32
+			for i := len(aercThreads) - 1; i >= 0; i-- {
+				aercThreads[i].Walk(func(t *types.Thread, level int, currentErr error) error { //nolint:errcheck // error indicates skipped threads
+					uids = append(uids, t.Uid)
+					return nil
+				})
+			}
+			imapw.seqMap.Initialize(uids)
+		}
 		imapw.worker.PostMessage(&types.DirectoryThreaded{
 			Message: types.RespondTo(msg),
 			Threads: aercThreads,
