@@ -12,9 +12,10 @@ import (
 	"github.com/syndtr/goleveldb/leveldb"
 
 	"git.sr.ht/~rjarry/aerc/lib"
-	"git.sr.ht/~rjarry/aerc/logging"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/handlers"
+	"git.sr.ht/~rjarry/aerc/worker/imap/extensions"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 )
 
@@ -32,8 +33,9 @@ var (
 
 type imapClient struct {
 	*client.Client
-	thread *sortthread.ThreadClient
-	sort   *sortthread.SortClient
+	thread     *sortthread.ThreadClient
+	sort       *sortthread.SortClient
+	liststatus *extensions.ListStatusClient
 }
 
 type imapConfig struct {
@@ -70,6 +72,9 @@ type IMAPWorker struct {
 	cache    *leveldb.DB
 
 	caps *models.Capabilities
+
+	threadAlgorithm sortthread.ThreadAlgorithm
+	liststatus      bool
 }
 
 func NewIMAPWorker(worker *types.Worker) (types.Backend, error) {
@@ -85,18 +90,32 @@ func NewIMAPWorker(worker *types.Worker) (types.Backend, error) {
 
 func (w *IMAPWorker) newClient(c *client.Client) {
 	c.Updates = w.updates
-	w.client = &imapClient{c, sortthread.NewThreadClient(c), sortthread.NewSortClient(c)}
+	w.client = &imapClient{
+		c,
+		sortthread.NewThreadClient(c),
+		sortthread.NewSortClient(c),
+		extensions.NewListStatusClient(c),
+	}
 	w.idler.SetClient(w.client)
 	w.observer.SetClient(w.client)
 	sort, err := w.client.sort.SupportSort()
 	if err == nil && sort {
 		w.caps.Sort = true
-		logging.Infof("Server Capability found: Sort")
+		log.Debugf("Server Capability found: Sort")
 	}
-	thread, err := w.client.thread.SupportThread()
-	if err == nil && thread {
-		w.caps.Thread = true
-		logging.Infof("Server Capability found: Thread")
+	for _, alg := range []sortthread.ThreadAlgorithm{sortthread.References, sortthread.OrderedSubject} {
+		ok, err := w.client.Support(fmt.Sprintf("THREAD=%s", string(alg)))
+		if err == nil && ok {
+			w.threadAlgorithm = alg
+			w.caps.Thread = true
+			log.Debugf("Server Capability found: Thread (algorithm: %s)", string(alg))
+			break
+		}
+	}
+	lStatus, err := w.client.liststatus.SupportListStatus()
+	if err == nil && lStatus {
+		w.liststatus = true
+		log.Debugf("Server Capability found: LIST-STATUS")
 	}
 }
 
@@ -227,7 +246,7 @@ func (w *IMAPWorker) handleMessage(msg types.WorkerMessage) error {
 }
 
 func (w *IMAPWorker) handleImapUpdate(update client.Update) {
-	logging.Debugf("(= %T", update)
+	log.Tracef("(= %T", update)
 	switch update := update.(type) {
 	case *client.MailboxUpdate:
 		status := update.Mailbox
@@ -250,7 +269,7 @@ func (w *IMAPWorker) handleImapUpdate(update client.Update) {
 		msg := update.Message
 		if msg.Uid == 0 {
 			if uid, found := w.seqMap.Get(msg.SeqNum); !found {
-				logging.Errorf("MessageUpdate unknown seqnum: %d", msg.SeqNum)
+				log.Errorf("MessageUpdate unknown seqnum: %d", msg.SeqNum)
 				return
 			} else {
 				msg.Uid = uid
@@ -270,7 +289,7 @@ func (w *IMAPWorker) handleImapUpdate(update client.Update) {
 		}, nil)
 	case *client.ExpungeUpdate:
 		if uid, found := w.seqMap.Pop(update.SeqNum); !found {
-			logging.Errorf("ExpungeUpdate unknown seqnum: %d", update.SeqNum)
+			log.Errorf("ExpungeUpdate unknown seqnum: %d", update.SeqNum)
 		} else {
 			w.worker.PostMessage(&types.MessagesDeleted{
 				Uids: []uint32{uid},
