@@ -10,6 +10,7 @@ import (
 	"strings"
 	"time"
 
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"github.com/emersion/go-message"
 	_ "github.com/emersion/go-message/charset"
@@ -19,7 +20,7 @@ import (
 // RFC 1123Z regexp
 var dateRe = regexp.MustCompile(`(((Mon|Tue|Wed|Thu|Fri|Sat|Sun))[,]?\s[0-9]{1,2})\s` +
 	`(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\s` +
-	`([0-9]{4})\s([0-9]{2}):([0-9]{2})(:([0-9]{2}))?\s([\+|\-][0-9]{4})\s?`)
+	`([0-9]{4})\s([0-9]{2}):([0-9]{2})(:([0-9]{2}))?\s([\+|\-][0-9]{4})`)
 
 func FetchEntityPartReader(e *message.Entity, index []int) (io.Reader, error) {
 	if len(index) == 0 {
@@ -46,7 +47,7 @@ func FetchEntityPartReader(e *message.Entity, index []int) (io.Reader, error) {
 	return nil, fmt.Errorf("FetchEntityPartReader: unexpected code reached")
 }
 
-//TODO: the UI doesn't seem to like readers which aren't buffers
+// TODO: the UI doesn't seem to like readers which aren't buffers
 func bufReader(e *message.Entity) (io.Reader, error) {
 	var buf bytes.Buffer
 	if _, err := io.Copy(&buf, e.Body); err != nil {
@@ -104,7 +105,7 @@ func ParseEntityStructure(e *message.Entity) (*models.BodyStructure, error) {
 	if cd := e.Header.Get("content-disposition"); cd != "" {
 		contentDisposition, cdParams, err := e.Header.ContentDisposition()
 		if err != nil {
-			return nil, fmt.Errorf("could not parse content disposition: %v", err)
+			return nil, fmt.Errorf("could not parse content disposition: %w", err)
 		}
 		body.Disposition = contentDisposition
 		body.DispositionParams = cdParams
@@ -113,7 +114,7 @@ func ParseEntityStructure(e *message.Entity) (*models.BodyStructure, error) {
 	if mpr := e.MultipartReader(); mpr != nil {
 		for {
 			part, err := mpr.NextPart()
-			if err == io.EOF {
+			if errors.Is(err, io.EOF) {
 				return &body, nil
 			} else if err != nil {
 				return nil, err
@@ -133,36 +134,47 @@ var DateParseError = errors.New("date parsing failed")
 func parseEnvelope(h *mail.Header) (*models.Envelope, error) {
 	from, err := parseAddressList(h, "from")
 	if err != nil {
-		return nil, fmt.Errorf("could not read from address: %v", err)
+		return nil, fmt.Errorf("could not read from address: %w", err)
 	}
 	to, err := parseAddressList(h, "to")
 	if err != nil {
-		return nil, fmt.Errorf("could not read to address: %v", err)
+		return nil, fmt.Errorf("could not read to address: %w", err)
 	}
 	cc, err := parseAddressList(h, "cc")
 	if err != nil {
-		return nil, fmt.Errorf("could not read cc address: %v", err)
+		return nil, fmt.Errorf("could not read cc address: %w", err)
 	}
 	bcc, err := parseAddressList(h, "bcc")
 	if err != nil {
-		return nil, fmt.Errorf("could not read bcc address: %v", err)
+		return nil, fmt.Errorf("could not read bcc address: %w", err)
 	}
 	replyTo, err := parseAddressList(h, "reply-to")
 	if err != nil {
-		return nil, fmt.Errorf("could not read reply-to address: %v", err)
+		return nil, fmt.Errorf("could not read reply-to address: %w", err)
 	}
 	subj, err := h.Subject()
 	if err != nil {
-		return nil, fmt.Errorf("could not read subject: %v", err)
+		return nil, fmt.Errorf("could not read subject: %w", err)
 	}
 	msgID, err := h.MessageID()
 	if err != nil {
-		//proper parsing failed, so fall back to whatever is there
+		// proper parsing failed, so fall back to whatever is there
 		msgID, err = h.Text("message-id")
 		if err != nil {
 			return nil, err
 		}
 	}
+	var irt string
+	irtList, err := h.MsgIDList("in-reply-to")
+	if err != nil {
+		// proper parsing failed, so fall back to whatever is there
+		irt, err = h.Text("in-reply-to")
+		if err != nil {
+			return nil, err
+		}
+	} else if len(irtList) > 0 {
+		irt = irtList[0]
+	}
 	date, err := parseDate(h)
 	if err != nil {
 		// still return a valid struct plus a sentinel date parsing error
@@ -178,6 +190,7 @@ func parseEnvelope(h *mail.Header) (*models.Envelope, error) {
 		To:        to,
 		Cc:        cc,
 		Bcc:       bcc,
+		InReplyTo: irt,
 	}, err
 }
 
@@ -217,22 +230,25 @@ func parseDate(h *mail.Header) (time.Time, error) {
 func parseReceivedHeader(h *mail.Header) (time.Time, error) {
 	guess, err := h.Text("received")
 	if err != nil {
-		return time.Time{}, fmt.Errorf("received header not parseable: %v",
+		return time.Time{}, fmt.Errorf("received header not parseable: %w",
 			err)
 	}
 	return time.Parse(time.RFC1123Z, dateRe.FindString(guess))
 }
 
 func parseAddressList(h *mail.Header, key string) ([]*mail.Address, error) {
-	addrs, err := h.AddressList(key)
-	if err == nil {
-		return addrs, nil
-	}
 	hdr, err := h.Text(key)
-	if err != nil {
+	if err != nil && !message.IsUnknownCharset(err) {
 		return nil, err
 	}
-	return []*mail.Address{{Name: hdr}}, nil
+	if hdr == "" {
+		return nil, nil
+	}
+	add, err := mail.ParseAddressList(hdr)
+	if err != nil {
+		return []*mail.Address{{Name: hdr}}, nil
+	}
+	return add, err
 }
 
 // RawMessage is an interface that describes a raw message
@@ -252,20 +268,20 @@ func MessageInfo(raw RawMessage) (*models.MessageInfo, error) {
 		return nil, err
 	}
 	defer r.Close()
-	msg, err := message.Read(r)
+	msg, err := ReadMessage(r)
 	if err != nil {
-		return nil, fmt.Errorf("could not read message: %v", err)
+		return nil, fmt.Errorf("could not read message: %w", err)
 	}
 	bs, err := ParseEntityStructure(msg)
 	if errors.As(err, new(message.UnknownEncodingError)) {
 		parseErr = err
 	} else if err != nil {
-		return nil, fmt.Errorf("could not get structure: %v", err)
+		return nil, fmt.Errorf("could not get structure: %w", err)
 	}
 	h := &mail.Header{Header: msg.Header}
 	env, err := parseEnvelope(h)
 	if err != nil && !errors.Is(err, DateParseError) {
-		return nil, fmt.Errorf("could not parse envelope: %v", err)
+		return nil, fmt.Errorf("could not parse envelope: %w", err)
 		// if only the date parsing failed we still get the rest of the
 		// envelop structure in a valid state.
 		// Date parsing errors are fairly common and it's better to be
@@ -298,6 +314,59 @@ func MessageInfo(raw RawMessage) (*models.MessageInfo, error) {
 	}, nil
 }
 
+// MessageHeaders populates a models.MessageInfo struct for the message.
+// based on the reader returned by NewReader. Minimal information is included.
+// There is no body structure or RFC822Headers set
+func MessageHeaders(raw RawMessage) (*models.MessageInfo, error) {
+	var parseErr error
+	r, err := raw.NewReader()
+	if err != nil {
+		return nil, err
+	}
+	defer r.Close()
+	msg, err := ReadMessage(r)
+	if err != nil {
+		return nil, fmt.Errorf("could not read message: %w", err)
+	}
+	h := &mail.Header{Header: msg.Header}
+	env, err := parseEnvelope(h)
+	if err != nil && !errors.Is(err, DateParseError) {
+		return nil, fmt.Errorf("could not parse envelope: %w", err)
+		// if only the date parsing failed we still get the rest of the
+		// envelop structure in a valid state.
+		// Date parsing errors are fairly common and it's better to be
+		// slightly off than to not be able to read the mails at all
+		// hence we continue here
+	}
+	recDate, _ := parseReceivedHeader(h)
+	if recDate.IsZero() {
+		// better than nothing, if incorrect
+		recDate = env.Date
+	}
+	flags, err := raw.ModelFlags()
+	if err != nil {
+		return nil, err
+	}
+	labels, err := raw.Labels()
+	if err != nil {
+		return nil, err
+	}
+	refs, err := h.MsgIDList("references")
+	if err != nil {
+		return nil, err
+	}
+	return &models.MessageInfo{
+		Envelope:     env,
+		Flags:        flags,
+		Labels:       labels,
+		InternalDate: recDate,
+		Refs:         refs,
+		Size:         0,
+		Uid:          raw.UID(),
+		Error:        parseErr,
+	}, nil
+}
+
 // NewCRLFReader returns a reader with CRLF line endings
 func NewCRLFReader(r io.Reader) io.Reader {
 	var buf bytes.Buffer
@@ -307,3 +376,17 @@ func NewCRLFReader(r io.Reader) io.Reader {
 	}
 	return &buf
 }
+
+// ReadMessage is a wrapper for the message.Read function to read a message
+// from r. The message's encoding and charset are automatically decoded to
+// UTF-8. If an unknown charset is encountered, the error is logged but a nil
+// error is returned since the entity object can still be read.
+func ReadMessage(r io.Reader) (*message.Entity, error) {
+	entity, err := message.Read(r)
+	if message.IsUnknownCharset(err) {
+		log.Warnf("unknown charset encountered")
+	} else if err != nil {
+		return nil, fmt.Errorf("could not read message: %w", err)
+	}
+	return entity, nil
+}
