@@ -10,13 +10,17 @@ import (
 	"os"
 	"os/exec"
 	"path/filepath"
+	"runtime"
 	"sort"
 	"strings"
+	"sync"
 
 	"github.com/emersion/go-maildir"
 	"github.com/fsnotify/fsnotify"
 
-	"git.sr.ht/~rjarry/aerc/logging"
+	aercLib "git.sr.ht/~rjarry/aerc/lib"
+	"git.sr.ht/~rjarry/aerc/lib/iterator"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/models"
 	"git.sr.ht/~rjarry/aerc/worker/handlers"
 	"git.sr.ht/~rjarry/aerc/worker/lib"
@@ -111,7 +115,7 @@ func (w *Worker) handleFSEvent(ev fsnotify.Event) {
 	}
 	err := w.c.SyncNewMail(*w.selected)
 	if err != nil {
-		logging.Errorf("could not move new to cur : %v", err)
+		log.Errorf("could not move new to cur : %v", err)
 		return
 	}
 
@@ -178,11 +182,11 @@ func (w *Worker) getDirectoryInfo(name string) *models.DirectoryInfo {
 
 		Caps: &models.Capabilities{
 			Sort:   true,
-			Thread: false,
+			Thread: true,
 		},
 	}
 
-	dir := w.c.Dir(name)
+	dir := w.c.Store.Dir(name)
 	var keyFlags map[string][]maildir.Flag
 	files, err := dirFiles(string(dir))
 	if err == nil {
@@ -190,18 +194,18 @@ func (w *Worker) getDirectoryInfo(name string) *models.DirectoryInfo {
 		for _, v := range files {
 			key, flags, err := splitMaildirFile(v)
 			if err != nil {
-				logging.Errorf("%q: error parsing flags (%q): %v", v, key, err)
+				log.Errorf("%q: error parsing flags (%q): %v", v, key, err)
 				continue
 			}
 			keyFlags[key] = flags
 		}
 	} else {
-		logging.Infof("disabled flags cache: %q: %v", dir, err)
+		log.Tracef("disabled flags cache: %q: %v", dir, err)
 	}
 
 	uids, err := w.c.UIDs(dir)
 	if err != nil {
-		logging.Errorf("could not get uids: %v", err)
+		log.Errorf("could not get uids: %v", err)
 		return dirInfo
 	}
 
@@ -209,7 +213,7 @@ func (w *Worker) getDirectoryInfo(name string) *models.DirectoryInfo {
 	for _, uid := range uids {
 		message, err := w.c.Message(dir, uid)
 		if err != nil {
-			logging.Errorf("could not get message: %v", err)
+			log.Errorf("could not get message: %v", err)
 			continue
 		}
 		var flags []maildir.Flag
@@ -217,17 +221,18 @@ func (w *Worker) getDirectoryInfo(name string) *models.DirectoryInfo {
 			ok := false
 			flags, ok = keyFlags[message.key]
 			if !ok {
-				logging.Debugf("message (key=%q uid=%d) not found in map cache", message.key, message.uid)
+				log.Tracef("message (key=%q uid=%d) not found in map cache",
+					message.key, message.uid)
 				flags, err = message.Flags()
 				if err != nil {
-					logging.Errorf("could not get flags: %v", err)
+					log.Errorf("could not get flags: %v", err)
 					continue
 				}
 			}
 		} else {
 			flags, err = message.Flags()
 			if err != nil {
-				logging.Errorf("could not get flags: %v", err)
+				log.Errorf("could not get flags: %v", err)
 				continue
 			}
 		}
@@ -263,6 +268,8 @@ func (w *Worker) handleMessage(msg types.WorkerMessage) error {
 		return w.handleOpenDirectory(msg)
 	case *types.FetchDirectoryContents:
 		return w.handleFetchDirectoryContents(msg)
+	case *types.FetchDirectoryThreaded:
+		return w.handleFetchDirectoryThreaded(msg)
 	case *types.CreateDirectory:
 		return w.handleCreateDirectory(msg)
 	case *types.RemoveDirectory:
@@ -294,7 +301,7 @@ func (w *Worker) handleMessage(msg types.WorkerMessage) error {
 func (w *Worker) handleConfigure(msg *types.Configure) error {
 	u, err := url.Parse(msg.Config.Source)
 	if err != nil {
-		logging.Errorf("error configuring maildir worker: %v", err)
+		log.Errorf("error configuring maildir worker: %v", err)
 		return err
 	}
 	dir := u.Path
@@ -310,11 +317,11 @@ func (w *Worker) handleConfigure(msg *types.Configure) error {
 	}
 	c, err := NewContainer(dir, w.maildirpp)
 	if err != nil {
-		logging.Errorf("could not configure maildir: %s", dir)
+		log.Errorf("could not configure maildir: %s", dir)
 		return err
 	}
 	w.c = c
-	logging.Infof("configured base maildir: %s", dir)
+	log.Debugf("configured base maildir: %s", dir)
 	return nil
 }
 
@@ -329,12 +336,12 @@ func (w *Worker) handleListDirectories(msg *types.ListDirectories) error {
 	if w.c == nil {
 		return errors.New("Incorrect maildir directory")
 	}
-	dirs, err := w.c.ListFolders()
+	dirs, err := w.c.Store.FolderMap()
 	if err != nil {
-		logging.Errorf("failed listing directories: %v", err)
+		log.Errorf("failed listing directories: %v", err)
 		return err
 	}
-	for _, name := range dirs {
+	for name := range dirs {
 		w.worker.PostMessage(&types.Directory{
 			Message: types.RespondTo(msg),
 			Dir: &models.Directory{
@@ -351,7 +358,7 @@ func (w *Worker) handleListDirectories(msg *types.ListDirectories) error {
 }
 
 func (w *Worker) handleOpenDirectory(msg *types.OpenDirectory) error {
-	logging.Infof("opening %s", msg.Directory)
+	log.Debugf("opening %s", msg.Directory)
 
 	// open the directory
 	dir, err := w.c.OpenDirectory(msg.Directory)
@@ -402,7 +409,8 @@ func (w *Worker) handleFetchDirectoryContents(
 		uids []uint32
 		err  error
 	)
-	if len(msg.FilterCriteria) > 0 {
+	// FilterCriteria always contains "filter" as first item
+	if len(msg.FilterCriteria) > 1 {
 		filter, err := parseSearch(msg.FilterCriteria)
 		if err != nil {
 			return err
@@ -414,13 +422,13 @@ func (w *Worker) handleFetchDirectoryContents(
 	} else {
 		uids, err = w.c.UIDs(*w.selected)
 		if err != nil {
-			logging.Errorf("failed scanning uids: %v", err)
+			log.Errorf("failed scanning uids: %v", err)
 			return err
 		}
 	}
 	sortedUids, err := w.sort(uids, msg.SortCriteria)
 	if err != nil {
-		logging.Errorf("failed sorting directory: %v", err)
+		log.Errorf("failed sorting directory: %v", err)
 		return err
 	}
 	w.currentSortCriteria = msg.SortCriteria
@@ -433,29 +441,129 @@ func (w *Worker) handleFetchDirectoryContents(
 
 func (w *Worker) sort(uids []uint32, criteria []*types.SortCriterion) ([]uint32, error) {
 	if len(criteria) == 0 {
+		// At least sort by uid, parallel searching can create random
+		// order
+		sort.Slice(uids, func(i int, j int) bool {
+			return uids[i] < uids[j]
+		})
 		return uids, nil
 	}
 	var msgInfos []*models.MessageInfo
+	mu := sync.Mutex{}
+	wg := sync.WaitGroup{}
+	// Hard limit at 2x CPU cores
+	max := runtime.NumCPU() * 2
+	limit := make(chan struct{}, max)
 	for _, uid := range uids {
-		info, err := w.msgInfoFromUid(uid)
-		if err != nil {
-			logging.Errorf("could not get message info: %v", err)
-			continue
-		}
-		msgInfos = append(msgInfos, info)
+		limit <- struct{}{}
+		wg.Add(1)
+		go func(uid uint32) {
+			defer wg.Done()
+			info, err := w.msgHeadersFromUid(uid)
+			if err != nil {
+				log.Errorf("could not get message info: %v", err)
+				<-limit
+				return
+			}
+			mu.Lock()
+			msgInfos = append(msgInfos, info)
+			mu.Unlock()
+			<-limit
+		}(uid)
 	}
+
+	wg.Wait()
 	sortedUids, err := lib.Sort(msgInfos, criteria)
 	if err != nil {
-		logging.Errorf("could not sort the messages: %v", err)
+		log.Errorf("could not sort the messages: %v", err)
 		return nil, err
 	}
 	return sortedUids, nil
 }
 
+func (w *Worker) handleFetchDirectoryThreaded(
+	msg *types.FetchDirectoryThreaded,
+) error {
+	var (
+		uids []uint32
+		err  error
+	)
+	if len(msg.FilterCriteria) > 1 {
+		filter, err := parseSearch(msg.FilterCriteria)
+		if err != nil {
+			return err
+		}
+		uids, err = w.search(filter)
+		if err != nil {
+			return err
+		}
+	} else {
+		uids, err = w.c.UIDs(*w.selected)
+		if err != nil {
+			log.Errorf("failed scanning uids: %v", err)
+			return err
+		}
+	}
+	threads, err := w.threads(uids, msg.SortCriteria)
+	if err != nil {
+		log.Errorf("failed sorting directory: %v", err)
+		return err
+	}
+	w.currentSortCriteria = msg.SortCriteria
+	w.worker.PostMessage(&types.DirectoryThreaded{
+		Message: types.RespondTo(msg),
+		Threads: threads,
+	}, nil)
+	return nil
+}
+
+func (w *Worker) threads(uids []uint32, criteria []*types.SortCriterion) ([]*types.Thread, error) {
+	builder := aercLib.NewThreadBuilder(iterator.NewFactory(false))
+	msgInfos := make([]*models.MessageInfo, 0, len(uids))
+	mu := sync.Mutex{}
+	wg := sync.WaitGroup{}
+	max := runtime.NumCPU() * 2
+	limit := make(chan struct{}, max)
+	for _, uid := range uids {
+		limit <- struct{}{}
+		wg.Add(1)
+		go func(uid uint32) {
+			defer wg.Done()
+			info, err := w.msgHeadersFromUid(uid)
+			if err != nil {
+				log.Errorf("could not get message info: %v", err)
+				<-limit
+				return
+			}
+			mu.Lock()
+			builder.Update(info)
+			msgInfos = append(msgInfos, info)
+			mu.Unlock()
+			<-limit
+		}(uid)
+	}
+	wg.Wait()
+	var err error
+	switch {
+	case len(criteria) == 0:
+		sort.Slice(uids, func(i int, j int) bool {
+			return uids[i] < uids[j]
+		})
+	default:
+		uids, err = lib.Sort(msgInfos, criteria)
+		if err != nil {
+			log.Errorf("could not sort the messages: %v", err)
+			return nil, err
+		}
+	}
+	threads := builder.Threads(uids, false, false)
+	return threads, nil
+}
+
 func (w *Worker) handleCreateDirectory(msg *types.CreateDirectory) error {
-	dir := w.c.Dir(msg.Directory)
+	dir := w.c.Store.Dir(msg.Directory)
 	if err := dir.Init(); err != nil {
-		logging.Errorf("could not create directory %s: %v",
+		log.Errorf("could not create directory %s: %v",
 			msg.Directory, err)
 		return err
 	}
@@ -463,9 +571,9 @@ func (w *Worker) handleCreateDirectory(msg *types.CreateDirectory) error {
 }
 
 func (w *Worker) handleRemoveDirectory(msg *types.RemoveDirectory) error {
-	dir := w.c.Dir(msg.Directory)
+	dir := w.c.Store.Dir(msg.Directory)
 	if err := os.RemoveAll(string(dir)); err != nil {
-		logging.Errorf("could not remove directory %s: %v",
+		log.Errorf("could not remove directory %s: %v",
 			msg.Directory, err)
 		return err
 	}
@@ -478,7 +586,7 @@ func (w *Worker) handleFetchMessageHeaders(
 	for _, uid := range msg.Uids {
 		info, err := w.msgInfoFromUid(uid)
 		if err != nil {
-			logging.Errorf("could not get message info: %v", err)
+			log.Errorf("could not get message info: %v", err)
 			w.worker.PostMessageInfoError(msg, uid, err)
 			continue
 		}
@@ -497,12 +605,12 @@ func (w *Worker) handleFetchMessageBodyPart(
 	// get reader
 	m, err := w.c.Message(*w.selected, msg.Uid)
 	if err != nil {
-		logging.Errorf("could not get message %d: %v", msg.Uid, err)
+		log.Errorf("could not get message %d: %v", msg.Uid, err)
 		return err
 	}
 	r, err := m.NewBodyPartReader(msg.Part)
 	if err != nil {
-		logging.Errorf(
+		log.Errorf(
 			"could not get body part reader for message=%d, parts=%#v: %w",
 			msg.Uid, msg.Part, err)
 		return err
@@ -522,12 +630,12 @@ func (w *Worker) handleFetchFullMessages(msg *types.FetchFullMessages) error {
 	for _, uid := range msg.Uids {
 		m, err := w.c.Message(*w.selected, uid)
 		if err != nil {
-			logging.Errorf("could not get message %d: %v", uid, err)
+			log.Errorf("could not get message %d: %v", uid, err)
 			return err
 		}
 		r, err := m.NewReader()
 		if err != nil {
-			logging.Errorf("could not get message reader: %v", err)
+			log.Errorf("could not get message reader: %v", err)
 			return err
 		}
 		defer r.Close()
@@ -558,7 +666,7 @@ func (w *Worker) handleDeleteMessages(msg *types.DeleteMessages) error {
 		}, nil)
 	}
 	if err != nil {
-		logging.Errorf("failed removing messages: %v", err)
+		log.Errorf("failed removing messages: %v", err)
 		return err
 	}
 	return nil
@@ -568,18 +676,18 @@ func (w *Worker) handleAnsweredMessages(msg *types.AnsweredMessages) error {
 	for _, uid := range msg.Uids {
 		m, err := w.c.Message(*w.selected, uid)
 		if err != nil {
-			logging.Errorf("could not get message: %v", err)
+			log.Errorf("could not get message: %v", err)
 			w.err(msg, err)
 			continue
 		}
 		if err := m.MarkReplied(msg.Answered); err != nil {
-			logging.Errorf("could not mark message as answered: %v", err)
+			log.Errorf("could not mark message as answered: %v", err)
 			w.err(msg, err)
 			continue
 		}
 		info, err := m.MessageInfo()
 		if err != nil {
-			logging.Errorf("could not get message info: %v", err)
+			log.Errorf("could not get message info: %v", err)
 			w.err(msg, err)
 			continue
 		}
@@ -600,19 +708,19 @@ func (w *Worker) handleFlagMessages(msg *types.FlagMessages) error {
 	for _, uid := range msg.Uids {
 		m, err := w.c.Message(*w.selected, uid)
 		if err != nil {
-			logging.Errorf("could not get message: %v", err)
+			log.Errorf("could not get message: %v", err)
 			w.err(msg, err)
 			continue
 		}
-		flag := flagToMaildir[msg.Flag]
+		flag := lib.FlagToMaildir[msg.Flag]
 		if err := m.SetOneFlag(flag, msg.Enable); err != nil {
-			logging.Errorf("could change flag %v to %v on message: %v", flag, msg.Enable, err)
+			log.Errorf("could change flag %v to %v on message: %v", flag, msg.Enable, err)
 			w.err(msg, err)
 			continue
 		}
 		info, err := m.MessageInfo()
 		if err != nil {
-			logging.Errorf("could not get message info: %v", err)
+			log.Errorf("could not get message info: %v", err)
 			w.err(msg, err)
 			continue
 		}
@@ -631,7 +739,7 @@ func (w *Worker) handleFlagMessages(msg *types.FlagMessages) error {
 }
 
 func (w *Worker) handleCopyMessages(msg *types.CopyMessages) error {
-	dest := w.c.Dir(msg.Destination)
+	dest := w.c.Store.Dir(msg.Destination)
 	err := w.c.CopyAll(dest, *w.selected, msg.Uids)
 	if err != nil {
 		return err
@@ -645,11 +753,12 @@ func (w *Worker) handleCopyMessages(msg *types.CopyMessages) error {
 }
 
 func (w *Worker) handleMoveMessages(msg *types.MoveMessages) error {
-	dest := w.c.Dir(msg.Destination)
+	dest := w.c.Store.Dir(msg.Destination)
 	moved, err := w.c.MoveAll(dest, *w.selected, msg.Uids)
-	destInfo := w.getDirectoryInfo(msg.Destination)
-	w.worker.PostMessage(&types.DirectoryInfo{
-		Info: destInfo,
+	w.worker.PostMessage(&types.MessagesMoved{
+		Message:     types.RespondTo(msg),
+		Destination: msg.Destination,
+		Uids:        moved,
 	}, nil)
 	w.worker.PostMessage(&types.MessagesDeleted{
 		Message: types.RespondTo(msg),
@@ -660,16 +769,16 @@ func (w *Worker) handleMoveMessages(msg *types.MoveMessages) error {
 
 func (w *Worker) handleAppendMessage(msg *types.AppendMessage) error {
 	// since we are the "master" maildir process, we can modify the maildir directly
-	dest := w.c.Dir(msg.Destination)
-	_, writer, err := dest.Create(translateFlags(msg.Flags))
+	dest := w.c.Store.Dir(msg.Destination)
+	_, writer, err := dest.Create(lib.ToMaildirFlags(msg.Flags))
 	if err != nil {
-		logging.Errorf("could not create message at %s: %v", msg.Destination, err)
-		return err
+		return fmt.Errorf("could not create message at %s: %w",
+			msg.Destination, err)
 	}
 	defer writer.Close()
 	if _, err := io.Copy(writer, msg.Reader); err != nil {
-		logging.Errorf("could not write message to destination: %v", err)
-		return err
+		return fmt.Errorf(
+			"could not write message to destination: %w", err)
 	}
 	w.worker.PostMessage(&types.Done{
 		Message: types.RespondTo(msg),
@@ -681,12 +790,12 @@ func (w *Worker) handleAppendMessage(msg *types.AppendMessage) error {
 }
 
 func (w *Worker) handleSearchDirectory(msg *types.SearchDirectory) error {
-	logging.Infof("Searching directory %v with args: %v", *w.selected, msg.Argv)
+	log.Debugf("Searching directory %v with args: %v", *w.selected, msg.Argv)
 	criteria, err := parseSearch(msg.Argv)
 	if err != nil {
 		return err
 	}
-	logging.Infof("Searching with parsed criteria: %#v", criteria)
+	log.Tracef("Searching with parsed criteria: %#v", criteria)
 	uids, err := w.search(criteria)
 	if err != nil {
 		return err
@@ -713,6 +822,18 @@ func (w *Worker) msgInfoFromUid(uid uint32) (*models.MessageInfo, error) {
 	return info, nil
 }
 
+func (w *Worker) msgHeadersFromUid(uid uint32) (*models.MessageInfo, error) {
+	m, err := w.c.Message(*w.selected, uid)
+	if err != nil {
+		return nil, err
+	}
+	info, err := m.MessageHeaders()
+	if err != nil {
+		return nil, err
+	}
+	return info, nil
+}
+
 func (w *Worker) handleCheckMail(msg *types.CheckMail) {
 	if msg.Command == "" {
 		w.err(msg, fmt.Errorf("checkmail: no command specified"))
@@ -733,18 +854,19 @@ func (w *Worker) handleCheckMail(msg *types.CheckMail) {
 		if err != nil {
 			w.err(msg, fmt.Errorf("checkmail: error running command: %w", err))
 		} else {
-			dirs, err := w.c.ListFolders()
+			dirs, err := w.c.Store.FolderMap()
 			if err != nil {
 				w.err(msg, fmt.Errorf("failed listing directories: %w", err))
 			}
-			for _, name := range dirs {
-				err := w.c.SyncNewMail(w.c.Dir(name))
+			for name, dir := range dirs {
+				err := w.c.SyncNewMail(dir)
 				if err != nil {
 					w.err(msg, fmt.Errorf("could not sync new mail: %w", err))
 				}
 				dirInfo := w.getDirectoryInfo(name)
 				w.worker.PostMessage(&types.DirectoryInfo{
-					Info: dirInfo,
+					Info:     dirInfo,
+					SkipSort: true,
 				}, nil)
 			}
 			w.done(msg)
