@@ -4,11 +4,12 @@
 package lib
 
 import (
+	"errors"
 	"fmt"
-	"log"
 	"time"
 
 	"git.sr.ht/~rjarry/aerc/lib/uidstore"
+	"git.sr.ht/~rjarry/aerc/log"
 	"git.sr.ht/~rjarry/aerc/worker/types"
 	notmuch "github.com/zenhack/go.notmuch"
 )
@@ -18,18 +19,15 @@ const MAX_DB_AGE time.Duration = 10 * time.Second
 type DB struct {
 	path         string
 	excludedTags []string
-	logger       *log.Logger
 	lastOpenTime time.Time
 	db           *notmuch.DB
 	uidStore     *uidstore.Store
 }
 
-func NewDB(path string, excludedTags []string,
-	logger *log.Logger) *DB {
+func NewDB(path string, excludedTags []string) *DB {
 	db := &DB{
 		path:         path,
 		excludedTags: excludedTags,
-		logger:       logger,
 		uidStore:     uidstore.NewStore(),
 	}
 	return db
@@ -58,24 +56,24 @@ func (db *DB) connect(writable bool) error {
 	var err error
 	db.db, err = notmuch.Open(db.path, mode)
 	if err != nil {
-		return fmt.Errorf("could not connect to notmuch db: %v", err)
+		return fmt.Errorf("could not connect to notmuch db: %w", err)
 	}
 	db.lastOpenTime = time.Now()
 	return nil
 }
 
-//withConnection calls callback on the DB object, cleaning up upon return.
-//the error returned is from the connection attempt, if not successful,
-//or from the callback otherwise.
+// withConnection calls callback on the DB object, cleaning up upon return.
+// the error returned is from the connection attempt, if not successful,
+// or from the callback otherwise.
 func (db *DB) withConnection(writable bool, cb func(*notmuch.DB) error) error {
 	too_old := time.Now().After(db.lastOpenTime.Add(MAX_DB_AGE))
 	if db.db == nil || writable || too_old {
 		if cerr := db.close(); cerr != nil {
-			db.logger.Printf("failed to close the notmuch db: %v", cerr)
+			log.Errorf("failed to close the notmuch db: %v", cerr)
 		}
 		err := db.connect(writable)
 		if err != nil {
-			db.logger.Printf("failed to open the notmuch db: %v", err)
+			log.Errorf("failed to open the notmuch db: %v", err)
 			return err
 		}
 	}
@@ -83,7 +81,7 @@ func (db *DB) withConnection(writable bool, cb func(*notmuch.DB) error) error {
 	if writable {
 		// we need to close to commit the changes, else we block others
 		if cerr := db.close(); cerr != nil {
-			db.logger.Printf("failed to close the notmuch db: %v", cerr)
+			log.Errorf("failed to close the notmuch db: %v", cerr)
 		}
 	}
 	return err
@@ -107,21 +105,37 @@ func (db *DB) ListTags() ([]string, error) {
 	return result, err
 }
 
-//getQuery returns a query based on the provided query string.
-//It also configures the query as specified on the worker
+// getQuery returns a query based on the provided query string.
+// It also configures the query as specified on the worker
 func (db *DB) newQuery(ndb *notmuch.DB, query string) (*notmuch.Query, error) {
 	q := ndb.NewQuery(query)
 	q.SetExcludeScheme(notmuch.EXCLUDE_ALL)
 	q.SetSortScheme(notmuch.SORT_OLDEST_FIRST)
 	for _, t := range db.excludedTags {
 		err := q.AddTagExclude(t)
-		if err != nil && err != notmuch.ErrIgnored {
+		if err != nil && !errors.Is(err, notmuch.ErrIgnored) {
 			return nil, err
 		}
 	}
 	return q, nil
 }
 
+func (db *DB) MsgIDFromFilename(filename string) (string, error) {
+	var key string
+
+	err := db.withConnection(false, func(ndb *notmuch.DB) error {
+		msg, err := ndb.FindMessageByFilename(filename)
+		if err != nil && !errors.Is(err, notmuch.ErrDuplicateMessageID) {
+			return err
+		}
+		defer msg.Close()
+		key = msg.ID()
+		return nil
+	})
+
+	return key, err
+}
+
 func (db *DB) MsgIDsFromQuery(q string) ([]string, error) {
 	var msgIDs []string
 	err := db.withConnection(false, func(ndb *notmuch.DB) error {
@@ -227,8 +241,63 @@ func (db *DB) MsgTags(key string) ([]string, error) {
 	return tags, err
 }
 
+func (db *DB) MsgFilenames(key string) ([]string, error) {
+	var filenames []string
+
+	err := db.withConnection(false, func(ndb *notmuch.DB) error {
+		msg, err := ndb.FindMessage(key)
+		if err != nil {
+			return err
+		}
+		defer msg.Close()
+
+		fns := msg.Filenames()
+		var filename string
+		for fns.Next(&filename) {
+			if err != nil && !errors.Is(err, notmuch.ErrDuplicateMessageID) {
+				return err
+			}
+			filenames = append(filenames, filename)
+		}
+
+		return nil
+	})
+
+	return filenames, err
+}
+
+func (db *DB) DeleteMessage(filename string) error {
+	return db.withConnection(true, func(ndb *notmuch.DB) error {
+		err := ndb.RemoveMessage(filename)
+		if err != nil && !errors.Is(err, notmuch.ErrDuplicateMessageID) {
+			return err
+		}
+
+		return nil
+	})
+}
+
+func (db *DB) IndexFile(filename string) (string, error) {
+	var key string
+
+	err := db.withConnection(true, func(ndb *notmuch.DB) error {
+		msg, err := ndb.AddMessage(filename)
+		if err != nil && !errors.Is(err, notmuch.ErrDuplicateMessageID) {
+			return err
+		}
+		defer msg.Close()
+		if err := msg.MaildirFlagsToTags(); err != nil {
+			return err
+		}
+		key = msg.ID()
+		return nil
+	})
+	return key, err
+}
+
 func (db *DB) msgModify(key string,
-	cb func(*notmuch.Message) error) error {
+	cb func(*notmuch.Message) error,
+) error {
 	err := db.withConnection(true, func(ndb *notmuch.DB) error {
 		msg, err := ndb.FindMessage(key)
 		if err != nil {
@@ -236,10 +305,14 @@ func (db *DB) msgModify(key string,
 		}
 		defer msg.Close()
 
-		cb(msg)
+		err = cb(msg)
+		if err != nil {
+			log.Warnf("callback failed: %v", err)
+		}
+
 		err = msg.TagsToMaildirFlags()
 		if err != nil {
-			db.logger.Printf("could not sync maildir flags: %v", err)
+			log.Errorf("could not sync maildir flags: %v", err)
 		}
 		return nil
 	})
@@ -250,10 +323,16 @@ func (db *DB) MsgModifyTags(key string, add, remove []string) error {
 	err := db.msgModify(key, func(msg *notmuch.Message) error {
 		ierr := msg.Atomic(func(msg *notmuch.Message) {
 			for _, t := range add {
-				msg.AddTag(t)
+				err := msg.AddTag(t)
+				if err != nil {
+					log.Warnf("failed to add tag: %v", err)
+				}
 			}
 			for _, t := range remove {
-				msg.RemoveTag(t)
+				err := msg.RemoveTag(t)
+				if err != nil {
+					log.Warnf("failed to remove tag: %v", err)
+				}
 			}
 		})
 		return ierr
@@ -284,7 +363,8 @@ func (db *DB) KeyFromUid(uid uint32) (string, bool) {
 }
 
 func (db *DB) enumerateThread(nt *notmuch.Threads,
-	valid map[string]struct{}) ([]*types.Thread, error) {
+	valid map[string]struct{},
+) ([]*types.Thread, error) {
 	var res []*types.Thread
 	var thread *notmuch.Thread
 	for nt.Next(&thread) {
@@ -295,7 +375,8 @@ func (db *DB) enumerateThread(nt *notmuch.Threads,
 }
 
 func (db *DB) makeThread(parent *types.Thread, msgs *notmuch.Messages,
-	valid map[string]struct{}) *types.Thread {
+	valid map[string]struct{},
+) *types.Thread {
 	var lastSibling *types.Thread
 	var msg *notmuch.Message
 	for msgs.Next(&msg) {
@@ -341,11 +422,12 @@ func (db *DB) makeThread(parent *types.Thread, msgs *notmuch.Messages,
 
 	// We want to return the root node
 	var root *types.Thread
-	if parent != nil {
+	switch {
+	case parent != nil:
 		root = parent
-	} else if lastSibling != nil {
+	case lastSibling != nil:
 		root = lastSibling // first iteration has no parent
-	} else {
+	default:
 		return nil // we don't have any messages at all
 	}
 
